@@ -15,7 +15,10 @@
   Software Foundation; either version 2 of the License, or (at your option)
   any later version.
 
-  Use the PortAudio library to play and record sound
+********************************************************************//**
+
+\class AudioIO
+\brief AudioIO uses the PortAudio library to play and record sound.
 
   Great care and attention to detail are necessary for understanding and
   modifying this system.  The code in this file is run from three
@@ -24,10 +27,19 @@
   To highlight this deliniation, the file is divided into three parts
   based on what thread context each function is intended to run in.
 
-  TODO: run through all functions called from audio and portaudio threads
+  \todo run through all functions called from audio and portaudio threads
   to verify they are thread-safe.
 
-**********************************************************************/
+*//****************************************************************//**
+
+\class AudioThread
+\brief Defined different on Mac and other platforms (on Mac it does not
+use wxWidgets wxThread), this class sits in a thread loop reading and 
+writing audio.
+
+*//*******************************************************************/
+
+#include "Audacity.h"
 
 #include <math.h>
 #include <stdlib.h>
@@ -41,8 +53,9 @@
 #include <wx/msgdlg.h>
 #include <wx/timer.h>
 #include <wx/intl.h>
+#include <wx/debug.h>
 
-
+#include "AudacityApp.h"
 #include "AudioIO.h"
 #include "WaveTrack.h"
 #include "Mix.h"
@@ -52,9 +65,7 @@
 
 #include "widgets/Meter.h"
 
-#if USE_PORTMIXER
-#include "MixerToolBar.h"
-#endif
+#define NO_STABLE_INDICATOR -1000000000
 
 AudioIO *gAudioIO;
 
@@ -145,10 +156,25 @@ void DeinitAudioIO()
    delete gAudioIO;
 }
 
+wxString DeviceName(const PaDeviceInfo* info)
+{
+#if USE_PORTAUDIO_V19
+   wxString hostapiName(Pa_GetHostApiInfo(info->hostApi)->name, wxConvLocal);
+   wxString infoName(info->name, wxConvLocal);
+
+   return wxString::Format(wxT("%s: %s"),
+                           hostapiName.c_str(),
+                           infoName.c_str());
+#else  
+   return wxString(info->name, wxConvLocal);
+#endif
+}
+
 AudioIO::AudioIO()
 {
    mAudioThreadShouldCallFillBuffersOnce = false;
    mAudioThreadFillBuffersLoopRunning = false;
+   mAudioThreadFillBuffersLoopActive = false;
 #if USE_PORTAUDIO_V19
    mPortStreamV19 = NULL;
 #else
@@ -161,10 +187,12 @@ AudioIO::AudioIO()
 
    mLastPaError = paNoError;
 
+   mLastRecordingOffset = 0.0;
    mNumCaptureChannels = 0;
    mPaused = false;
    mPlayLooped = false;
 
+   mListener = NULL;
    mUpdateMeters = false;
    mUpdatingMeters = false;
 
@@ -315,12 +343,27 @@ void AudioIO::HandleDeviceChange()
       return;
 
 #if defined(USE_PORTMIXER)
+   const PaDeviceInfo* info;
+
    if( mPortMixer )
       Px_CloseMixer(mPortMixer);
    mPortMixer = NULL;
 
+#if USE_PORTAUDIO_V19
+   int recDeviceNum = Pa_GetDefaultInputDevice();
+   int playDeviceNum = Pa_GetDefaultOutputDevice();
+#else
    int recDeviceNum = Pa_GetDefaultInputDeviceID();
    int playDeviceNum = Pa_GetDefaultOutputDeviceID();
+#endif
+
+   // Sometimes PortAudio returns -1 if it cannot find a suitable default
+   // device, so we just use the first one available
+   if (recDeviceNum < 0)
+      recDeviceNum = 0;
+   if (playDeviceNum < 0)
+      playDeviceNum = 0;
+      
    wxString recDevice = gPrefs->Read(wxT("/AudioIO/RecordingDevice"), wxT(""));
    wxString playDevice = gPrefs->Read(wxT("/AudioIO/PlaybackDevice"), wxT(""));
    int j;
@@ -329,13 +372,26 @@ void AudioIO::HandleDeviceChange()
    // available on this device, using 44.1kHz as the default, if the info
    // cannot be fetched.
 
-   for(j=0; j<Pa_CountDevices(); j++) {
-      const PaDeviceInfo* info = Pa_GetDeviceInfo(j);
+#if USE_PORTAUDIO_V19
+   if (Pa_GetDeviceCount() <= 0)
+      return; // no devices found!?
+      
+   for(j=0; j<Pa_GetDeviceCount(); j++) {
+#else
+   if (Pa_CountDevices() <= 0)
+      return; // no devices found!?
 
-      if (LAT1CTOWX(info->name) == playDevice && info->maxOutputChannels > 0)
+   for(j=0; j<Pa_CountDevices(); j++) {
+#endif
+
+      info = Pa_GetDeviceInfo(j);
+      if(!info)
+         continue;
+
+      if (DeviceName(info) == playDevice && info->maxOutputChannels > 0)
          playDeviceNum = j;
 
-      if (LAT1CTOWX(info->name) == recDevice && info->maxInputChannels > 0)
+      if (DeviceName(info) == recDevice && info->maxInputChannels > 0)
          recDeviceNum = j;
    }
    
@@ -346,8 +402,54 @@ void AudioIO::HandleDeviceChange()
    mMixerInputVol = 1.0;
    mMixerOutputVol = 1.0;
 
-   PortAudioStream *stream;
    int error;
+
+#if USE_PORTAUDIO_V19
+
+   PaStream *stream;
+
+   PaStreamParameters playbackParameters;
+
+   playbackParameters.device = playDeviceNum;
+   playbackParameters.sampleFormat = paFloat32;
+   playbackParameters.hostApiSpecificStreamInfo = NULL;
+   playbackParameters.channelCount = 2;
+   if (Pa_GetDeviceInfo(playDeviceNum))
+      playbackParameters.suggestedLatency =
+         Pa_GetDeviceInfo(playDeviceNum)->defaultLowOutputLatency;
+   else
+      playbackParameters.suggestedLatency = 100; // we're just probing anyway
+
+   PaStreamParameters captureParameters;
+ 
+   captureParameters.device = recDeviceNum;
+   captureParameters.sampleFormat = paFloat32;;
+   captureParameters.hostApiSpecificStreamInfo = NULL;
+   captureParameters.channelCount = 2;
+   if (Pa_GetDeviceInfo(recDeviceNum))
+      captureParameters.suggestedLatency =
+         Pa_GetDeviceInfo(recDeviceNum)->defaultLowOutputLatency;
+   else
+      captureParameters.suggestedLatency = 100; // we're just probing anyway
+ 
+   error = Pa_OpenStream(&stream,
+                         &captureParameters, &playbackParameters,
+                         highestSampleRate, paFramesPerBufferUnspecified,
+                         paClipOff | paDitherOff,
+                         audacityAudioCallback, NULL);
+
+   if( error ) {
+      error = Pa_OpenStream(&stream,
+                            &captureParameters, NULL,
+                            highestSampleRate, paFramesPerBufferUnspecified,
+                            paClipOff | paDitherOff,
+                            audacityAudioCallback, NULL);
+   }
+  
+#else
+
+   PortAudioStream *stream;
+
    error = Pa_OpenStream(&stream, recDeviceNum, 2, paFloat32, NULL,
                          playDeviceNum, 2, paFloat32, NULL,
                          highestSampleRate, 512, 1, paClipOff | paDitherOff,
@@ -360,6 +462,8 @@ void AudioIO::HandleDeviceChange()
                             audacityAudioCallback, NULL);
    }
 
+#endif
+
    if( error )
       return;
 
@@ -429,7 +533,13 @@ bool AudioIO::StartPortAudioStream(double sampleRate,
                                    sampleFormat captureFormat)
 {
    mLastPaError = paNoError;
-   mRate = sampleRate;
+   mRate = GetBestRate(sampleRate);
+   
+   // Special case: Our 24-bit sample format is different from PortAudio's
+   // 3-byte packed format. So just make PortAudio return float samples,
+   // since we need float values anyway to apply the gain.
+   if (captureFormat == int24Sample)
+      captureFormat = floatSample;
 
    mNumPlaybackChannels = numPlaybackChannels;
    mNumCaptureChannels = numCaptureChannels;
@@ -437,6 +547,9 @@ bool AudioIO::StartPortAudioStream(double sampleRate,
 #if USE_PORTAUDIO_V19
    PaStreamParameters *playbackParameters = NULL;
    PaStreamParameters *captureParameters = NULL;
+   
+   double latencyDuration = 100.0;
+   gPrefs->Read(wxT("/AudioIO/LatencyDuration"), &latencyDuration);
 
    if( numPlaybackChannels > 0)
    {
@@ -445,13 +558,19 @@ bool AudioIO::StartPortAudioStream(double sampleRate,
       const PaDeviceInfo *playbackDeviceInfo;
       
       playbackParameters->device = Pa_GetDefaultOutputDevice();
+      if (playbackParameters->device < 0)
+         playbackParameters->device = 0;
       
       if( playbackDeviceName != wxT("") )
       {
          for( int i = 0; i < Pa_GetDeviceCount(); i++)
          {
             const PaDeviceInfo* info = Pa_GetDeviceInfo(i);
-            if (LAT1CTOWX(info->name) == playbackDeviceName && info->maxOutputChannels > 0)
+
+            if(!info)
+               continue;
+
+            if (DeviceName(info) == playbackDeviceName && info->maxOutputChannels > 0)
                playbackParameters->device = i;
          }
       }
@@ -465,25 +584,36 @@ bool AudioIO::StartPortAudioStream(double sampleRate,
       playbackParameters->sampleFormat = paFloat32;
       playbackParameters->hostApiSpecificStreamInfo = NULL;
       playbackParameters->channelCount = mNumPlaybackChannels;
-      playbackParameters->suggestedLatency =
-         playbackDeviceInfo->defaultLowOutputLatency;
+
+      if (mSoftwarePlaythrough)
+         playbackParameters->suggestedLatency =
+            playbackDeviceInfo->defaultLowOutputLatency;
+      else
+         playbackParameters->suggestedLatency = latencyDuration/1000.0;
    }
 
    if( numCaptureChannels > 0)
    {
       mCaptureFormat = captureFormat;
+      
       captureParameters = new PaStreamParameters;
       const PaDeviceInfo *captureDeviceInfo;
       wxString captureDeviceName = gPrefs->Read(wxT("/AudioIO/RecordingDevice"), wxT(""));
 
       captureParameters->device = Pa_GetDefaultInputDevice();
+      if (captureParameters->device < 0)
+         captureParameters->device = 0;
 
       if( captureDeviceName != wxT("") )
       {
          for( int i = 0; i < Pa_GetDeviceCount(); i++)
          {
             const PaDeviceInfo* info = Pa_GetDeviceInfo(i);
-            if (LAT1CTOWX(info->name) == captureDeviceName && info->maxInputChannels > 0)
+
+            if(!info)
+               continue;
+
+            if (DeviceName(info) == captureDeviceName && info->maxInputChannels > 0)
                captureParameters->device = i;
          }
       }
@@ -498,8 +628,12 @@ bool AudioIO::StartPortAudioStream(double sampleRate,
 
       captureParameters->hostApiSpecificStreamInfo = NULL;
       captureParameters->channelCount = mNumCaptureChannels;
-      captureParameters->suggestedLatency =
-         captureDeviceInfo->defaultHighInputLatency;
+
+      if (mSoftwarePlaythrough)
+         captureParameters->suggestedLatency =
+            captureDeviceInfo->defaultHighInputLatency;
+      else
+         captureParameters->suggestedLatency = latencyDuration/1000.0;
    }
 
    mLastPaError = Pa_OpenStream( &mPortStreamV19,
@@ -508,14 +642,33 @@ bool AudioIO::StartPortAudioStream(double sampleRate,
                                  paNoFlag,
                                  audacityAudioCallback, NULL );
 
-#if 0 //USE_PORTMIXER  TODO: support PortMixer with v19
-   if (mPortMixer)
+#if USE_PORTMIXER
+   if (mPortMixer) {
+      #if __WXMAC__
+      if (Px_SupportsPlaythrough(mPortMixer) && mPreviousHWPlaythrough >= 0.0)
+         Px_SetPlaythrough(mPortMixer, mPreviousHWPlaythrough);
+      #endif
       Px_CloseMixer(mPortMixer);
+   }
    mPortMixer = NULL;
-   if (mPortStream != NULL && mLastPaError == paNoError) {
-      mPortMixer = Px_OpenMixer(mPortStream, 0);
-      if (mPortMixer)
-         AdjustMixer();
+   if (mPortStreamV19 != NULL && mLastPaError == paNoError) {
+      mPortMixer = Px_OpenMixer(mPortStreamV19, 0);
+
+      #ifdef __MACOSX__
+      if (mPortMixer) {
+         if (Px_SupportsPlaythrough(mPortMixer)) {
+            bool playthrough;
+
+            mPreviousHWPlaythrough = Px_GetPlaythrough(mPortMixer);
+
+            gPrefs->Read(wxT("/AudioIO/Playthrough"), &playthrough, false);
+            if (playthrough)
+               Px_SetPlaythrough(mPortMixer, 1.0);
+            else
+               Px_SetPlaythrough(mPortMixer, 0.0);
+         }
+      }
+      #endif
    }
 #endif
 
@@ -539,7 +692,11 @@ bool AudioIO::StartPortAudioStream(double sampleRate,
          for( int i = 0; i < Pa_CountDevices(); i++)
          {
             const PaDeviceInfo* info = Pa_GetDeviceInfo(i);
-            if (LAT1CTOWX(info->name) == playbackDeviceName && info->maxOutputChannels > 0)
+
+            if(!info)
+               continue;
+
+            if (DeviceName(info) == playbackDeviceName && info->maxOutputChannels > 0)
                playbackDevice = i;
          }
       }
@@ -557,7 +714,11 @@ bool AudioIO::StartPortAudioStream(double sampleRate,
          for( int i = 0; i < Pa_CountDevices(); i++)
          {
             const PaDeviceInfo* info = Pa_GetDeviceInfo(i);
-            if (LAT1CTOWX(info->name) == captureDeviceName && info->maxInputChannels > 0)
+
+            if(!info)
+               continue;
+
+            if (DeviceName(info) == captureDeviceName && info->maxInputChannels > 0)
                captureDevice = i;
          }
       }
@@ -658,7 +819,11 @@ void AudioIO::StartMonitoring(double sampleRate)
 int AudioIO::StartStream(WaveTrackArray playbackTracks,
                          WaveTrackArray captureTracks,
                          TimeTrack *timeTrack, double sampleRate,
-                         double t0, double t1, bool playLooped /* = false */)
+                         double t0, double t1,
+                         AudioIOListener* listener,
+                         bool playLooped /* = false */,
+                         double cutPreviewGapStart /* = 0.0 */,
+                         double cutPreviewGapLen /* = 0.0 */)
 {
    if( IsBusy() )
       return 0;
@@ -694,20 +859,22 @@ int AudioIO::StartStream(WaveTrackArray playbackTracks,
 
    gPrefs->Read(wxT("/AudioIO/SWPlaythrough"), &mSoftwarePlaythrough, false);
 
+   mListener = listener;
    mInputMeter = NULL;
    mOutputMeter = NULL;
    mRate    = sampleRate;
    mT0      = t0;
    mT       = t0;
    mT1      = t1;
+   mTime    = t0;
+   mSeek    = 0;
+   mLastRecordingOffset = 0;
+   mPlaySpeed = 100.0 / (timeTrack ? timeTrack->GetRangeUpper() : 100);
    mPlaybackTracks = playbackTracks;
    mCaptureTracks  = captureTracks;
-   mTotalSamplesPlayed = 0;
    mPlayLooped = playLooped;
-
-#ifndef USE_PORTAUDIO_V19
-   mPausedSeconds = 0;
-#endif
+   mCutPreviewGapStart = cutPreviewGapStart;
+   mCutPreviewGapLen = cutPreviewGapLen;
 
    //
    // The RingBuffer sizes, and the max amount of the buffer to
@@ -745,12 +912,18 @@ int AudioIO::StartStream(WaveTrackArray playbackTracks,
       // track wouldn't get the benefit of all 24 bits the card is capable
       // of.
       captureFormat = mCaptureTracks[0]->GetSampleFormat();
-   }   
-
+      
+      // Tell project that we are about to start recording
+      if (mListener)
+         mListener->OnAudioIOStartRecording();
+   }
+   
    bool success = StartPortAudioStream(sampleRate, playbackChannels,
                                        captureChannels, captureFormat);
    
    if (!success) {
+      if (mListener && captureChannels > 0)
+         mListener->OnAudioIOStopRecording();
       mStreamToken = 0;
       return 0;
    }
@@ -818,6 +991,8 @@ int AudioIO::StartStream(WaveTrackArray playbackTracks,
       // TODO
       // we'll need a more complete way to indicate error.
       // AND we need to delete the ring buffers and mixers, etc.
+      if (mListener && mNumCaptureChannels > 0)
+         mListener->OnAudioIOStopRecording();
       wxPrintf(wxT("%hs\n"), Pa_GetErrorText(err));
       mStreamToken = 0;
       return 0;
@@ -832,13 +1007,6 @@ int AudioIO::StartStream(WaveTrackArray playbackTracks,
    //
    mStreamToken = (++mNextStreamToken);
 
-#if USE_PORTAUDIO_V19
-   // To make GetStreamTime behave correctly before the callback sets
-   // mLastBufferAudibleTime the first time, we make a rough guess
-   // TODO: guess better
-   mLastBufferAudibleTime = Pa_GetStreamTime( mPortStreamV19 ) + (2048 / mRate);
-#endif
-
    return mStreamToken;
 }
 
@@ -957,7 +1125,8 @@ void AudioIO::StopStream()
 
       while( mAudioThreadShouldCallFillBuffersOnce == true )
       {
-         wxYield();
+         // LLL:  Experienced recursive yield here...once.
+         wxGetApp().Yield( true );
          wxMilliSleep( 50 );
       }
 
@@ -978,13 +1147,20 @@ void AudioIO::StopStream()
          delete[] mPlaybackMixers;
       }
 
+      //
+      // Offset all recorded tracks to account for latency
+      //
+      double latencyCorrection = 0;
+      gPrefs->Read(wxT("/AudioIO/LatencyCorrection"), &latencyCorrection);
+      
       if( mCaptureTracks.GetCount() > 0 )
       {
          for( unsigned int i = 0; i < mCaptureTracks.GetCount(); i++ )
             {
                delete mCaptureBuffers[i];
                mCaptureTracks[i]->Flush();
-               mCaptureTracks[i]->Offset(mLastRecordingOffset);
+               mCaptureTracks[i]->Offset(mLastRecordingOffset +
+                                         latencyCorrection/1000.0);
             }
          
          delete[] mCaptureBuffers;
@@ -996,6 +1172,9 @@ void AudioIO::StopStream()
    if (mOutputMeter)
       mOutputMeter->Reset(mRate, false);
 
+   if (mListener && mNumCaptureChannels > 0)
+      mListener->OnAudioIOStopRecording();
+      
    //
    // Only set token to 0 after we're totally finished with everything
    //
@@ -1005,23 +1184,6 @@ void AudioIO::StopStream()
 
 void AudioIO::SetPaused(bool state)
 {
-   if(state)
-   {
-      // When we are beginning a pause, we note the time so that GetStreamTime
-      // can always return this position while we're paused
-      mPausePosition = GetStreamTime();
-   }
-   else
-   {
-#if USE_PORTAUDIO_V19
-      // When we're coming out of a pause, we guess mLastBufferAudibleTime so
-      // that the indicator won't be erratic between now and when the callback
-      // sets mLastBufferAudibleTime again.  This is similar to what happens
-      // in StartStream
-      mLastBufferAudibleTime = Pa_GetStreamTime( mPortStreamV19 ) + (2048 / mRate);
-#endif
-   }
-
    mPaused = state;
 }
 
@@ -1056,7 +1218,7 @@ bool AudioIO::IsStreamActive()
 {
 #if USE_PORTAUDIO_V19
    if( mPortStreamV19 )
-      return Pa_IsStreamActive( mPortStreamV19 );
+      return Pa_IsStreamActive( mPortStreamV19 ) != 0;
    else
       return false;
 #else
@@ -1100,6 +1262,10 @@ double AudioIO::NormalizeStreamTime(double absoluteTime) const
    // the current stream time, and this query is not always accurate.
    // Sometimes it's a little behind or ahead, and so this function
    // makes sure that at least we clip it to the selection.
+   //
+   // msmeyer: There is also the possibility that we are using "cut preview"
+   //          mode. In this case, we should jump over a defined "gap" in the
+   //          audio.
 
    // msmeyer: Just to be sure, the returned stream time should
    //          never be smaller than the actual start time.
@@ -1117,8 +1283,16 @@ double AudioIO::NormalizeStreamTime(double absoluteTime) const
       if (absoluteTime > mT1)
          absoluteTime = mT1;
    }
-      
-   return absoluteTime;
+   
+   if (mCutPreviewGapLen > 0)
+   {
+      // msmeyer: We're in cut preview mode, so if we are on the right
+      // side of the gap, we jump over it.
+      if (absoluteTime > mCutPreviewGapStart)
+         absoluteTime += mCutPreviewGapLen;
+   }
+
+   return absoluteTime / mPlaySpeed;
 }
 
 double AudioIO::GetStreamTime()
@@ -1126,43 +1300,7 @@ double AudioIO::GetStreamTime()
    if( !IsStreamActive() )
       return -1000000000;
 
-   if( mPaused )
-      return NormalizeStreamTime(mPausePosition);
-
-#if USE_PORTAUDIO_V19
-
-   PaStream *stream = mPortStreamV19;
-
-   // Based on the number of samples we have written, this is the value
-   // in time that represents how far we are through the source data
-   double lastBufferTime =  mT0 + (mTotalSamplesPlayed / mRate);
-
-   // This is the number of seconds ago that the last buffer started
-   // being heard.  It will be negative if the last buffer is not
-   // yet audible.
-   double deltat = Pa_GetStreamTime(stream) - mLastBufferAudibleTime;
-
-   // [JH]: I need to diagram this, but I'm pretty sure this calculation
-   // is off by one buffer size
-   double time = lastBufferTime + deltat;
-
-   return NormalizeStreamTime(time);
-#else
-   PaStream *stream = mPortStreamV18;
-
-   double streamTime = Pa_StreamTime(stream);
-   double indicator = mT0 + (streamTime / mRate) - mPausedSeconds;
-
-   // Pa_StreamTime can sometimes return wacky results, so we
-   // try to filter those out...
-   if (fabs(indicator - mLastIndicator) > 0.1) {
-      mLastIndicator = indicator;
-      return NormalizeStreamTime(mLastStableIndicator);
-   }
-   mLastIndicator = indicator;
-   mLastStableIndicator = indicator;
-   return NormalizeStreamTime(indicator);
-#endif
+   return NormalizeStreamTime(mTime);
 }
 
 
@@ -1181,6 +1319,14 @@ wxArrayLong AudioIO::GetSupportedSampleRates(wxString playDevice, wxString recDe
 
    const PaDeviceInfo* playInfo = NULL;
    const PaDeviceInfo* recInfo = NULL;
+   int playIndex = -1, recIndex = -1;
+
+#if USE_PORTAUDIO_V19
+   double latencyDuration = 100.0;
+   long recordChannels = 1;
+   gPrefs->Read(wxT("/AudioIO/LatencyDuration"), &latencyDuration);
+   gPrefs->Read(wxT("/AudioIO/RecordChannels"), &recordChannels);
+#endif
 
    if (playDevice.IsEmpty())
       playDevice = gPrefs->Read(wxT("/AudioIO/PlaybackDevice"), wxT(""));
@@ -1197,10 +1343,19 @@ wxArrayLong AudioIO::GetSupportedSampleRates(wxString playDevice, wxString recDe
 #endif
       const PaDeviceInfo* info = Pa_GetDeviceInfo(i);
 
-      if (LAT1CTOWX(info->name) == playDevice && info->maxOutputChannels > 0)
+      if(!info)
+         continue;
+
+      if (DeviceName(info) == playDevice && info->maxOutputChannels > 0)
+      {
          playInfo = info;
-      if (LAT1CTOWX(info->name) == recDevice && info->maxInputChannels > 0)
+         playIndex = i;
+      }
+      if (DeviceName(info) == recDevice && info->maxInputChannels > 0)
+      {
          recInfo = info;
+         recIndex = i;
+      }
    }
 
    // msmeyer: Check which sample rates the play device supports
@@ -1209,9 +1364,19 @@ wxArrayLong AudioIO::GetSupportedSampleRates(wxString playDevice, wxString recDe
    if (playInfo)
    {
 #if USE_PORTAUDIO_V19
-      // TODO: implement using IsFormatSupported()
+      PaStreamParameters pars;
+
+      pars.device = playIndex;
+      pars.channelCount = 2;
+      pars.sampleFormat = paFloat32;
+      pars.suggestedLatency = latencyDuration;
+      pars.hostApiSpecificStreamInfo = NULL;
+      
       for (i = 0; i < numDefaultRates; i++)
-         playSampleRates.Add(defaultRates[i]);
+      {
+         if (Pa_IsFormatSupported(NULL, &pars, defaultRates[i]) == 0)
+            playSampleRates.Add(defaultRates[i]);
+      }
 #else
       if (playInfo->numSampleRates == -1)
       {
@@ -1239,9 +1404,19 @@ wxArrayLong AudioIO::GetSupportedSampleRates(wxString playDevice, wxString recDe
    if (recInfo)
    {
 #if USE_PORTAUDIO_V19
-      // TODO: implement using IsFormatSupported()
+      PaStreamParameters pars;
+
+      pars.device = recIndex;
+      pars.channelCount = recordChannels;
+      pars.sampleFormat = paFloat32;
+      pars.suggestedLatency = latencyDuration;
+      pars.hostApiSpecificStreamInfo = NULL;
+      
       for (i = 0; i < numDefaultRates; i++)
-         recSampleRates.Add(defaultRates[i]);
+      {
+         if (Pa_IsFormatSupported(&pars, NULL, defaultRates[i]) == 0)
+            recSampleRates.Add(defaultRates[i]);
+      }
 #else
       if (recInfo->numSampleRates == -1)
       {
@@ -1294,6 +1469,17 @@ int AudioIO::GetOptimalSupportedSampleRate()
    return rates[rates.GetCount() - 1];
 }
 
+long AudioIO::GetBestRate(double sampleRate)
+{
+   wxArrayLong rates = GetSupportedSampleRates();
+   long rate = (long)sampleRate;
+   
+   if (rates.Index(rate) != wxNOT_FOUND) {
+      return rate;
+   }
+
+   return rates[rates.GetCount() - 1];
+}      
 
 
 //////////////////////////////////////////////////////////////////////
@@ -1360,6 +1546,8 @@ void AudioIO::FillBuffers()
 {
    unsigned int i;
 
+   gAudioIO->mAudioThreadFillBuffersLoopActive = true;
+   
    if( mPlaybackTracks.GetCount() > 0 )
    {
       // Though extremely unlikely, it is possible that some buffers
@@ -1398,7 +1586,6 @@ void AudioIO::FillBuffers()
          // This is the purpose of this loop.
          do {
             deltat = secsAvail;
-
             if( mT + deltat > mT1 )
             {
                deltat = mT1 - mT;
@@ -1406,6 +1593,7 @@ void AudioIO::FillBuffers()
                   deltat = 0.0;
             }
             mT += deltat;
+            
             secsAvail -= deltat;
 
             for( i = 0; i < mPlaybackTracks.GetCount(); i++ )
@@ -1427,6 +1615,7 @@ void AudioIO::FillBuffers()
                   mPlaybackMixers[i]->Restart();
                mT = mT0;
             }
+
          } while (mPlayLooped && secsAvail > 0 && deltat > 0);
       }
    }
@@ -1445,19 +1634,38 @@ void AudioIO::FillBuffers()
       {
          // Append captured samples to the end of the WaveTracks.
          // The WaveTracks have their own buffering for efficiency.
-         for( i = 0; i < mCaptureTracks.GetCount(); i++ )
+         XMLStringWriter blockFileLog;
+         int numChannels = mCaptureTracks.GetCount();
+         
+         for( i = 0; (int)i < numChannels; i++ )
          {
             int avail = commonlyAvail;
             sampleFormat trackFormat = mCaptureTracks[i]->GetSampleFormat();
+
+            XMLStringWriter appendLog;
+            
             samplePtr temp = NewSamples(avail, trackFormat);
-
             mCaptureBuffers[i]->Get   (temp, trackFormat, avail);
-            mCaptureTracks[i]-> Append(temp, trackFormat, avail);
-
+            mCaptureTracks[i]-> Append(temp, trackFormat, avail, 1, 
+                                       &appendLog);
             DeleteSamples(temp);
+                                       
+            if (!appendLog.IsEmpty())
+            {
+               blockFileLog.StartTag(wxT("recordingrecovery"));
+               blockFileLog.WriteAttr(wxT("channel"), (int)i);
+               blockFileLog.WriteAttr(wxT("numchannels"), numChannels);
+               blockFileLog.WriteSubTree(appendLog);
+               blockFileLog.EndTag(wxT("recordingrecovery"));
+            }
          }
+         
+         if (mListener && !blockFileLog.IsEmpty())
+            mListener->OnAudioIONewBlockFiles(blockFileLog);
       }
    }
+
+   gAudioIO->mAudioThreadFillBuffersLoopActive = false;
 }
 
 //////////////////////////////////////////////////////////////////////
@@ -1538,15 +1746,11 @@ int audacityAudioCallback(void *inputBuffer, void *outputBuffer,
          }
       }
 
-#ifndef USE_PORTAUDIO_V19
-      gAudioIO->mPausedSeconds += (float)framesPerBuffer / gAudioIO->mRate;
-#endif
       return paContinue;
    }
 
    if (gAudioIO->mStreamToken > 0)
    {
-
       //
       // Mix and copy to PortAudio's output buffer
       //
@@ -1570,6 +1774,43 @@ int audacityAudioCallback(void *inputBuffer, void *outputBuffer,
                                   (float *)outputBuffer, (int)framesPerBuffer, gain);
          }
          
+         if (gAudioIO->mSeek)
+         {
+            // Pause audio thread and wait for it to finish
+            gAudioIO->mAudioThreadFillBuffersLoopRunning = false;
+            while( gAudioIO->mAudioThreadFillBuffersLoopActive == true )
+            {
+               wxMilliSleep( 50 );
+            }
+
+            // Calculate the new time position
+            gAudioIO->mTime += gAudioIO->mSeek;
+            if (gAudioIO->mTime < gAudioIO->mT0)
+                gAudioIO->mTime = gAudioIO->mT0;
+            else if (gAudioIO->mTime > gAudioIO->mT1)
+                gAudioIO->mTime = gAudioIO->mT1;
+            gAudioIO->mSeek = 0.0;
+            
+            // Reset mixer positions and flush buffers for all tracks
+            for (i = 0; i < (unsigned int)numPlaybackTracks; i++)
+            {
+               gAudioIO->mPlaybackMixers[i]->Reposition( gAudioIO->mTime );
+               gAudioIO->mPlaybackBuffers[i]->Discard( gAudioIO->mPlaybackBuffers[i]->AvailForGet() );
+            }
+
+            // Reload the ring buffers
+            gAudioIO->mAudioThreadShouldCallFillBuffersOnce = true;
+            while( gAudioIO->mAudioThreadShouldCallFillBuffersOnce == true )
+            {
+               wxMilliSleep( 50 );
+            }
+
+            // Reenable the audio thread
+            gAudioIO->mAudioThreadFillBuffersLoopRunning = true;
+               
+            return paContinue;
+         }
+
          int numSolo = 0;
          for( t = 0; t < numPlaybackTracks; t++ )
             if( gAudioIO->mPlaybackTracks[t]->GetSolo() )
@@ -1600,7 +1841,7 @@ int audacityAudioCallback(void *inputBuffer, void *outputBuffer,
                   gAudioIO->mPlaybackBuffers[t]->Discard(framesPerBuffer);
                   continue;
                }
-            
+
             unsigned int len = (unsigned int)
                gAudioIO->mPlaybackBuffers[t]->Get((samplePtr)tempFloats,
                                                   floatSample,
@@ -1700,18 +1941,13 @@ int audacityAudioCallback(void *inputBuffer, void *outputBuffer,
                      tempFloats[i] =
                         inputFloats[numCaptureChannels*i+t] * gain;
                } break;
-               case int24Sample: {
-                  int *inputInts = (int *)inputBuffer;
-                  int *tempInts = (int *)tempBuffer;
-                  for( i = 0; i < len; i++) {
-                     float tmp = inputInts[numCaptureChannels*i+t] * gain;
-                     if (tmp > 8388607)
-                        tmp = 8388607;
-                     if (tmp < -8388608)
-                        tmp = -8388608;
-                     tempInts[i] = (int)(tmp);
-                  }
-               } break;
+               case int24Sample:
+                  // We should never get here. Audacity's int24Sample format
+                  // is different from PortAudio's sample format and so we
+                  // make PortAudio return float samples when recording in
+                  // 24-bit samples.
+                  wxASSERT(false);
+                  break;
                case int16Sample: {
                   short *inputShorts = (short *)inputBuffer;
                   short *tempShorts = (short *)tempBuffer;
@@ -1733,21 +1969,28 @@ int audacityAudioCallback(void *inputBuffer, void *outputBuffer,
          }
       }
       
-     #if USE_PORTAUDIO_V19
-      gAudioIO->mTotalSamplesPlayed += framesPerBuffer;
-      if (numPlaybackChannels > 0)
-         gAudioIO->mLastBufferAudibleTime = timeInfo->outputBufferDacTime;
-      else
-         gAudioIO->mLastBufferAudibleTime = timeInfo->inputBufferAdcTime;
-     #endif
-
+      // Update the current time position
+      gAudioIO->mTime += (framesPerBuffer / gAudioIO->mRate);
+      
       // Record the reported latency from PortAudio.
       // TODO: Don't recalculate this with every callback?
      #if USE_PORTAUDIO_V19
-      if (numCaptureChannels > 0 && numPlaybackChannels > 0)
+      // As of 06/17/2006, portaudio v19 returns inputBufferAdcTime set to
+      // zero.  It is being worked on, but for now we just can't do much
+      // but follow the leader.
+      //
+      // 08/27/2006: too inconsistent for now...just leave it a zero.
+      //
+      /*
+      if (numCaptureChannels > 0 && numPlaybackChannels > 0 && timeInfo->inputBufferAdcTime > 0)
          gAudioIO->mLastRecordingOffset = timeInfo->inputBufferAdcTime - timeInfo->outputBufferDacTime;
-      else
-         gAudioIO->mLastRecordingOffset = 0;
+      else {
+         if (gAudioIO->mLastRecordingOffset == 0.0) {
+            const PaStreamInfo* si = Pa_GetStreamInfo( gAudioIO->mPortStreamV19 );
+            gAudioIO->mLastRecordingOffset = -si->inputLatency;
+         }
+      }
+      */
      #else
       if (numCaptureChannels > 0 && numPlaybackChannels > 0)
          gAudioIO->mLastRecordingOffset = (Pa_StreamTime(gAudioIO->mPortStreamV18) - outTime) / gAudioIO->mRate;
