@@ -7,7 +7,40 @@
   Joshua Haberman
   Dominic Mazzoni
 
-**********************************************************************/
+*******************************************************************//*!
+
+\class BlockFile
+\brief A BlockFile is a chunk of immutable audio data.
+
+A BlockFile represents a chunk of audio data.  These chunks are
+assembled into sequences by the class Sequence.  These classes
+are at the heart of how Audacity stores audio data.
+
+BlockFile is an abstract base class that can be implemented in
+many different ways.  However it does have a fairly large amount
+of shared code that deals with the physical file and manipulating
+the summary data.
+
+BlockFile should be thought of as an immutable class.  After it
+is constructed, it is essentially never changed (though there are
+a few exceptions).  Most notably, the audio data and summary data
+are never altered once it is constructed.  This is important to
+some of the derived classes that are actually aliases to audio
+data stored in existing files.
+
+BlockFiles are reference-counted, and deleted when their reference
+count hits zero.  DirManager is the class responsible for
+constructing and managing BlockFiles and managing their reference
+counts.
+
+*//****************************************************************//**
+
+\class SummaryInfo
+\brief Works with BlockFile to hold info about max and min and RMS 
+over multiple samples, which in turn allows rapid drawing when zoomed 
+out.
+
+*//*******************************************************************/
 
 #include <float.h>
 #include <math.h>
@@ -20,6 +53,17 @@
 #include "BlockFile.h"
 #include "Internat.h"
 
+// msmeyer: Define this to add debug output via printf()
+//#define DEBUG_BLOCKFILE
+
+#ifdef DEBUG_BLOCKFILE
+#define BLOCKFILE_DEBUG_OUTPUT(op, i) \
+   wxPrintf(wxT("[BlockFile %x %s] %s: %i\n"), (unsigned)this, \
+            mFileName.GetFullName().c_str(), wxT(op), i);
+#else
+#define BLOCKFILE_DEBUG_OUTPUT(op, i)
+#endif
+
 const int headerTagLen = 20;
 char headerTag[headerTagLen + 1] = "AudacityBlockFile112";
 
@@ -62,7 +106,7 @@ BlockFile::BlockFile(wxFileName fileName, sampleCount samples):
 BlockFile::~BlockFile()
 {
    if (!IsLocked() && mFileName.HasName())
-      wxRemoveFile(FILENAME(mFileName.GetFullPath()));
+      wxRemoveFile(mFileName.GetFullPath());
 }
 
 /// Returns the file name of the disk file associated with this
@@ -85,12 +129,14 @@ wxFileName BlockFile::GetFileName()
 void BlockFile::Lock()
 {
    mLockCount++;
+   BLOCKFILE_DEBUG_OUTPUT("Lock", mLockCount);
 }
 
 /// Marks this BlockFile as "unlocked."
 void BlockFile::Unlock()
 {
    mLockCount--;
+   BLOCKFILE_DEBUG_OUTPUT("Unlock", mLockCount);
 }
 
 /// Returns true if the block is locked.
@@ -104,6 +150,7 @@ bool BlockFile::IsLocked()
 void BlockFile::Ref()
 {
    mRefCount++;
+   BLOCKFILE_DEBUG_OUTPUT("Ref", mRefCount);
 }
 
 /// Decreases the reference count of this block by one.  If this
@@ -112,6 +159,7 @@ void BlockFile::Ref()
 bool BlockFile::Deref()
 {
    mRefCount--;
+   BLOCKFILE_DEBUG_OUTPUT("Deref", mRefCount);
    if (mRefCount <= 0) {
       delete this;
       return true;
@@ -495,7 +543,7 @@ void AliasBlockFile::WriteSummary()
    // I would much rather have this code as part of the constructor, but
    // I can't call virtual functions from the constructor.  So we just
    // need to ensure that every derived class calls this in *its* constructor
-   wxFFile summaryFile(FILENAME(mFileName.GetFullPath()).c_str(), wxT("wb"));
+   wxFFile summaryFile(mFileName.GetFullPath(), wxT("wb"));
 
    if( !summaryFile.IsOpened() ){
       // Never silence the Log w.r.t write errors; they always count
@@ -529,7 +577,7 @@ AliasBlockFile::~AliasBlockFile()
 ///              be at least mSummaryInfo.totalSummaryBytes long.
 bool AliasBlockFile::ReadSummary(void *data)
 {
-   wxFFile summaryFile(FILENAME(mFileName.GetFullPath()).c_str(), wxT("rb"));
+   wxFFile summaryFile(mFileName.GetFullPath(), wxT("rb"));
    wxLogNull *silence=0;
    if(mSilentLog)silence= new wxLogNull();
 
@@ -574,7 +622,7 @@ void AliasBlockFile::ChangeAliasedFile(wxFileName newAliasedFile)
 
 int AliasBlockFile::GetSpaceUsage()
 {
-   wxFFile summaryFile(FILENAME(mFileName.GetFullPath()));
+   wxFFile summaryFile(mFileName.GetFullPath());
    return summaryFile.Length();
 }
 
