@@ -8,7 +8,23 @@
   Dr William Bland (integration - the Calculus kind)
   Monty (xiphmont) (important bug fixes)
 
-**********************************************************************/
+*******************************************************************//**
+
+\class Envelope
+\brief Draggable curve used in TrackPanel for varying amplification.
+
+  This class manages an envelope - i.e. a piecewise linear funtion
+  that the user can edit by dragging control points around.  The
+  envelope is most commonly used to control the amplitude of a
+  waveform, but it is also used to shape a general FFT filter.
+
+*//****************************************************************//**
+
+\class EnvPoint
+\brief EnvPoint, derived from XMLTagHandler, provides Envelope with 
+a draggable point type.
+
+*//*******************************************************************/
 
 #include "Envelope.h"
 
@@ -49,8 +65,7 @@ Envelope::Envelope()
 
    mMirror = true;
 
-   mPen.SetColour(110, 110, 220);
-   mBrush.SetColour(110, 110, 220);
+   mButton = wxMOUSE_BTN_NONE;
 }
 
 Envelope::~Envelope()
@@ -165,14 +180,14 @@ void Envelope::Draw(wxDC & dc, wxRect & r, double h, double pps, bool dB,
    double tright = h + (r.width / pps);
    double dBr = gPrefs->Read(wxT("/GUI/EnvdBRange"), ENV_DB_RANGE);
 
-   dc.SetPen(mPen);
+   dc.SetPen(AColor::envelopePen);
    dc.SetBrush(*wxWHITE_BRUSH);
 
    for (int i = 0; i < (int)mEnv.Count(); i++) {
       if (mEnv[i]->t >= h && mEnv[i]->t <= tright) {
          if (i == mDragPoint) {
-            dc.SetPen(mPen);
-            dc.SetBrush(mBrush);
+            dc.SetPen(AColor::envelopePen);
+            dc.SetBrush(AColor::envelopeBrush);
          }
 
          double v = mEnv[i]->val;
@@ -200,7 +215,7 @@ void Envelope::Draw(wxDC & dc, wxRect & r, double h, double pps, bool dB,
          }
 
          if (i == mDragPoint) {
-            dc.SetPen(mPen);
+            dc.SetPen(AColor::envelopePen);
             dc.SetBrush(*wxWHITE_BRUSH);
          }
       }
@@ -238,26 +253,21 @@ XMLTagHandler *Envelope::HandleXMLChild(const wxChar *tag)
       return NULL;
 }
 
-void Envelope::WriteXML(int depth, FILE *fp)
+void Envelope::WriteXML(XMLWriter &xmlFile)
 {
    unsigned int ctrlPt;
-   int i;
 
-   for (i = 0; i < depth; i++)
-      fprintf(fp, "\t");
-   fprintf(fp, "<envelope numpoints='%ld'>\n", (long)mEnv.GetCount());
+   xmlFile.StartTag(wxT("envelope"));
+   xmlFile.WriteAttr(wxT("numpoints"), mEnv.GetCount());
 
    for (ctrlPt = 0; ctrlPt < mEnv.GetCount(); ctrlPt++) {
-      for(i = 0; i < depth+1; i++)
-         fprintf(fp, "\t");
-      fprintf(fp, "<controlpoint t='%s' val='%s'/>\n",
-            (const char *)Internat::ToString(mEnv[ctrlPt]->t, 12).mb_str(),
-            (const char *)Internat::ToString(mEnv[ctrlPt]->val, 12).mb_str());
+      xmlFile.StartTag(wxT("controlpoint"));
+      xmlFile.WriteAttr(wxT("t"), mEnv[ctrlPt]->t, 12);
+      xmlFile.WriteAttr(wxT("val"), mEnv[ctrlPt]->val, 12);
+      xmlFile.EndTag(wxT("controlpoint"));
    }
 
-   for (i = 0; i < depth; i++)
-      fprintf(fp, "\t");
-   fprintf(fp, "</envelope>\n");
+   xmlFile.EndTag(wxT("envelope"));
 }
 
 #ifndef SQR
@@ -266,7 +276,7 @@ void Envelope::WriteXML(int depth, FILE *fp)
 
 
 float Envelope::ValueOfPixel( int y, int height, bool upper, bool dB,
-                              float zoomMin, float zoomMax ){
+                              float zoomMin, float zoomMax, float eMin ){
    float v;
 
    v = zoomMax - (y/(float)height) * (zoomMax - zoomMin);
@@ -280,8 +290,8 @@ float Envelope::ValueOfPixel( int y, int height, bool upper, bool dB,
    if (dB)
       v = fromDB(v);
 
-   if ((upper && v < 0) || (!upper && v > 0))
-      v = 0;
+   if ((upper && v < eMin) || (!upper && v > 0))
+      v = eMin;
    else
    if (!upper)
       v = -v;
@@ -291,14 +301,17 @@ float Envelope::ValueOfPixel( int y, int height, bool upper, bool dB,
 
 bool Envelope::HandleMouseButtonDown(wxMouseEvent & event, wxRect & r,
                                      double h, double pps, bool dB,
-                                     float zoomMin, float zoomMax)
+                                     float zoomMin, float zoomMax, float eMin, float eMax)
 {
    bool upper;
    int ctr = (int)(r.height * zoomMax / (zoomMax - zoomMin));
    upper = (event.m_y - r.y < ctr);
-   
+   if(zoomMin == eMin)
+	   upper = true;
+
+   mButton = event.GetButton();
    int clip_y = event.m_y - r.y;
-   if(clip_y < 0) clip_y = 0;
+   if(clip_y < 0) clip_y = 0;	//keeps point in rect r, even if mouse isn't
    if(clip_y > r.height) clip_y = r.height;
    
    mIsDeleting = false;
@@ -314,7 +327,7 @@ bool Envelope::HandleMouseButtonDown(wxMouseEvent & event, wxRect & r,
    //   wxLogDebug(wxT("Y:%i Height:%i Offset:%i"), y, height, mContourOffset );
    
    int len = mEnv.Count();
-   for (int i = 0; i < len; i++) {
+   for (int i = 0; i < len; i++) {	//search for control point nearest click
       if (mEnv[i]->t >= tleft && mEnv[i]->t <= tright) {
          
          int x = int ((mEnv[i]->t + mOffset - h) * pps) + r.x;
@@ -387,13 +400,12 @@ bool Envelope::HandleMouseButtonDown(wxMouseEvent & event, wxRect & r,
       }
       
       double newVal = ValueOfPixel(clip_y, r.height, upper, dB,
-                                   zoomMin, zoomMax);
+                                   zoomMin, zoomMax, eMin);
       
       //float MaxAmplify = ( mContourOffset ) ? 1.4 : 1.0;
-      float MaxAmplify = 2.0;
-      
-      if (newVal > MaxAmplify)
-         newVal = MaxAmplify;
+
+	  if(newVal > eMax)
+		  newVal = eMax;
       
       mDragPoint = Insert(when, newVal);
       mDirty = true;
@@ -412,7 +424,7 @@ bool Envelope::HandleMouseButtonDown(wxMouseEvent & event, wxRect & r,
 
 bool Envelope::HandleDragging( wxMouseEvent & event, wxRect & r,
                                double h, double pps, bool dB,
-                               float zoomMin, float zoomMax )
+                               float zoomMin, float zoomMax, float eMin, float eMax)
 {
    int clip_y = event.m_y - r.y;
    if(clip_y < 0) clip_y = 0;
@@ -457,13 +469,12 @@ bool Envelope::HandleDragging( wxMouseEvent & event, wxRect & r,
 
 
    double newVal = ValueOfPixel(clip_y, r.height, mUpper, dB,
-                                zoomMin, zoomMax);   
+                                zoomMin, zoomMax, eMin);   
 
    //float MaxAmplify = ( mContourOffset ) ? 1.4 : 1.0;
-   float MaxAmplify = 2.0;
 
-   if (newVal > MaxAmplify)
-      newVal = MaxAmplify;
+   if(newVal > eMax)
+	   newVal = eMax;
 
    // We no longer tolerate multiple envelope points at the same t.
    // epsilon is less than the time offset of a single sample
@@ -498,25 +509,31 @@ bool Envelope::HandleMouseButtonUp( wxMouseEvent & event, wxRect & r,
       mEnv.RemoveAt(mDragPoint);
    }
    mDragPoint = -1;
+   mButton = wxMOUSE_BTN_NONE;
    return true;
 }
 
+void Envelope::Delete( int point )
+{
+   delete mEnv[point];
+   mEnv.RemoveAt(point);
+}
 
 // Returns true if parent needs to be redrawn
 bool Envelope::MouseEvent(wxMouseEvent & event, wxRect & r,
                           double h, double pps, bool dB,
-                          float zoomMin, float zoomMax)
+                          float zoomMin, float zoomMax, float eMin, float eMax)
 {
 
-   if (event.ButtonDown())
+   if (event.ButtonDown() && mButton == wxMOUSE_BTN_NONE)
       return HandleMouseButtonDown( event, r, h, pps,dB,
-                                    zoomMin, zoomMax);
+                                    zoomMin, zoomMax, eMin, eMax);
 
    if (event.Dragging() && mDragPoint >= 0) 
       return HandleDragging( event, r, h, pps,dB,
-                             zoomMin, zoomMax);
+                             zoomMin, zoomMax, eMin, eMax);
 
-   if (event.ButtonUp()) 
+   if (event.ButtonUp() && event.GetButton() == mButton)
       return HandleMouseButtonUp( event, r, h, pps, dB,
                                   zoomMin, zoomMax);
 
@@ -564,6 +581,14 @@ void Envelope::CollapseRegion(double t0, double t1)
 
 void Envelope::Paste(double t0, Envelope *e)
 {
+   if (e->mEnv.Count() == 0 && e->mDefaultValue == this->mDefaultValue)
+   {
+      // msmeyer: The envelope is empty and has the same default value, so
+      // there is nothing that must be inserted, just return. This avoids
+      // the creation of unnecessary duplicate control points
+      return;
+   }
+   
    t0 = wxMin(t0 - mOffset, mTrackLen);
    double deltat = e->mTrackLen;
 
