@@ -6,13 +6,34 @@
 
   Dominic Mazzoni
 
-**********************************************************************/
+*******************************************************************//**
 
-#include "FreqWindow.h"
+\class FreqWindow
+\brief Displays a spectrum plot of the waveform.  Has options for 
+selecting parameters of the plot.
+
+Has a feature that finds peaks and rerports their value as you move 
+the mouse around.
+
+*//****************************************************************//**
+
+\class FreqPlot
+\brief Works with FreqWindow to dsplay a spectrum plot of the waveform.
+This class actually does the graph display.
+
+Has a feature that finds peaks and rerports their value as you move 
+the mouse around.
+
+*//*******************************************************************/
+
+
+#include "Audacity.h"
 
 // For compilers that support precompilation, includes "wx/wx.h".
 #include <wx/wxprec.h>
 
+
+
 #ifdef __BORLANDC__
 #pragma hdrstop
 #endif
@@ -27,17 +48,23 @@
 #include <wx/file.h>
 #include <wx/filedlg.h>
 #include <wx/intl.h>
+#include <wx/sizer.h>
+#include <wx/stattext.h>
+#include <wx/statusbr.h>
 #endif
 
 #include <wx/textfile.h>
 
 #include <math.h>
 
+#include "FreqWindow.h"
+
 #include "AColor.h"
 #include "FFT.h"
 #include "Internat.h"
 #include "PitchName.h"
-
+#include "Theme.h"
+#include "AllThemeResources.h"
 
 enum {
    FirstID = 7000,
@@ -52,6 +79,8 @@ enum {
 
 FreqWindow *gFreqWindow = NULL;
 
+// These specify the minimum plot window width
+
 #define FREQ_WINDOW_WIDTH 480
 #define FREQ_WINDOW_HEIGHT 330
 
@@ -70,10 +99,9 @@ void InitFreqWindow(wxWindow * parent)
 
 // FreqWindow
 
-BEGIN_EVENT_TABLE(FreqWindow, wxFrame)
+BEGIN_EVENT_TABLE(FreqWindow, wxDialog)
     EVT_CLOSE(FreqWindow::OnCloseWindow)
     EVT_SIZE(FreqWindow::OnSize)
-    EVT_PAINT(FreqWindow::OnPaint)
     EVT_BUTTON(FreqCloseButtonID, FreqWindow::OnCloseButton)
     EVT_BUTTON(FreqExportButtonID, FreqWindow::OnExport)
     EVT_CHOICE(FreqAlgChoiceID, FreqWindow::OnAlgChoice)
@@ -85,46 +113,22 @@ END_EVENT_TABLE()
 FreqWindow::FreqWindow(wxWindow * parent, wxWindowID id,
                            const wxString & title,
                            const wxPoint & pos):
-  wxFrame(parent, id, title, pos, wxSize(FREQ_WINDOW_WIDTH, FREQ_WINDOW_HEIGHT)),
+  wxDialog(parent, id, title, pos, wxDefaultSize,
+     wxDEFAULT_DIALOG_STYLE | wxRESIZE_BORDER ),
   mData(NULL), mProcessed(NULL), mBitmap(NULL)
 {
    mMouseX = 0;
    mMouseY = 0;
 
+   mFreqFont = wxFont(fontSize, wxSWISS, wxNORMAL, wxNORMAL);
    mArrowCursor = new wxCursor(wxCURSOR_ARROW);
    mCrossCursor = new wxCursor(wxCURSOR_CROSS);
 
-   mFreqPlot = new FreqPlot(this, 0,
-                            wxPoint(0, 0), wxSize(FREQ_WINDOW_WIDTH, 250));
-
-   mUpdateRect.x = 0;
-   mUpdateRect.y = 0;
-   mUpdateRect.width = FREQ_WINDOW_WIDTH;
-   mUpdateRect.height = 250;
-
-   mPlotRect.x = 10;
-   mPlotRect.y = 10;
-   mPlotRect.width = 460;
-   mPlotRect.height = 215;
-
    mLeftMargin = 40;
    mBottomMargin = 20;
 
-   mInfoRect.x = 10;
-   mInfoRect.y = 230;
-   mInfoRect.width = 460;
-   mInfoRect.height = 15;
-
-   mExportButton = new wxButton(this, FreqExportButtonID,
-                                _("Export..."),
-                                wxPoint(390, 260), wxSize(70, 20));
-
-   mCloseButton = new wxButton(this, FreqCloseButtonID,
-                               _("Close"), wxPoint(390, 290), wxSize(70, 20));
-#ifndef TARGET_CARBON
-   mCloseButton->SetDefault();
-   mCloseButton->SetFocus();
-#endif
+   mFreqPlot = new FreqPlot(this, 0,
+                            wxDefaultPosition, wxDefaultSize);
 
    wxString algChoiceStrings[5] = { _("Spectrum"),
       _("Standard Autocorrelation"),
@@ -136,9 +140,10 @@ FreqWindow::FreqWindow(wxWindow * parent, wxWindowID id,
       _("Cepstrum")
    };
 
+   wxStaticText *algLabel = new wxStaticText(this, wxID_ANY, _("Algorithm:"));
    mAlgChoice = new wxChoice(this, FreqAlgChoiceID,
-                             wxPoint(10, 260),
-                             wxSize(200, 20), 4, algChoiceStrings);
+                             wxDefaultPosition, wxDefaultSize,
+                             4, algChoiceStrings);
 
    mAlgChoice->SetSelection(0);
 
@@ -152,9 +157,10 @@ FreqWindow::FreqWindow(wxWindow * parent, wxWindowID id,
       wxT("16384")
    };
 
+   wxStaticText *sizeLabel = new wxStaticText(this, wxID_ANY, _("Size:"));
    mSizeChoice = new wxChoice(this, FreqSizeChoiceID,
-                              wxPoint(220, 260),
-                              wxSize(160, 20), 8, sizeChoiceStrings);
+                              wxDefaultPosition, wxDefaultSize,
+                              8, sizeChoiceStrings);
 
    mSizeChoice->SetSelection(2);
 
@@ -167,9 +173,10 @@ FreqWindow::FreqWindow(wxWindow * parent, wxWindowID id,
       funcChoiceStrings[i] = WindowFuncName(i) + wxString(_(" window"));
    }
 
+   wxStaticText *funcLabel = new wxStaticText(this, wxID_ANY, _("Function:"));
    mFuncChoice = new wxChoice(this, FreqFuncChoiceID,
-                              wxPoint(10, 290),
-                              wxSize(200, 20), f, funcChoiceStrings);
+                              wxDefaultPosition, wxDefaultSize,
+                              f, funcChoiceStrings);
 
    mFuncChoice->SetSelection(3);
    delete[]funcChoiceStrings;
@@ -178,26 +185,134 @@ FreqWindow::FreqWindow(wxWindow * parent, wxWindowID id,
       _("Log frequency")
    };
 
+   wxStaticText *axisLabel = new wxStaticText(this, wxID_ANY, _("Axis:"));
    mAxisChoice = new wxChoice(this, FreqAxisChoiceID,
-                              wxPoint(220, 290),
-                              wxSize(160, 20), 2, axisChoiceStrings);
+                              wxDefaultPosition, wxDefaultSize,
+                              2, axisChoiceStrings);
 
    mAxisChoice->SetSelection(0);
 
    mLogAxis = false;
 
-  #ifdef __WXMAC__
-   mBackgroundBrush.SetColour(wxColour(255, 255, 255));
-   mBackgroundPen.SetColour(wxColour(255, 255, 255));
-   SetBackgroundColour(wxColour(255, 255, 255));
-  #else
-   mBackgroundBrush.SetColour(wxColour(204, 204, 204));
-   mBackgroundPen.SetColour(wxColour(204, 204, 204));
-   SetBackgroundColour(wxColour(204, 204, 204));
-  #endif
+   mExportButton = new wxButton(this, FreqExportButtonID,
+                                _("Export..."));
 
-   // Min size, max size
-   SetSizeHints(FREQ_WINDOW_WIDTH, FREQ_WINDOW_HEIGHT, 20000, 20000);
+   mCloseButton = new wxButton(this, FreqCloseButtonID,
+                               _("Close"));
+
+#ifndef TARGET_CARBON
+   mCloseButton->SetDefault();
+   mCloseButton->SetFocus();
+#endif
+
+   mInfo = new wxStatusBar(this, wxID_ANY, wxST_SIZEGRIP);
+
+   // Set minimum sizes
+
+   mFreqPlot->SetMinSize( wxSize( FREQ_WINDOW_WIDTH, FREQ_WINDOW_HEIGHT ) );
+
+   wxRect r( 0, 0, 0, 0 );
+
+   r.Union( algLabel->GetRect() );
+   r.Union( funcLabel->GetRect() );
+   algLabel->SetMinSize( r.GetSize() );
+   funcLabel->SetMinSize( r.GetSize() );
+
+   r = wxRect( 0, 0, 0, 0 );
+
+   r.Union( mAlgChoice->GetRect() );
+   r.Union( mFuncChoice->GetRect() );
+   mAlgChoice->SetMinSize( r.GetSize() );
+   mFuncChoice->SetMinSize( r.GetSize() );
+
+   r = wxRect( 0, 0, 0, 0 );
+
+   r.Union( sizeLabel->GetRect() );
+   r.Union( axisLabel->GetRect() );
+   sizeLabel->SetMinSize( r.GetSize() );
+   axisLabel->SetMinSize( r.GetSize() );
+
+   r = wxRect( 0, 0, 0, 0 );
+
+   r.Union( mSizeChoice->GetRect() );
+   r.Union( mAxisChoice->GetRect() );
+   mSizeChoice->SetMinSize( r.GetSize() );
+   mAxisChoice->SetMinSize( r.GetSize() );
+
+   r = wxRect( 0, 0, 0, 0 );
+
+   r.Union( mExportButton->GetRect() );
+   r.Union( mCloseButton->GetRect() );
+   mExportButton->SetMinSize( r.GetSize() );
+   mCloseButton->SetMinSize( r.GetSize() );
+   
+   // Add everything to the sizers
+
+   wxBoxSizer *vs = new wxBoxSizer( wxVERTICAL );
+   wxBoxSizer *hs;
+
+   szr = new wxFlexGridSizer(2);
+   szr->AddGrowableCol( 1, 1 );
+   szr->AddGrowableRow( 0, 1 );
+   szr->SetFlexibleDirection( wxBOTH );
+
+   vRuler = new RulerPanel(this, wxID_ANY);
+   vRuler->ruler.SetBounds(0, 0, 100, 100); // Ruler can't handle small sizes
+   vRuler->ruler.SetOrientation(wxVERTICAL);
+   vRuler->ruler.SetRange(10.0, -90.0);
+   vRuler->ruler.SetFormat(Ruler::LinearDBFormat);
+   vRuler->ruler.SetUnits(_("dB"));
+   vRuler->ruler.SetLabelEdges(true);
+   int w, h;
+   vRuler->ruler.GetMaxSize(&w, NULL);
+   vRuler->SetSize(wxSize(w, 150));  // height needed for wxGTK
+   szr->Add( vRuler, 0, wxEXPAND|wxTOP|wxBOTTOM, 1 ); //for border around graph
+   szr->Add( mFreqPlot, 1, wxEXPAND );
+   szr->Add(1,1); //spacer on next row, under vRuler
+
+   hRuler  = new RulerPanel(this, wxID_ANY);
+   hRuler->ruler.SetBounds(0, 0, 100, 100); // Ruler can't handle small sizes
+   hRuler->ruler.SetOrientation(wxHORIZONTAL);
+   hRuler->ruler.SetLog(false);  //default for freq axis
+   hRuler->ruler.SetRange(10, 20000);  //dummy values - will get replaced
+   hRuler->ruler.SetFormat(Ruler::RealFormat);
+   hRuler->ruler.SetUnits(_("Hz"));
+   hRuler->ruler.SetFlip(true);
+   hRuler->ruler.SetLabelEdges(false);
+   hRuler->ruler.GetMaxSize(NULL, &h);
+   hRuler->SetMinSize(wxSize(-1, h));
+   szr->Add( hRuler, 0, wxEXPAND|wxLEFT|wxRIGHT, 1 ); //for border around graph
+   szr->Add(1,1); //spacer
+   mInfoText = new wxStaticText(this, wxID_ANY, wxT(""));   //box for info text
+   szr->Add( mInfoText, 0, wxEXPAND|wxALL, 5);
+
+   vs->Add(szr, 1, wxEXPAND|wxALL, 5);
+
+   vs->Add( 1, 5, 0 );
+
+   hs = new wxBoxSizer( wxHORIZONTAL );
+   hs->Add( algLabel, 0, wxALIGN_CENTER | wxLEFT | wxRIGHT, 5 );
+   hs->Add( mAlgChoice, 0, wxALIGN_CENTER | wxLEFT | wxRIGHT, 5 );
+   hs->Add( sizeLabel, 0, wxALIGN_CENTER | wxLEFT | wxRIGHT, 5 );
+   hs->Add( mSizeChoice, 0, wxALIGN_CENTER | wxLEFT | wxRIGHT, 5 );
+   hs->Add( 10, 1, 0 );
+   hs->Add( mExportButton, 0, wxALIGN_CENTER | wxALIGN_RIGHT | wxLEFT | wxRIGHT, 5 );
+   vs->Add( hs, 0, wxALIGN_CENTER_HORIZONTAL | wxLEFT | wxRIGHT | wxBOTTOM, 5 );
+
+   hs = new wxBoxSizer( wxHORIZONTAL );
+   hs->Add( funcLabel, 0, wxALIGN_CENTER | wxLEFT | wxRIGHT, 5 );
+   hs->Add( mFuncChoice, 0, wxALIGN_CENTER | wxLEFT | wxRIGHT, 5 );
+   hs->Add( axisLabel, 0, wxALIGN_CENTER | wxLEFT | wxRIGHT, 5 );
+   hs->Add( mAxisChoice, 0, wxALIGN_CENTER | wxLEFT | wxRIGHT, 5 );
+   hs->Add( 10, 1, 0 );
+   hs->Add( mCloseButton, 0, wxALIGN_CENTER | wxALIGN_RIGHT | wxLEFT | wxRIGHT, 5 );
+   vs->Add( hs, 0, wxALIGN_CENTER_HORIZONTAL | wxLEFT | wxRIGHT | wxBOTTOM, 5 );
+
+   vs->Add( mInfo, 0, wxEXPAND | wxBOTTOM, 0 );
+
+   SetAutoLayout( false );
+   SetSizerAndFit( vs );
+   Layout();
 }
 
 FreqWindow::~FreqWindow()
@@ -217,58 +332,166 @@ FreqWindow::~FreqWindow()
 
 void FreqWindow::OnSize(wxSizeEvent & event)
 {
-   int width, height;
-   GetClientSize(&width, &height);
-
-   // Original size was 440 x 330
+   Layout();
 
    mUpdateRect.x = 0;
    mUpdateRect.y = 0;
-   mUpdateRect.width = width;
-   mUpdateRect.height = height - 80;
+   mUpdateRect.SetSize( mFreqPlot->GetSize() );
+   mPlotRect = mUpdateRect;
 
-   mFreqPlot->SetSize(0, 0, width, height - 80);
+   DrawPlot();
 
-   mPlotRect.x = 10;
-   mPlotRect.y = 10;
-   mPlotRect.width = width - 20;
-   mPlotRect.height = height - 115;
-
-   mLeftMargin = 40;
-   mBottomMargin = 20;
-
-   mInfoRect.x = 10;
-   mInfoRect.y = height - 100;
-   mInfoRect.width = width - 20;
-   mInfoRect.height = 15;
-
-   mExportButton->SetSize(width - 90, height - 70, 70, 20);
-   mCloseButton->SetSize(width - 90, height - 40, 70, 20);
-
-   mAlgChoice->SetSize(10, height - 70, 160, 20);
-   mSizeChoice->SetSize(180, height - 70, 160, 20);
-   mFuncChoice->SetSize(10, height - 40, 160, 20);
-   mAxisChoice->SetSize(180, height - 40, 160, 20);
-
-   if (mBitmap)
-      delete mBitmap;
-   mBitmap = NULL;
-
-#ifdef __WXMAC__
+#if defined(__WXMSW__)
    Refresh(true);
 #endif
 }
 
+void FreqWindow::DrawPlot()
+{
+   if (mBitmap)
+   {
+      delete mBitmap;
+      mBitmap = NULL;
+   }
+
+   mBitmap = new wxBitmap(mUpdateRect.width, mUpdateRect.height);
+
+   wxMemoryDC memDC;
+   memDC.SelectObject(*mBitmap);
+
+   memDC.SetBackground(wxBrush(wxColour(254, 254, 254)));// DONT-THEM Mask colour.
+   memDC.Clear();
+
+   wxRect r = mPlotRect;
+
+   memDC.SetPen(*wxBLACK_PEN);
+   memDC.SetBrush(*wxWHITE_BRUSH);
+   memDC.DrawRectangle(r);
+
+   if (!mProcessed) {
+      if (mData && mDataLen < mWindowSize)
+         memDC.DrawText(_("Not enough data selected."), r.x + 5, r.y + 5);
+
+      return;
+   }
+
+   float yTotal = (mYMax - mYMin);
+
+   int alg = mAlgChoice->GetSelection();
+
+   int i;
+
+   memDC.SetFont(mFreqFont);
+
+   // Set up y axis ruler
+
+   if (alg == 0) {
+      vRuler->ruler.SetUnits(_("dB"));
+      vRuler->ruler.SetFormat(Ruler::LinearDBFormat);
+   } else {
+      vRuler->ruler.SetUnits(wxT(""));
+      vRuler->ruler.SetFormat(Ruler::RealFormat);
+   }
+   int w1, w2, h;
+   vRuler->ruler.GetMaxSize(&w1, &h);
+   vRuler->ruler.SetRange(mYMax, mYMin);
+   vRuler->ruler.GetMaxSize(&w2, &h);
+   if( w1 != w2 )   // Reduces flicker
+   {
+      vRuler->SetSize(wxSize(w2,h));
+      szr->Layout();
+      hRuler->Refresh(false);
+   }
+   vRuler->Refresh(false);
+
+   // Set up x axis ruler
+
+   int width = r.width - 2;
+
+   float xMin, xMax, xPos, xRatio, xLast, xStep;
+
+   if (alg == 0) {
+      xMin = mRate / mWindowSize;
+      xMax = mRate / 2;
+      xRatio = xMax / xMin;
+      xPos = xMin;
+      xLast = xPos / 2.0;
+      if (mLogAxis)
+      {
+         xStep = pow(2.0f, (log(xRatio) / log(2.0f)) / width);
+         hRuler->ruler.SetLog(true);
+      }
+      else
+      {
+         xStep = (xMax - xMin) / width;
+         hRuler->ruler.SetLog(false);
+      }
+      hRuler->ruler.SetUnits(_("Hz"));
+   } else {
+      xMin = 0;
+      xMax = mProcessedSize / mRate;
+      xPos = xMin;
+      xLast = xPos / 2.0;
+      xStep = (xMax - xMin) / width;
+      hRuler->ruler.SetLog(false);
+      hRuler->ruler.SetUnits(_("s"));
+   }
+   hRuler->ruler.SetRange(xMin, xMax-xStep);
+   hRuler->Refresh(false);
+
+   // Draw the plot
+
+   if (alg == 0)
+      memDC.SetPen(wxPen(theTheme.Colour( clrHzPlot ), 1, wxSOLID));
+   else
+      memDC.SetPen(wxPen(theTheme.Colour( clrWavelengthPlot), 1, wxSOLID));
+
+   xPos = xMin;
+
+   for (i = 0; i < width; i++) {
+      float y;
+
+      if (mLogAxis)
+         y = GetProcessedValue(xPos, xPos * xStep);
+      else
+         y = GetProcessedValue(xPos, xPos + xStep);
+
+      float ynorm = (y - mYMin) / yTotal;
+
+      int lineheight = int (ynorm * (r.height - 1));
+
+      if (lineheight > r.height - 2)
+         lineheight = r.height - 2;
+
+      if (ynorm > 0.0)
+         memDC.DrawLine(r.x + 1 + i, r.y + r.height - 1 - lineheight,
+                        r.x + 1 + i, r.y + r.height - 1);
+
+      if (mLogAxis)
+         xPos *= xStep;
+      else
+         xPos += xStep;
+   }
+
+   // Outline the graph
+   memDC.SetPen(*wxBLACK_PEN);
+   memDC.SetBrush(*wxTRANSPARENT_BRUSH);
+   memDC.DrawRectangle(r);
+
+   memDC.SelectObject( wxNullBitmap );
+
+   mBitmap->SetMask( new wxMask( *mBitmap, wxColour(254, 254, 254) ) );
+
+}
+
+
 void FreqWindow::PlotMouseEvent(wxMouseEvent & event)
 {
-   if (event.Moving() && event.m_x != mMouseX) {
+   if (event.Moving() && (event.m_x != mMouseX || event.m_y != mMouseY)) {
       mMouseX = event.m_x;
       mMouseY = event.m_y;
 
       wxRect r = mPlotRect;
-      r.x += mLeftMargin;
-      r.width -= mLeftMargin;
-      r.height -= mBottomMargin;
 
       if (r.Inside(mMouseX, mMouseY))
          mFreqPlot->SetCursor(*mCrossCursor);
@@ -307,7 +530,8 @@ void FreqWindow::OnAxisChoice(wxCommandEvent & event)
 {
    mLogAxis = (mAxisChoice->GetSelection())?true:false;
 
-   mFreqPlot->Refresh(false);
+   DrawPlot();
+   mFreqPlot->Refresh(true);
 }
 
 // If f(0)=y0, f(1)=y1, f(2)=y2, and f(3)=y3, this function finds
@@ -421,130 +645,19 @@ float FreqWindow::GetProcessedValue(float freq0, float freq1)
    return value;
 }
 
-void FreqWindow::OnPaint(wxPaintEvent & evt)
-{
-   int width, height;
-   GetSize(&width, &height);
-
-   wxPaintDC dc(this);
-
-   dc.SetBrush(mBackgroundBrush);
-   dc.SetPen(mBackgroundPen);
-   dc.DrawRectangle(0, mUpdateRect.height,
-                    width, height - mUpdateRect.height);
-}
-
 void FreqWindow::PlotPaint(wxPaintEvent & evt)
 {
-   wxPaintDC dc(mFreqPlot);
-
-   if (!mBitmap)
-      mBitmap = new wxBitmap(mUpdateRect.width, mUpdateRect.height);
-
-   wxMemoryDC memDC;
-
-   memDC.SelectObject(*mBitmap);
-
-   memDC.SetBrush(mBackgroundBrush);
-   memDC.SetPen(mBackgroundPen);
-   memDC.DrawRectangle(0, 0, mUpdateRect.width, mUpdateRect.height);
-
-   wxRect r = mPlotRect;
-   r.x += mLeftMargin;
-   r.width -= mLeftMargin;
-   r.height -= mBottomMargin;
-
-   memDC.SetPen(*wxBLACK_PEN);
-   memDC.SetBrush(*wxWHITE_BRUSH);
-   memDC.DrawRectangle(r);
-
-   if (!mProcessed) {
-      if (mData && mDataLen < mWindowSize)
-         memDC.DrawText(_("Not enough data selected."), r.x + 5, r.y + 5);
+   wxPaintDC dc( (wxWindow *) evt.GetEventObject() );
 
+   dc.DrawBitmap( *mBitmap, 0, 0, true );
+   if( mProcessed == NULL )
       return;
-   }
-
-   float yTotal = (mYMax - mYMin);
 
    int alg = mAlgChoice->GetSelection();
 
-   int i;
+   dc.SetFont(mFreqFont);
 
-#ifdef __WXMSW__
-   const int fontSize = 8;
-#else
-   const int fontSize = 10;
-#endif
-
-   wxFont freqWindowFont(fontSize, wxSWISS, wxNORMAL, wxNORMAL);
-   memDC.SetFont(freqWindowFont);
-
-   // Draw y axis and gridlines
-
-   if (alg == 0) {
-      for (float yi = mYMin; yi <= mYMax; yi += mYStep) {
-         i = (int) yi;
-         int y =
-             (int) (r.y + r.height - 1 -
-                    (yi - mYMin) * (r.height - 1) / yTotal);
-
-         // Light blue gridline
-         memDC.SetPen(wxPen(wxColour(204, 204, 255), 1, wxSOLID));
-         memDC.DrawLine(r.x, y, r.x + r.width, y);
-
-         // Pure blue axis line
-         memDC.SetPen(wxPen(wxColour(0, 0, 255), 1, wxSOLID));
-         memDC.DrawLine(mInfoRect.x, y, r.x, y);
-
-         if (i != (int) mYMin) {
-            wxString label = wxString::Format(wxT("%d dB"), i);
-            long labelWidth, labelHeight;
-            memDC.GetTextExtent(label, &labelWidth, &labelHeight);
-            memDC.DrawText(label, r.x - labelWidth - 2, y + 1);
-         }
-      }
-   } else {
-      wxString label;
-      long labelWidth, labelHeight;
-
-      if (fabs(mYMax) < 1.0)
-         label.Printf(wxT("%.3f"), mYMax);
-      else
-         label.Printf(wxT("%.1f"), mYMax);
-      memDC.GetTextExtent(label, &labelWidth, &labelHeight);
-      memDC.DrawText(label, r.x - labelWidth - 2, r.y + 2);
-
-      if (fabs(mYMin) < 1.0)
-         label.Printf(wxT("%.3f"), mYMin);
-      else
-         label.Printf(wxT("%.1f"), mYMin);
-      memDC.GetTextExtent(label, &labelWidth, &labelHeight);
-      memDC.DrawText(label,
-                     r.x - labelWidth - 2,
-                     r.y + r.height - labelHeight - 2);
-
-      if (mYMax > 0.0 && mYMin < 0.0) {
-         int y = int ((mYMax / (mYMax - mYMin)) * r.height);
-
-         if (y > labelHeight + 4 && y < r.height - 2 * labelHeight - 4) {
-            label = wxT("0.0");
-            memDC.GetTextExtent(label, &labelWidth, &labelHeight);
-            memDC.DrawText(label,
-                           r.x - labelWidth - 2, r.y + y - labelHeight);
-
-            // Light blue gridline
-            memDC.SetPen(wxPen(wxColour(204, 204, 255), 1, wxSOLID));
-            memDC.DrawLine(r.x, r.y + y, r.x + r.width, r.y + y);
-
-            // Pure blue axis line
-            memDC.SetPen(wxPen(wxColour(0, 0, 255), 1, wxSOLID));
-            memDC.DrawLine(mInfoRect.x, r.y + y, r.x, r.y + y);
-         }
-      }
-   }
-
-   // Draw x axis and gridlines
+   wxRect r = mPlotRect;
 
    int width = r.width - 2;
 
@@ -568,91 +681,10 @@ void FreqWindow::PlotPaint(wxPaintEvent & evt)
       xStep = (xMax - xMin) / width;
    }
 
-   int nextx = 0;
-
-   for (i = 0; i < width; i++) {
-
-      if ((mLogAxis && xPos / xLast >= 2.0) ||
-          (!mLogAxis && (i % 60) == 0)) {
-         int x = i + 1;
-
-         // Light blue gridline
-         memDC.SetPen(wxPen(wxColour(204, 204, 255), 1, wxSOLID));
-         memDC.DrawLine(r.x + x, r.y, r.x + x, r.y + r.height);
-
-         if (x >= nextx) {
-
-            // Pure blue axis line
-            memDC.SetPen(wxPen(wxColour(0, 0, 255), 1, wxSOLID));
-            memDC.DrawLine(r.x + x, r.y + r.height, r.x + x,
-                           r.y + r.height + 15);
-
-            // Label
-
-            wxString label;
-            if (alg == 0) {
-               if (xPos < 950.0)
-                  label = wxString::Format(wxT("%dHz"), int (xPos + 0.5));
-               else
-                  label =
-                      wxString::Format(wxT("%dKHz"),
-                                       int ((xPos / 1000.0) + 0.5));
-            } else
-               label = wxString::Format(wxT("%.4f s"), xPos);
-            long labelWidth, labelHeight;
-            memDC.GetTextExtent(label, &labelWidth, &labelHeight);
-            if (x + labelWidth < width)
-               memDC.DrawText(label, r.x + x + 3, r.y + r.height + 2);
-            nextx = x + labelWidth + 4;
-         }
-
-         xLast *= 2.0;
-      }
-
-      if (mLogAxis)
-         xPos *= xStep;
-      else
-         xPos += xStep;
-   }
-
-   // Draw the plot
-
-   if (alg == 0)
-      memDC.SetPen(wxPen(wxColour(140, 60, 190), 1, wxSOLID));
-   else
-      memDC.SetPen(wxPen(wxColour(200, 50, 150), 1, wxSOLID));
-
-   xPos = xMin;
-
-   for (i = 0; i < width; i++) {
-      float y;
-
-      if (mLogAxis)
-         y = GetProcessedValue(xPos, xPos * xStep);
-      else
-         y = GetProcessedValue(xPos, xPos + xStep);
-
-      float ynorm = (y - mYMin) / yTotal;
-
-      int lineheight = int (ynorm * (r.height - 1));
-
-      if (lineheight > r.height - 2)
-         lineheight = r.height - 2;
-
-      if (ynorm > 0.0)
-         memDC.DrawLine(r.x + 1 + i, r.y + r.height - 1 - lineheight,
-                        r.x + 1 + i, r.y + r.height - 1);
-
-      if (mLogAxis)
-         xPos *= xStep;
-      else
-         xPos += xStep;
-   }
-
    // Find the peak nearest the cursor and plot it
 
    float bestpeak = float(0.0);
-   if (r.Inside(mMouseX, mMouseY)) {
+   if ( r.Inside(mMouseX, mMouseY) & (mMouseX!=0) & (mMouseX!=r.width-1) ) {
       if (mLogAxis)
          xPos = xMin * pow(xStep, mMouseX - (r.x + 1));
       else
@@ -692,36 +724,13 @@ void FreqWindow::PlotPaint(wxPaintEvent & evt)
       if (mLogAxis)
          px = int (log(bestpeak / xMin) / log(xStep));
       else
-      px = int ((bestpeak - xMin) * width / (xMax - xMin));
+         px = int ((bestpeak - xMin) * width / (xMax - xMin));
 
-      memDC.SetPen(wxPen(wxColour(160,160,160), 1, wxSOLID));
-      memDC.DrawLine(r.x + 1 + px, r.y, r.x + 1 + px, r.y + r.height);
-   }
-   // Outline the graph
+      dc.SetPen(wxPen(wxColour(160,160,160), 1, wxSOLID));
+      dc.DrawLine(r.x + 1 + px, r.y, r.x + 1 + px, r.y + r.height);
 
-   memDC.SetPen(*wxBLACK_PEN);
-   memDC.SetBrush(*wxTRANSPARENT_BRUSH);
-   memDC.DrawRectangle(r);
+       // print out info about the cursor location
 
-   // Draw the bevel around the info rect
-
-   AColor::Dark(&memDC, false);
-   memDC.DrawLine(mInfoRect.x, mInfoRect.y,
-                  mInfoRect.x + mInfoRect.width, mInfoRect.y);
-   memDC.DrawLine(mInfoRect.x, mInfoRect.y,
-                  mInfoRect.x, mInfoRect.y + mInfoRect.height);
-   AColor::Light(&memDC, false);
-   memDC.DrawLine(mInfoRect.x, mInfoRect.y + mInfoRect.height,
-                  mInfoRect.x + mInfoRect.width,
-                  mInfoRect.y + mInfoRect.height);
-   memDC.DrawLine(mInfoRect.x + mInfoRect.width, mInfoRect.y,
-                  mInfoRect.x + mInfoRect.width,
-                  mInfoRect.y + mInfoRect.height);
-
-   // If the mouse cursor is pointing inside the graph, print out info about the
-   // cursor location
-
-   if (r.Inside(mMouseX, mMouseY)) {
       float value;
 
       if (mLogAxis) {
@@ -733,31 +742,39 @@ void FreqWindow::PlotPaint(wxPaintEvent & evt)
       }
 
       wxString info;
+      wxString xpitch;
+      wxString peakpitch;
+      const wxChar *xp;
+      const wxChar *pp;
 
       if (alg == 0) {
-         wxString xpitch = PitchName_Absolute(Freq2Pitch(xPos));
-         wxString peakpitch = PitchName_Absolute(Freq2Pitch(bestpeak));
-         const wxChar *xp = xpitch.c_str();
-         const wxChar *pp = peakpitch.c_str();
+         xpitch = PitchName_Absolute(Freq2Pitch(xPos));
+         peakpitch = PitchName_Absolute(Freq2Pitch(bestpeak));
+         xp = xpitch.c_str();
+         pp = peakpitch.c_str();
          info.Printf(_("Cursor: %d Hz (%s) = %d dB    Peak: %d Hz (%s)"),
                      int (xPos + 0.5),
                      xp, int (value + 0.5), int (bestpeak + 0.5), pp);
       } else if (xPos > 0.0 && bestpeak > 0.0) {
-         wxString xpitch = PitchName_Absolute(Freq2Pitch(1.0 / xPos));
-         wxString peakpitch = PitchName_Absolute(Freq2Pitch(1.0 / bestpeak));
-         const wxChar *xp = xpitch.c_str();
-         const wxChar *pp = peakpitch.c_str();
+         xpitch = PitchName_Absolute(Freq2Pitch(1.0 / xPos));
+         peakpitch = PitchName_Absolute(Freq2Pitch(1.0 / bestpeak));
+         xp = xpitch.c_str();
+         pp = peakpitch.c_str();
          info.Printf(_("Cursor: %.4f sec (%d Hz) (%s) = %f,    Peak: %.4f sec (%d Hz) (%s)"),
                      xPos,
                      int (1.0 / xPos + 0.5),
                      xp, value, bestpeak, int (1.0 / bestpeak + 0.5), pp);
       }
-
-      memDC.DrawText(info, mInfoRect.x + 2, mInfoRect.y + 2);
+      mInfoText->SetLabel(info);
    }
+   else
+      mInfoText->SetLabel(wxT(""));
 
-   dc.Blit(0, 0, mUpdateRect.width, mUpdateRect.height,
-           &memDC, 0, 0, wxCOPY, FALSE);
+
+   // Outline the graph
+   dc.SetPen(*wxBLACK_PEN);
+   dc.SetBrush(*wxTRANSPARENT_BRUSH);
+   dc.DrawRectangle(r);
 }
 
 void FreqWindow::OnCloseWindow(wxCloseEvent & WXUNUSED(event))
@@ -780,6 +797,8 @@ void FreqWindow::Plot(int len, float *data, double rate)
    for (int i = 0; i < len; i++)
       mData[i] = data[i];
    Recalc();
+   wxSizeEvent dummy;
+   OnSize( dummy );
 }
 
 void FreqWindow::Recalc()
@@ -789,7 +808,7 @@ void FreqWindow::Recalc()
    mProcessed = NULL;
 
    if (!mData) {
-      mFreqPlot->Refresh(false);
+      mFreqPlot->Refresh(true);
       return;
    }
 
@@ -800,14 +819,14 @@ void FreqWindow::Recalc()
 
    if (!(windowSize >= 32 && windowSize <= 65536 &&
          alg >= 0 && alg <= 3 && windowFunc >= 0 && windowFunc <= 3)) {
-      mFreqPlot->Refresh(false);
+      mFreqPlot->Refresh(true);
       return;
    }
 
    mWindowSize = windowSize;
 
    if (mDataLen < mWindowSize) {
-      mFreqPlot->Refresh(false);
+      mFreqPlot->Refresh(true);
       return;
    }
 
@@ -986,7 +1005,8 @@ void FreqWindow::Recalc()
    delete[]out;
    delete[]out2;
 
-   mFreqPlot->Refresh(false);
+   DrawPlot();
+   mFreqPlot->Refresh(true);
 }
 
 void FreqWindow::OnExport(wxCommandEvent & WXUNUSED(event))
@@ -999,10 +1019,10 @@ void FreqWindow::OnExport(wxCommandEvent & WXUNUSED(event))
    if (fName == wxT(""))
       return;
 
-   wxTextFile f(FILENAME(fName).c_str());
+   wxTextFile f(fName);
 #ifdef __WXMAC__
    wxFile *temp = new wxFile();
-   temp->Create(FILENAME(fName));
+   temp->Create(fName);
    delete temp;
 #else
    f.Create();
