@@ -6,7 +6,25 @@
 
   Dominic Mazzoni
 
-**********************************************************************/
+*******************************************************************//**
+\file Sequence.cpp
+\brief Appears to duplicate some structures such as SeqBlock that are 
+also found in Track.cpp. 
+
+*//****************************************************************//**
+
+\class Sequence
+\brief Has a sequence of samples.  Compare with RingBuffer.
+
+*//****************************************************************//**
+
+\class SeqBlock
+\brief Pointer to a BlockFile along with a start time.  Element of 
+a BlockArray.  WARNING defined differently in Track.cpp and 
+Sequence.cpp.  Not yet sure why.
+
+*//*******************************************************************/
+
 
 #include "Audacity.h"
 
@@ -24,17 +42,9 @@
 #include "BlockFile.h"
 #include "DirManager.h"
 
+#include "blockfile/SimpleBlockFile.h"
 #include "blockfile/SilentBlockFile.h"
 
-class SeqBlock {
- public:
-   BlockFile * f;
-
-   sampleCount start;
-};
-
-WX_DEFINE_ARRAY(SeqBlock *, BlockArray);
-
 int Sequence::sMaxDiskBlockSize = 1048576;
 
 // Sequence methods
@@ -559,7 +569,7 @@ bool Sequence::AppendBlock(SeqBlock * b)
    newBlock->start = mNumSamples;
    newBlock->f = mDirManager->CopyBlockFile(b->f);
    if (!newBlock->f) {
-      // TODO error Could not paste!  (Out of disk space?)
+      /// \todo Error Could not paste!  (Out of disk space?)
       return false;
    }
 
@@ -700,39 +710,28 @@ XMLTagHandler *Sequence::HandleXMLChild(const wxChar *tag)
    }
 }
 
-void Sequence::WriteXML(int depth, FILE *fp)
+void Sequence::WriteXML(XMLWriter &xmlFile)
 {
-   int i;
    unsigned int b;
    
-   for(i=0; i<depth; i++)
-      fprintf(fp, "\t");
-   fprintf(fp, "<sequence ");
-   fprintf(fp, "maxsamples=\"%d\" ", (int)mMaxSamples);
-   fprintf(fp, "sampleformat=\"%d\" ", (int)mSampleFormat);
-   fprintf(fp, "numsamples=\"%d\" ", (int)mNumSamples);
-   fprintf(fp, ">\n");
+   xmlFile.StartTag(wxT("sequence"));
+
+   xmlFile.WriteAttr(wxT("maxsamples"), mMaxSamples);
+   xmlFile.WriteAttr(wxT("sampleformat"), mSampleFormat);
+   xmlFile.WriteAttr(wxT("numsamples"), mNumSamples);
 
    for (b = 0; b < mBlock->Count(); b++) {
       SeqBlock *bb = mBlock->Item(b);
-      for(i=0; i<depth+1; i++)
-         fprintf(fp, "\t");
-      fprintf(fp, "<waveblock ");
-      fprintf(fp, "start=\"%d\" ", bb->start);
-      fprintf(fp, ">\n");
 
-      wxFFile xmlFile(fp);
-      bb->f->SaveXML(depth+2, xmlFile);
-      xmlFile.Detach();
+      xmlFile.StartTag(wxT("waveblock"));
+      xmlFile.WriteAttr(wxT("start"), bb->start);
 
-      for(i=0; i<depth+1; i++)
-         fprintf(fp, "\t");
-      fprintf(fp, "</waveblock>\n");
+      bb->f->SaveXML(xmlFile);
+
+      xmlFile.EndTag(wxT("waveblock"));
    }
 
-   for(i=0; i<depth; i++)
-      fprintf(fp, "\t");
-   fprintf(fp, "</sequence>\n");
+   xmlFile.EndTag(wxT("sequence"));
 }
 
 int Sequence::FindBlock(sampleCount pos, sampleCount lo,
@@ -1100,7 +1099,7 @@ sampleCount Sequence::GetIdealAppendLen()
 }
 
 bool Sequence::Append(samplePtr buffer, sampleFormat format,
-                      sampleCount len)
+                      sampleCount len, XMLWriter* blockFileLog /*=NULL*/)
 {
    // Quick check to make sure that it doesn't overflow
    if (((double)mNumSamples) + ((double)len) > 2147483647)
@@ -1127,23 +1126,20 @@ bool Sequence::Append(samplePtr buffer, sampleFormat format,
       samplePtr buffer2 = NewSamples((lastBlock->f->GetLength() + addLen), mSampleFormat);
       Read(buffer2, mSampleFormat, lastBlock, 0, lastBlock->f->GetLength());
 
-      if (format == mSampleFormat)
-         memcpy(buffer2 + lastBlock->f->GetLength() * SAMPLE_SIZE(format),
-                buffer,
-                addLen * SAMPLE_SIZE(format));
-      else {
-         CopySamples(buffer, format, temp, mSampleFormat, addLen);
-         memcpy(buffer2 + lastBlock->f->GetLength() * SAMPLE_SIZE(format),
-                temp,
-                addLen * SAMPLE_SIZE(format));
-      }
+      CopySamples(buffer,
+                  format,
+                  buffer2 + lastBlock->f->GetLength() * SAMPLE_SIZE(format),
+                  mSampleFormat,
+                  addLen);
 
       newLastBlock->start = lastBlock->start;
       int newLastBlockLen = lastBlock->f->GetLength() + addLen;
 
       newLastBlock->f =
          mDirManager->NewSimpleBlockFile(buffer2, newLastBlockLen, mSampleFormat);
-
+      if (blockFileLog)
+         ((SimpleBlockFile*)newLastBlock->f)->SaveXML(*blockFileLog);
+         
       DeleteSamples(buffer2);
 
       mDirManager->Deref(lastBlock->f);
@@ -1161,8 +1157,7 @@ bool Sequence::Append(samplePtr buffer, sampleFormat format,
       SeqBlock *w = new SeqBlock();
       w->start = mNumSamples;
 
-      if (format == mSampleFormat)
-      {
+      if (format == mSampleFormat) {
          w->f = mDirManager->NewSimpleBlockFile(buffer, l, mSampleFormat);
       }
       else {
@@ -1170,6 +1165,9 @@ bool Sequence::Append(samplePtr buffer, sampleFormat format,
          w->f = mDirManager->NewSimpleBlockFile(temp, l, mSampleFormat);
       }
 
+      if (blockFileLog)
+         ((SimpleBlockFile*)w->f)->SaveXML(*blockFileLog);
+
       mBlock->Add(w);
 
       buffer += l * SAMPLE_SIZE(format);
@@ -1488,6 +1486,17 @@ void Sequence::SetMaxDiskBlockSize(int bytes)
    sMaxDiskBlockSize = bytes;
 }
 
+void Sequence::AppendBlockFile(BlockFile* blockFile)
+{
+   SeqBlock *w = new SeqBlock();
+   w->start = mNumSamples;
+   w->f = blockFile;
+   mBlock->Add(w);
+   mNumSamples += blockFile->GetLength();
+
+   ConsistencyCheck(wxT("AppendBlockFile"));
+}
+
 
 // Indentation settings for Vim and Emacs and unique identifier for Arch, a
 // version control system. Please do not modify past this point.
