@@ -6,14 +6,20 @@
 
   Dominic Mazzoni
 
-  This class handles the actual rendering of WaveTracks (both
+
+*******************************************************************//*!
+
+\class TrackArtist
+\brief   This class handles the actual rendering of WaveTracks (both
   waveforms and spectra), NoteTracks, LabelTracks and TimeTracks.
 
   It's actually a little harder than it looks, because for
   waveforms at least it needs to cache the samples that are
   currently on-screen.
 
-**********************************************************************/
+
+*//*******************************************************************/
+
 #include "Audacity.h"
 #include "TrackArtist.h"
 
@@ -44,6 +50,8 @@
 #include "Spectrum.h"
 #include "ViewInfo.h"
 #include "widgets/Ruler.h"
+#include "Theme.h"
+#include "AllThemeResources.h"
 
 #undef PROFILE_WAVEFORM
 #ifdef PROFILE_WAVEFORM
@@ -69,30 +77,12 @@ const int notePos[12] = { 1, 6, 11, 16, 21,
 
 TrackArtist::TrackArtist()
 {
-   AColor::Init();
-
    mInsetLeft   = 0;
    mInsetTop    = 0;
    mInsetRight  = 0;
    mInsetBottom = 0;
 
-   blankBrush     .SetColour(214, 214, 214);
-   unselectedBrush.SetColour(192, 192, 192);
-   selectedBrush  .SetColour(148, 148, 170);
-   sampleBrush    .SetColour( 50,  50, 200);
-   selsampleBrush .SetColour( 50,  50, 200);
-   dragsampleBrush.SetColour(  0,   0,   0);
-
-   blankPen     .SetColour(214, 214, 214);
-   unselectedPen.SetColour(192, 192, 192);
-   selectedPen  .SetColour(148, 148, 170);
-   samplePen    .SetColour( 50,  50, 200);
-   selsamplePen .SetColour( 50,  50, 200);
-   muteSamplePen.SetColour(136, 136, 144);
-   rmsPen       .SetColour(100, 100, 220);
-   muteRmsPen   .SetColour(136, 136, 144);
-   shadowPen    .SetColour(148, 148, 148);
-
+   SetColours();
    vruler = new Ruler();
 }
 
@@ -101,6 +91,26 @@ TrackArtist::~TrackArtist()
    delete vruler;
 }
 
+void TrackArtist::SetColours()
+{
+   theTheme.SetBrushColour( blankBrush,      clrBlank );
+   theTheme.SetBrushColour( unselectedBrush, clrUnselected);
+   theTheme.SetBrushColour( selectedBrush,   clrSelected);
+   theTheme.SetBrushColour( sampleBrush,     clrSample);
+   theTheme.SetBrushColour( selsampleBrush,  clrSelSample);
+   theTheme.SetBrushColour( dragsampleBrush, clrDragSample);
+
+   theTheme.SetPenColour(   blankPen,        clrBlank);
+   theTheme.SetPenColour(   unselectedPen,   clrUnselected);
+   theTheme.SetPenColour(   selectedPen,     clrSelected);
+   theTheme.SetPenColour(   samplePen,       clrSample);
+   theTheme.SetPenColour(   selsamplePen,    clrSelSample);
+   theTheme.SetPenColour(   muteSamplePen,   clrMuteSample);
+   theTheme.SetPenColour(   rmsPen,          clrRms);
+   theTheme.SetPenColour(   muteRmsPen,      clrMuteRms);
+   theTheme.SetPenColour(   shadowPen,       clrShadow);
+}
+
 void TrackArtist::SetInset(int left, int top, int right, int bottom)
 {
    mInsetLeft   = left;
@@ -110,13 +120,16 @@ void TrackArtist::SetInset(int left, int top, int right, int bottom)
 }
 
 void TrackArtist::DrawTracks(TrackList * tracks,
-                             wxDC & dc, wxRect & r,
+                             wxDC & dc,
+                             wxRegion & reg,
+                             wxRect & r,
                              wxRect & clip,
                              ViewInfo * viewInfo,
                              bool drawEnvelope,
                              bool drawSamples,
                              bool drawSliders)
 {
+   SetColours();
    wxRect trackRect = r;
    wxRect stereoTrackRect;
 
@@ -176,7 +189,8 @@ void TrackArtist::DrawTracks(TrackList * tracks,
       }
 
       if (stereoTrackRect.y < (clip.y + clip.height) &&
-          stereoTrackRect.y + stereoTrackRect.height > clip.y) {
+          stereoTrackRect.y + stereoTrackRect.height > clip.y
+          && reg.Contains( stereoTrackRect )) {
 
          wxRect rr = trackRect;
          rr.x += mInsetLeft;
@@ -440,11 +454,11 @@ int TrackArtist::GetWaveYPos(float value, int height, bool dB, float dBr)
    }
 }
 
-// Takes a value between min    and max and returns a value between 
-//                       height and 0 
-// ToDo:  Should this function move int GuiWaveTrack where it can
-// then use the zoomMin, zoomMax and height values without having 
-// to have them passed in to it??
+/// Takes a value between min and max and returns a value between 
+/// height and 0 
+/// \todo  Should this function move int GuiWaveTrack where it can
+/// then use the zoomMin, zoomMax and height values without having 
+/// to have them passed in to it??
 int GetWaveYPosNew(float value, float min, float max,
 		   int height, bool dB, bool outer,
 		   float dBr, bool clip)
@@ -463,11 +477,11 @@ int GetWaveYPosNew(float value, float min, float max,
         value *= sign;
      }
    }else
-     if(!outer) 
-       if( value >= 0.0)
-	 value -= 0.5;
-       else
-	 value += 0.5;
+      if(!outer) 
+         if( value >= 0.0)
+            value -= 0.5;
+         else
+            value += 0.5;
 
    if (clip) {
       if (value < min)
@@ -475,10 +489,8 @@ int GetWaveYPosNew(float value, float min, float max,
       if (value > max)
          value = max;
    }
-
    value = (max - value) / (max - min);
-
-   return (int) (value * height + sign * 0.5);
+   return (int) (value * (height - 1.0) + 0.5);
 }
 
 // This function isn't used anymore.  Use GetWaveYPosNew instead.
@@ -644,11 +656,11 @@ void TrackArtist::DrawWaveformBackground(wxDC &dc, wxRect r,
             dc.DrawLine(r.x + x, r.y + maxtop[x],
                         r.x + x, r.y + maxbot[x]);
             dc.DrawLine(r.x + x, r.y + mintop[x],
-                        r.x + x, r.y + minbot[x]);
+                        r.x + x, r.y + minbot[x] + 1);
          }
          else
             dc.DrawLine(r.x + x, r.y + maxtop[x],
-                        r.x + x, r.y + minbot[x]);
+                        r.x + x, r.y + minbot[x] + 1);
       }
    }
 
@@ -1108,12 +1120,12 @@ void TrackArtist::DrawClipWaveform(WaveTrack* track, WaveClip* clip,
    //is spread across r.height.  Draw the line at the proper place.
 
    if(trackmin < 0 && trackmax > 0)
-      {
+   {
          float height = (trackmax/(trackmax-trackmin))*mid.height;
          dc.SetPen(*wxBLACK_PEN);
          dc.DrawLine(mid.x, (int) (mid.y+height), mid.x+mid.width,
                (int) (mid.y+height));
-      }
+   }
 
 
    if (!showIndividualSamples)
@@ -1135,9 +1147,9 @@ void TrackArtist::DrawClipWaveform(WaveTrack* track, WaveClip* clip,
    #endif
 
    if (showIndividualSamples)
-      DrawIndividualSamples(dc, mid, track, t0, pps, h,
-                            zoomMin, zoomMax, 
-                            dB, drawSamples, showPoints, muted);
+      DrawIndividualClipSamples(dc, mid, clip, t0, pps, h,
+                                zoomMin, zoomMax, 
+                                dB, drawSamples, showPoints, muted);
 
    if (drawEnvelope) {
       dc.SetPen(AColor::envelopePen);
