@@ -15,12 +15,13 @@
 #include <wx/timer.h>
 #include <wx/window.h>
 #include <wx/panel.h>
+#include <wx/dcmemory.h>
 
 //Stm:  The following included because of the sampleCount struct.
 #include "Sequence.h"  
 #include "WaveClip.h"
 #include "WaveTrack.h"
-  
+
 class wxMenu;
 class wxRect;
 
@@ -34,10 +35,16 @@ class Ruler;
 class AdornedRulerPanel;
 class LWSlider;
 class ControlToolBar; //Needed because state of controls can affect what gets drawn.
+class ToolsToolBar; //Needed because state of controls can affect what gets drawn.
 class AudacityProject;
 
+class TrackPanelAx;
+
 struct ViewInfo;
 
+#ifdef OLD_CODE
+/// Holds a bitmap and its position, this is an element of a tpBitmapArray,
+/// which appears to be not used anymore.
 struct tpBitmap
 {
    wxBitmap *bitmap;
@@ -46,6 +53,8 @@ struct tpBitmap
 };
 
 WX_DEFINE_ARRAY(tpBitmap *, tpBitmapArray);
+#endif
+
 WX_DEFINE_ARRAY(LWSlider *, LWSliderArray);
 
 class TrackClip
@@ -67,6 +76,7 @@ class TrackPanelListener {
    virtual void TP_DisplaySelection() = 0;
    virtual void TP_DisplayStatusMessage(wxString msg) = 0;
    virtual int TP_GetCurrentTool() = 0;
+   virtual ToolsToolBar * TP_GetToolsToolBar() = 0;
    virtual ControlToolBar * TP_GetControlToolBar() = 0;
    virtual void TP_OnPlayKey() = 0;
    virtual void TP_PushState(wxString shortDesc, wxString longDesc,
@@ -77,23 +87,10 @@ class TrackPanelListener {
    virtual void TP_ScrollRight() = 0;
    virtual void TP_ScrollWindow(double scrollto) = 0;
    virtual void TP_ScrollUpDown(int delta) = 0;
-   virtual void TP_HasMouse() = 0;
    virtual void TP_HandleResize() = 0;
-   virtual void TP_GiveFocus(bool bForward) = 0;
 };
 
 
-/// The TrackLabel is shown to the side of a track 
-/// It has the menus, pan and gain controls displayed in it.
-///
-/// In its current implementation it is not derived from a
-/// wxWindow.  Following the original coding style, it has 
-/// been coded as a 'flyweight' class, which is passed 
-/// state as needed, except for the array of gains and pans.
-/// 
-/// An alternative way to code this is to have an instance
-/// of this class for each instance displayed.
-/// 
 class TrackLabel
 {
 public:
@@ -133,9 +130,7 @@ public:
 
 const int DragThreshold = 3;// Anything over 3 pixels is a drag, else a click.
 
-/// The TrackPanel manages multiple tracks and their TrackLabels.
-/// Note that with stereo tracks there will be one TrackLabel
-/// being used by two wavetracks.
+
 class TrackPanel:public wxPanel {
  public:
 
@@ -145,15 +140,24 @@ class TrackPanel:public wxPanel {
               const wxPoint & pos,
               const wxSize & size,
               TrackList * tracks,
-              ViewInfo * viewInfo, TrackPanelListener * listener);
+              ViewInfo * viewInfo,
+              TrackPanelListener * listener,
+              AdornedRulerPanel * ruler );
 
    virtual ~ TrackPanel();
 
    void UpdatePrefs();
 
+   void OnSize(wxSizeEvent & event);
+   void OnErase(wxEraseEvent & event);
    void OnPaint(wxPaintEvent & event);
    void OnMouseEvent(wxMouseEvent & event);
-   void OnKeyEvent(wxKeyEvent & event);
+   void OnCaptureKey(wxCommandEvent & event);
+   void OnKeyDown(wxKeyEvent & event);
+   void OnChar(wxKeyEvent & event);
+
+   void OnSetFocus(wxFocusEvent & event);
+   void OnKillFocus(wxFocusEvent & event);
 
    void OnContextMenu(wxContextMenuEvent & event);
 
@@ -161,7 +165,6 @@ class TrackPanel:public wxPanel {
 
    void OnTimer();
 
-   int GetRulerHeight();
    int GetLeftOffset() const { return GetLabelWidth() + 1;}
 
    void GetTracksUsableArea(int *width, int *height) const;
@@ -172,31 +175,43 @@ class TrackPanel:public wxPanel {
 
    virtual void Refresh(bool eraseBackground = TRUE,
                         const wxRect *rect = (const wxRect *) NULL);
+   void RefreshTrack(Track *trk, bool refreshbacking = false);
+
    void DisplaySelection();
 
    void SetSelectionFormat(int iformat);
    void SetSnapTo(int snapto);
 
    void HandleShiftKey(bool down);
+   void HandleControlKey(bool down);
    AudacityProject * GetProject() const;
 
-   void OnPrevTrack();
-   void OnNextTrack();
-   void OnTrackPan(Track * t);
-   void OnTrackPanLeft(Track * t);
-   void OnTrackPanRight(Track * t);
-   void OnTrackGain(Track * t);
-   void OnTrackGainDec(Track * t);
-   void OnTrackGainInc(Track * t);
-   void OnTrackMenu(Track * t);
-   void OnTrackMute(Track * t, bool shiftdown);
-   void OnTrackSolo(Track * t, bool shiftdown);
-   void OnTrackClose(Track * t);
+   void OnPrevTrack(bool shift = false);
+   void OnNextTrack(bool shift = false);
+   void OnToggle();
+
+   void OnCursorLeft(bool shift, bool ctrl);
+   void OnCursorRight(bool shift, bool ctrl);
+   void ScrollIntoView(double pos);
+
+   void OnTrackPan();
+   void OnTrackPanLeft();
+   void OnTrackPanRight();
+   void OnTrackGain();
+   void OnTrackGainDec();
+   void OnTrackGainInc();
+   void OnTrackMenu(Track *t = NULL);
+   void OnTrackMute(bool shiftdown, Track *t = NULL);
+   void OnTrackSolo(bool shiftdown, Track *t = NULL);
+   void OnTrackClose();
    Track * GetFirstSelectedTrack();
 
    void EnsureVisible(Track * t);
 
-   void TakeFocus(bool bForward);
+   Track *GetFocusedTrack();
+   void SetFocusedTrack(Track *t);
+
+   void HandleCursorForLastMouseEvent();
 
  private:
 
@@ -204,15 +219,15 @@ class TrackPanel:public wxPanel {
    bool HandleLabelTrackMouseEvent(LabelTrack * lTrack, wxRect &r, wxMouseEvent & event);
    bool HandleTrackLocationMouseEvent(WaveTrack * track, wxRect &r, wxMouseEvent &event);
    void HandleTrackSpecificMouseEvent(wxMouseEvent & event);
-   void DrawCursors(wxDC * dc = NULL);
-   void RemoveStaleCursors(wxRegionIterator * upd);
+   void DrawIndicator();
+   void DoDrawIndicator(wxDC & dc);
+   void DrawCursor();
+   void DoDrawCursor(wxDC & dc);
 
    void ScrollDuringDrag();
-   void UpdateIndicator(wxDC * dc = NULL);
-   void RemoveStaleIndicators(wxRegionIterator * upd);
 
    // Working out where to dispatch the event to.
-   int DetermineToolToUse( ControlToolBar * pCtb, wxMouseEvent & event);
+   int DetermineToolToUse( ToolsToolBar * pTtb, wxMouseEvent & event);
    bool HitTestEnvelope(Track *track, wxRect &r, wxMouseEvent & event);
    bool HitTestSamples(Track *track, wxRect &r, wxMouseEvent & event);
    bool HitTestSlide(Track *track, wxRect &r, wxMouseEvent & event);
@@ -224,7 +239,7 @@ class TrackPanel:public wxPanel {
 			     Track* pTrack, wxRect r, int num);
    void StartSelection (int, int);
    void ExtendSelection(int, int);
-   void SelectAllTracks();
+   void SelectTracksByLabel( LabelTrack *t );
 
 
    // AS: Cursor handling
@@ -253,8 +268,8 @@ class TrackPanel:public wxPanel {
    void HandleZoomDrag(wxMouseEvent & event);
    void HandleZoomButtonUp(wxMouseEvent & event);
 
-   void DragZoom(int x);
-   void DoZoomInOut(wxMouseEvent &event, int x_center);
+   void DragZoom(wxMouseEvent &event, int x);
+   void DoZoomInOut(wxMouseEvent &event, int x);
 
    void HandleVZoom(wxMouseEvent & event);
    void HandleVZoomClick(wxMouseEvent & event);
@@ -262,7 +277,7 @@ class TrackPanel:public wxPanel {
    void HandleVZoomButtonUp(wxMouseEvent & event);
 
    // Handle sample editing using the 'draw' tool.
-   bool IsSampleEditingPossible( wxMouseEvent & event );
+   bool IsSampleEditingPossible( wxMouseEvent & event, Track * t );
    void HandleSampleEditing(wxMouseEvent & event);
    void HandleSampleEditingClick( wxMouseEvent & event );
    void HandleSampleEditingDrag( wxMouseEvent & event );
@@ -271,9 +286,6 @@ class TrackPanel:public wxPanel {
    // MM: Handle mouse wheel rotation
    void HandleWheelRotation(wxMouseEvent & event);
 
-   void DoPopupMenu(wxMouseEvent &event, wxRect& titleRect, 
-		    Track* t, wxRect &r, int num);
-
    // Handle resizing.
    void HandleResizeClick(wxMouseEvent & event);
    void HandleResizeDrag(wxMouseEvent & event);
@@ -284,11 +296,14 @@ class TrackPanel:public wxPanel {
    void HandleRearrange(wxMouseEvent & event);
    void CalculateRearrangingThresholds(wxMouseEvent & event);
    void HandleClosing(wxMouseEvent & event);
+   void HandlePopping(wxMouseEvent & event);
    void HandleMutingSoloing(wxMouseEvent & event, bool solo);
    void HandleMinimizing(wxMouseEvent & event);
    void HandleSliders(wxMouseEvent &event, bool pan);
    bool MuteSoloFunc(Track *t, wxRect r, int x, int f, bool solo);
    bool MinimizeFunc(Track *t, wxRect r, int x, int f);
+   bool CloseFunc(Track * t, wxRect r, int x, int y);
+   bool PopupFunc(Track * t, wxRect r, int x, int y);
    bool GainFunc(Track * t, wxRect r, wxMouseEvent &event,
                  int index, int x, int y);
    bool PanFunc(Track * t, wxRect r, wxMouseEvent &event,
@@ -344,9 +359,6 @@ class TrackPanel:public wxPanel {
    int GetLabelWidth() const { return mTrackLabel.GetTitleWidth() + GetVRulerWidth();}
 
 private:
-   void DrawRuler(wxDC * dc, bool text = true);
-   void DrawTrackIndicator(wxDC *dc);
-
    void DrawTracks(wxDC * dc);
 
    void DrawEverythingElse(wxDC *dc, const wxRect panelRect, const wxRect clip);
@@ -380,6 +392,8 @@ private:
 
    AdornedRulerPanel *mRuler;
 
+   double mSeekShort;
+   double mSeekLong;
 
    TrackArtist *mTrackArtist;
 
@@ -390,24 +404,25 @@ private:
    } mTimer;
    
 
-   //This stores the parts of the screen that get overwritten by the indicator
-   tpBitmapArray mScreenAtIndicator;
-   
-   // This indicates whether the last indicator drawing
-   // existed, so that we can draw over it to erase it
-   bool mPlayIndicatorExists;
-
-   tpBitmapArray mPreviousCursorData;
+   // This stores the parts of the screen that get overwritten by the indicator
+   // and cursor
+   double mLastIndicator;
+   double mLastCursor;
 
    int mTimeCount;
 
-   wxBitmap *mBitmap;
+   wxMemoryDC mBackingDC;
+   wxBitmap *mBacking;
+   bool mRefreshBacking;
    int mPrevWidth;
    int mPrevHeight;
 
+   wxLongLong mLastSelectionAdjustment;
+
    double mSelStart;
 
    Track *mCapturedTrack;
+   Envelope *mCapturedEnvelope;
    WaveClip *mCapturedClip;
    TrackClipArray mCapturedClipArray;
    bool mCapturedClipIsSelection;
@@ -484,7 +499,8 @@ private:
       IsPanSliding,
       IsMinimizing,
       IsOverCutLine,
-      WasOverCutLine
+      WasOverCutLine,
+      IsPopping
    };
 
    enum MouseCaptureEnum mMouseCapture;
@@ -492,6 +508,9 @@ private:
 
    bool mAdjustSelectionEdges;
    bool mSlideUpDownOnly;
+   bool mCircularTrackNavigation;
+   bool mPresentTrackNumber;
+   float mdBr;
 
    // JH: if the user is dragging a track, at what y
    //   coordinate should the dragging track move up or down?
@@ -526,6 +545,10 @@ private:
 
    Track *mPopupMenuTarget;
 
+   friend class TrackPanelAx;
+
+   TrackPanelAx *mAx;
+
  public:
 
    DECLARE_EVENT_TABLE()
