@@ -1,3 +1,31 @@
+/**********************************************************************
+
+  Audacity: A Digital Audio Editor
+
+  WaveClip.cpp
+
+  ?? Dominic Mazzoni
+  ?? Markus Meyer
+
+*******************************************************************//**
+
+\class WaveClip
+\brief This allows multiple clips to be a part of one WaveTrack.
+
+*//****************************************************************//**
+
+\class WaveCache
+\brief Cache used with WaveClip to cache wave information (for drawing).
+
+*//****************************************************************//**
+
+\class SpecCache
+\brief Cache used with WaveClip to cache spectrum information (for 
+drawing).
+
+*//*******************************************************************/
+
+
 #include <math.h>
 #include <wx/log.h>
 
@@ -5,7 +33,8 @@
 #include "Prefs.h"
 #include "WaveClip.h"
 #include "Envelope.h"
-
+#include "Resample.h"
+#include "Project.h"
 
 #include <wx/listimpl.cpp>
 WX_DEFINE_LIST(WaveClipList);
@@ -533,7 +562,8 @@ void WaveClip::GetDisplayRect(wxRect* r)
 }
 
 bool WaveClip::Append(samplePtr buffer, sampleFormat format,
-                      sampleCount len, unsigned int stride /* = 1 */)
+                      sampleCount len, unsigned int stride /* = 1 */,
+                      XMLWriter* blockFileLog /*=NULL*/)
 {
    //wxLogDebug(wxT("Append: len=%i\n"), len);
    
@@ -547,7 +577,8 @@ bool WaveClip::Append(samplePtr buffer, sampleFormat format,
    for(;;) {
       if (mAppendBufferLen >= blockSize) {
          bool success =
-            mSequence->Append(mAppendBuffer, seqFormat, blockSize);
+            mSequence->Append(mAppendBuffer, seqFormat, blockSize,
+                              blockFileLog);
          if (!success)
             return false;
          memmove(mAppendBuffer,
@@ -662,25 +693,18 @@ XMLTagHandler *WaveClip::HandleXMLChild(const wxChar *tag)
       return NULL;
 }
 
-void WaveClip::WriteXML(int depth, FILE *fp)
+void WaveClip::WriteXML(XMLWriter &xmlFile)
 {
-   int i;
+   xmlFile.StartTag(wxT("waveclip"));
+   xmlFile.WriteAttr(wxT("offset"), mOffset, 8);
 
-   for(i=0; i<depth; i++)
-      fprintf(fp, "\t");
-   fprintf(fp, "<waveclip ");
-   fprintf(fp, "offset=\"%s\" ", (const char *)Internat::ToString(mOffset, 8).mb_str());
-   fprintf(fp, ">\n");
-
-   mSequence->WriteXML(depth+1, fp);
-   mEnvelope->WriteXML(depth+1, fp);
+   mSequence->WriteXML(xmlFile);
+   mEnvelope->WriteXML(xmlFile);
 
    for (WaveClipList::Node* it=mCutLines.GetFirst(); it; it=it->GetNext())
-      it->GetData()->WriteXML(depth+1, fp);
+      it->GetData()->WriteXML(xmlFile);
 
-   for(i=0; i<depth; i++)
-      fprintf(fp, "\t");
-   fprintf(fp, "</waveclip>\n");
+   xmlFile.EndTag(wxT("waveclip"));
 }
 
 bool WaveClip::CreateFromCopy(double t0, double t1, WaveClip* other)
@@ -710,8 +734,29 @@ bool WaveClip::CreateFromCopy(double t0, double t1, WaveClip* other)
 
 bool WaveClip::Paste(double t0, WaveClip* other)
 {
+   WaveClip* resampledClip;
+
+   bool clipNeedsResampling = other->mRate != mRate;
+
+   if (clipNeedsResampling)
+   {
+      // The other clip's rate is different to our's, so resample
+      resampledClip = new WaveClip(*other, mSequence->GetDirManager());
+      if (!resampledClip->Resample(mRate))
+      {
+         delete resampledClip;
+         return false;
+      }
+   } else
+   {
+      // No resampling needed, just use original clip without making a copy
+      resampledClip = other;
+   }
+
    longSampleCount s0;
    TimeToSamplesClip(t0, &s0);
+   
+   bool result = false;
 
    if (mSequence->Paste(s0, other->mSequence))
    {
@@ -729,10 +774,16 @@ bool WaveClip::Paste(double t0, WaveClip* other)
          mCutLines.Append(newCutLine);
       }
       
-      return true;
+      result = true;
+   }
+   
+   if (clipNeedsResampling)
+   {
+      // Clip was constructed as a copy, so delete it
+      delete resampledClip;
    }
 
-   return false;
+   return result;
 }
 
 bool WaveClip::InsertSilence(double t, double len)
@@ -756,12 +807,51 @@ bool WaveClip::InsertSilence(double t, double len)
 bool WaveClip::Clear(double t0, double t1)
 {
    longSampleCount s0, s1;
-   
+
    TimeToSamplesClip(t0, &s0);
    TimeToSamplesClip(t1, &s1);
    
    if (GetSequence()->Delete(s0, s1-s0))
    {
+      // msmeyer
+      //
+      // Delete all cutlines that are within the given area, if any.
+      //
+      // Note that when cutlines are active, two functions are used:
+      // Clear() and ClearAndAddCutLine(). ClearAndAddCutLine() is called
+      // whenever the user directly calls a command that removes some audio, e.g.
+      // "Cut" or "Clear" from the menu. This command takes care about recursive
+      // preserving of cutlines within clips. Clear() is called when internal
+      // operations want to remove audio. In the latter case, it is the right
+      // thing to just remove all cutlines within the area.
+      //
+      double clip_t0 = t0;
+      double clip_t1 = t1;
+      if (clip_t0 < GetStartTime())
+         clip_t0 = GetStartTime();
+      if (clip_t1 > GetEndTime())
+         clip_t1 = GetEndTime();
+
+      WaveClipList::Node* nextIt = (WaveClipList::Node*)-1;
+
+      for (WaveClipList::Node* it = mCutLines.GetFirst(); it; it=nextIt)
+      {
+         nextIt = it->GetNext();
+         WaveClip* clip = it->GetData();
+         double cutlinePosition = mOffset + clip->GetOffset();
+         if (cutlinePosition >= t0 && cutlinePosition <= t1)
+         {
+            // This cutline is within the area, delete it
+            delete clip;
+            mCutLines.DeleteNode(it);
+         } else
+         if (cutlinePosition >= t1)
+         {
+            clip->Offset(clip_t0-clip_t1);
+         }
+      }
+
+      // Collapse envelope
       GetEnvelope()->CollapseRegion(t0, t1);
       if (t0 < GetStartTime())
          Offset(-(GetStartTime() - t0));
@@ -811,15 +901,29 @@ bool WaveClip::ClearAndAddCutLine(double t0, double t1)
          clip->Offset(clip_t0-clip_t1);
       }
    }
+   
+   // Clear actual audio data
+   longSampleCount s0, s1;
 
-   if (Clear(t0, t1))
+   TimeToSamplesClip(t0, &s0);
+   TimeToSamplesClip(t1, &s1);
+   
+   if (GetSequence()->Delete(s0, s1-s0))
    {
+      // Collapse envelope
+      GetEnvelope()->CollapseRegion(t0, t1);
+      if (t0 < GetStartTime())
+         Offset(-(GetStartTime() - t0));
+
+      MarkChanged();
+
       mCutLines.Append(newClip);
       return true;
+   } else
+   {
+      delete newClip;
+      return false;
    }
-
-   delete newClip;
-   return false;
 }
 
 bool WaveClip::ExpandCutLine(double cutLinePosition, double* cutlineStart, double* cutlineEnd)
@@ -878,3 +982,103 @@ void WaveClip::OffsetCutLines(double t0, double len)
          cutLine->Offset(len);
    }
 }
+
+void WaveClip::Lock()
+{
+   GetSequence()->Lock();
+   for (WaveClipList::Node* it = mCutLines.GetFirst(); it; it=it->GetNext())
+      it->GetData()->Lock();
+}
+
+void WaveClip::Unlock()
+{
+   GetSequence()->Unlock();
+   for (WaveClipList::Node* it = mCutLines.GetFirst(); it; it=it->GetNext())
+      it->GetData()->Unlock();
+}
+
+bool WaveClip::Resample(int rate, bool progress)
+{
+   if (rate == mRate)
+      return true; // Nothing to do
+
+   double factor = (double)rate / (double)mRate;
+   ::Resample* resample = new ::Resample(true, factor, factor);
+   
+   int bufsize = 65536;
+   float* inBuffer = new float[bufsize];
+   float* outBuffer = new float[bufsize];
+   int pos = 0;
+   bool error = false;
+   sampleCount numSamples = mSequence->GetNumSamples();
+
+   Sequence* newSequence =
+      new Sequence(mSequence->GetDirManager(), mSequence->GetSampleFormat());
+   
+   while (pos < numSamples)
+   {
+      int inLen = numSamples - pos;
+      if (inLen > bufsize)
+         inLen = bufsize;
+         
+      bool isLast = ((pos + inLen) == numSamples);
+      
+      if (!mSequence->Get((samplePtr)inBuffer, floatSample, pos, inLen))
+      {
+         error = true;
+         break;
+      }
+      
+      int inBufferUsed = 0;
+      int outGenerated = resample->Process(factor, inBuffer, inLen, isLast,
+                                           &inBufferUsed, outBuffer, bufsize);
+                                           
+      pos += inBufferUsed;
+      
+      if (outGenerated < 0)
+      {
+         error = true;
+         break;
+      }
+      
+      if (!newSequence->Append((samplePtr)outBuffer, floatSample,
+                               outGenerated))
+      {
+         error = true;
+         break;
+      }
+
+      if (progress)
+      {
+         error = !GetActiveProject()->ProgressUpdate((int) (1000 * ((float)pos / numSamples)));
+         if (error)
+         {
+            break;
+         }
+      }
+   }
+   
+   delete[] inBuffer;
+   delete[] outBuffer;
+   delete resample;
+   
+   if (error)
+   {
+      delete newSequence;
+   } else
+   {
+      delete mSequence;
+      mSequence = newSequence;
+      mRate = rate;
+
+      // Invalidate wave display cache
+      if (mWaveCache)
+      {
+         delete mWaveCache;
+         mWaveCache = NULL;
+      }
+      mWaveCache = new WaveCache(1);
+   }
+
+   return !error;
+}
