@@ -7,9 +7,73 @@
   Brian Gunlogson
   Dominic Mazzoni
 
-  See CommandManager.h for an overview of this class.
+*******************************************************************//**
 
-**********************************************************************/
+\class CommandManager
+\brief CommandManager implements a system for organizing all user-callable
+  commands.  
+  
+  It creates and manages a menu bar with a command
+  associated with each item, and managing other commands callable
+  by keyboard shortcuts.
+
+  Commands are implemented by overriding an abstract functor class.
+  See Menus.cpp for an example use.
+
+  Menus or submenus containing lists of items can be added at once,
+  with a single function (functor) to be called when any of the
+  items is selected, with the index number of the selection as the
+  parameter.  This is useful for dynamic menus (effects) and
+  submenus containing a list of choices (selection formats).
+
+  Menu items can be enabled or disabled individually, groups of
+  "multi-items" can be enabled or disabled all at once, or entire
+  sets of commands can be enabled or disabled all at once using
+  flags.  The flags should be a bitfield stored in a 32-bit
+  integer but can be whatever you want.  You specify both the
+  desired values of the flags, and the set of flags relevant to
+  a particular command, by using a combination of a flags parameter
+  and a mask parameter.  Any flag set to 0 in the mask parameter is
+  the same as "don't care".  Any command whose mask is set to zero
+  will not be affected by enabling/disabling by flags.
+
+*//****************************************************************//**
+
+\class CommandFunctor
+\brief CommandFunctor is a very small class that works with 
+CommandManager.  It holds the callback for one command.
+
+*//****************************************************************//**
+
+\class MenuBarListEntry
+\brief MenuBarListEntry is a structure used by CommandManager.
+
+*//****************************************************************//**
+
+\class SubMenuListEntry
+\brief SubMenuListEntry is a structure used by CommandManager.
+
+*//****************************************************************//**
+
+\class CommandListEntry
+\brief CommandListEntry is a structure used by CommandManager.
+
+*//****************************************************************//**
+
+\class MenuBarList
+\brief List of MenuBarListEntry.
+
+*//****************************************************************//**
+
+\class SubMenuList
+\brief List of SubMenuListEntry.
+
+*//****************************************************************//**
+
+\class CommandList
+\brief List of CommandListEntry.
+
+*//******************************************************************/
 
 #include "../Audacity.h"
 
@@ -46,6 +110,7 @@ CommandManager::CommandManager():
    mCurrentID(0),
    mHiddenID(0),
    mCurrentMenu(NULL),
+   mOpenMenu(NULL),
    mDefaultFlags(0),
    mDefaultMask(0)
 {
@@ -280,7 +345,6 @@ void CommandManager::AddItem(wxString name, wxString label,
    
    if (shortcut) {
       dummy = dummy + wxT("\t") + mCommandIDHash[ID]->key;
-      newLabel = newLabel + wxT("\t") + mCommandIDHash[ID]->key;
    }
 
    CurrentMenu()->Append(ID, dummy);
@@ -377,6 +441,10 @@ int CommandManager::NewIdentifier(wxString name, wxString label, wxMenu *menu,
 {
    CommandListEntry *tmpEntry = new CommandListEntry;
    
+   wxString labelPrefix;
+   if( !mSubMenuList.IsEmpty() )
+      labelPrefix = mSubMenuList[ mSubMenuList.GetCount() - 1 ]->name;
+   
    // wxMac 2.5 and higher will do special things with the
    // Preferences, Exit (Quit), and About menu items,
    // if we give them the right IDs.
@@ -397,9 +465,10 @@ int CommandManager::NewIdentifier(wxString name, wxString label, wxMenu *menu,
    else if (name == wxT("About"))
       tmpEntry->id = wxID_ABOUT;
   #endif
-
+  
    tmpEntry->name = name;
    tmpEntry->label = label;
+   tmpEntry->labelPrefix = labelPrefix;
    tmpEntry->menu = menu;
    tmpEntry->callback = callback;
    tmpEntry->multi = multi;
@@ -514,13 +583,109 @@ void CommandManager::Modify(wxString name, wxString newLabel)
    CommandListEntry *entry = mCommandNameHash[name];
    if (entry && entry->menu) {
       newLabel = newLabel.BeforeFirst(wxT('\t'));
-      if (entry->key)
-        newLabel = newLabel + wxT("\t") + entry->key;
+      if (!entry->key.IsEmpty())
+         newLabel = newLabel + wxT("\t") + entry->key;
+      entry->label = newLabel;
       entry->menu->SetLabel(entry->id, newLabel);
    }
 }
 
+void CommandManager::HandleMenuOpen(wxMenuEvent &evt)
+{
+   // Ensure we have a menu and that it's a top-level menu.
+   wxMenu *m = evt.GetMenu();
+   if (!m || m->GetParent())
+      return;
 
+   // Windows does not send a CLOSE event if you move from one 
+   // top-level menu to another, so simulate it.
+   if (mOpenMenu) {
+      wxMenuEvent dummy;
+      HandleMenuClose(dummy);
+   }
+
+   // Remember this menu
+   mOpenMenu = m;
+
+   // Turn on the accelerators
+   ToggleAccels(m, true);
+
+   return;
+}
+
+void CommandManager::HandleMenuClose(wxMenuEvent &evt)
+{
+   // This can happen when if the Windows system menu is used   
+   if (mOpenMenu == NULL)
+      return;
+
+   // GetMenu() under Windows will always return NULL.  And on other
+   // platforms we must ensure we are a top-level menu.
+   wxMenu *m = evt.GetMenu();
+   if (m && m->GetParent())
+      return;
+
+   // Turn off the accelerators
+   ToggleAccels(mOpenMenu, false);
+
+   // Forget about it
+   mOpenMenu = NULL;
+
+   return;
+}
+ 
+void CommandManager::ToggleAccels(wxMenu *m, bool show)
+{
+   // Add the top-level menu to the stack;
+   wxArrayPtrVoid stack;
+   stack.Add(m);
+
+   // Process all sub-menus in this tree
+   while (!stack.IsEmpty()) {
+
+      // Pop the bottom entry
+      m = (wxMenu *) stack.Item(0);
+      stack.RemoveAt(0);
+
+      // Retrieve menuitem info for this menu
+      wxMenuItemList mil = m->GetMenuItems();
+      int iCnt = m->GetMenuItemCount();
+      int iNdx;
+
+      // Iterate all menuitems at this level
+      for (iNdx = 0; iNdx < iCnt; iNdx++) {
+
+         // Retrieve the menuitem
+         wxMenuItem *mi = mil.Item(iNdx)->GetData();
+         if (!mi)
+            continue;
+
+         // Stack the menu if this item represents a submenu
+         if (mi->IsSubMenu()) {
+            stack.Add(mi->GetSubMenu());
+            continue;
+         }
+
+         // Retrieve the command entry for this item
+         CommandListEntry *entry = mCommandIDHash[mi->GetId()];
+         if (!entry)
+            continue;
+
+         // Rebuild the label based on whether the accelerator should
+         // be shown.
+         wxString label = entry->label.BeforeFirst(wxT('\t'));
+         if (show && !entry->key.IsEmpty()) {
+            label = label + wxT("\t") + entry->key;
+         }
+
+         // Set the new label
+         mi->SetText( label );
+      }
+   }
+
+   return;
+}
+ 
 /// HandleCommandEntry() takes a CommandListEntry and executes it
 /// returning true iff successful.  If you pass any flags,
 ///the command won't be executed unless the flags are compatible
@@ -535,9 +700,9 @@ bool CommandManager::HandleCommandEntry(CommandListEntry * entry, wxUint32 flags
       bool allowed = ((flags & combinedMask) ==
                       (entry->flags & combinedMask));
       if (!allowed)
-         return true;
+         return false;
    }
-   
+
    (*(entry->callback))(entry->index);
 
    return true;
@@ -580,7 +745,7 @@ bool CommandManager::HandleTextualCommand(wxString & Str, wxUint32 flags, wxUint
          if( Str.IsSameAs( mCommandList[i]->name ))
          {
             return HandleCommandEntry( mCommandList[i], flags, mask);
-   }
+         }
       }
    }
    // Not one of the singleton commands.
@@ -624,6 +789,15 @@ wxString CommandManager::GetLabelFromName(wxString name)
    return entry->label;
 }
 
+wxString CommandManager::GetPrefixedLabelFromName(wxString name)
+{
+   CommandListEntry *entry = mCommandNameHash[name];
+   if (!entry)
+      return wxT("");
+
+   return entry->labelPrefix + entry->label;
+}
+
 wxString CommandManager::GetKeyFromName(wxString name)
 {
    CommandListEntry *entry = mCommandNameHash[name];
@@ -689,31 +863,28 @@ XMLTagHandler *CommandManager::HandleXMLChild(const wxChar *tag)
    return this;
 }
 
-void CommandManager::WriteXML(int depth, FILE *fp)
+void CommandManager::WriteXML(XMLWriter &xmlFile)
 {
-   int i;
    unsigned int j;
 
-   for(i=0; i<depth; i++)
-      fprintf(fp, "\t");
-   fprintf(fp, "<audacitykeyboard audacityversion=\"%s\">\n",
-           AUDACITY_VERSION_STRING);
-   for(j=0; j<mCommandList.GetCount(); j++)
+   xmlFile.StartTag(wxT("audacitykeyboard"));
+   xmlFile.WriteAttr(wxT("audacityversion"), wxT(AUDACITY_VERSION_STRING));
+
+   for(j=0; j<mCommandList.GetCount(); j++) {
       if (!mCommandList[j]->multi) {
-         for(i=0; i<depth+1; i++)
-            fprintf(fp, "\t");
          
          wxString label = mCommandList[j]->label;
          label = wxMenuItem::GetLabelFromText(label.BeforeFirst(wxT('\t')));
          
-         fprintf(fp, "<command name=\"%s\" label=\"%s\" key=\"%s\" />\n",
-                 (const char *)XMLEsc(mCommandList[j]->name).mb_str(),
-                 (const char *)XMLEsc(label).mb_str(),
-                 (const char *)XMLEsc(mCommandList[j]->key).mb_str());
+         xmlFile.StartTag(wxT("command"));
+         xmlFile.WriteAttr(wxT("name"), mCommandList[j]->name);
+         xmlFile.WriteAttr(wxT("label"), label);
+         xmlFile.WriteAttr(wxT("key"), mCommandList[j]->key);
+         xmlFile.EndTag(wxT("command"));
       }
-   for(i=0; i<depth; i++)
-      fprintf(fp, "\t");
-   fprintf(fp, "</audacitykeyboard>\n");
+   }
+   
+   xmlFile.EndTag(wxT("audacitykeyboard"));
 }
 
 void CommandManager::SetDefaultFlags(wxUint32 flags, wxUint32 mask)
