@@ -6,8 +6,23 @@
 
   Vincent A. Busam
 
-**********************************************************************/
+*******************************************************************//**
 
+\class EffectAvcCompressor
+\brief An unused Effect (we use EffectCompressor instead) derived 
+from EffectSimplePairedTwoTrackBase
+
+*//****************************************************************//**
+
+\class iAVCBufferList
+\brief An unused class used by EffectAvcCompressor.
+
+*//****************************************************************//**
+
+\class AvcCompressorDialog
+\brief An unused class used by unused effect, EffectAvcCompressor.
+
+*//*******************************************************************/
 /*  TODO List:
 
   1.  Add graph shows curve specified by grid, keep it up to date,
@@ -723,7 +738,6 @@ wxSizer *AvcCompressorDialog::MakeAvcCompressorDialog(wxWindow * parent, bool ca
        new wxButton(parent, wxID_OK, _("OK"), wxDefaultPosition,
                     wxDefaultSize, 0);
 	button->SetDefault();
-	button->SetFocus();
 	boxSizer->Add(button, 0, wxALIGN_CENTRE | wxALL, 5);
 
 	mainSizer->Add(boxSizer, 0, wxALIGN_CENTRE | wxALL, 5);
