@@ -92,12 +92,11 @@ VSTEffectGUIDialog::VSTEffectGUIDialog(wxWindow *parent, wxWindowID id,
 
    hSizer->Add(20, 10); // horizontal spacer
 
-   wxButton *cancel = new wxButton(this, wxID_CANCEL, _("Cancel"));
+   wxButton *cancel = new wxButton(this, wxID_CANCEL, _("&Cancel"));
    hSizer->Add(cancel, 0, wxALIGN_CENTRE|wxALL, 5);
 
-   wxButton *ok = new wxButton(this, wxID_OK, _("OK"));
+   wxButton *ok = new wxButton(this, wxID_OK, _("&OK"));
    ok->SetDefault();
-   ok->SetFocus();
    hSizer->Add(ok, 0, wxALIGN_CENTRE|wxALL, 5);
 
    mainSizer->Add(hSizer, 0, wxALIGN_CENTRE|wxALIGN_CENTER_VERTICAL|wxALL, 5);
