@@ -8,12 +8,27 @@
   
   This class implements a Ladspa Plug-in effect.
 
-**********************************************************************/
+*******************************************************************//**
+
+\class LadspaEffect
+\brief An Effect that calls up a LADSPA plug in, i.e. many possible 
+effects from this one class.
+
+*//****************************************************************//**
+
+\class LadspaEffectDialog
+\brief Dialog used with Effect
+
+*//*******************************************************************/
+
+
+#include "../Audacity.h"
 
 #include "ladspa.h"
 
 #include <wx/wxprec.h>
 #include <wx/button.h>
+#include <wx/checkbox.h>
 #include <wx/msgdlg.h>
 #include <wx/sizer.h>
 #include <wx/slider.h>
@@ -21,6 +36,7 @@
 #include <wx/stattext.h>
 #include <wx/textctrl.h>
 #include <wx/intl.h>
+#include <wx/scrolwin.h>
 
 #include "../Effect.h"          // Audacity Effect base class
 #include "LadspaEffect.h"       // This class's header file
@@ -31,13 +47,13 @@ LadspaEffect::LadspaEffect(const LADSPA_Descriptor *data)
    mData = data;
    pluginName = LAT1CTOWX(data->Name);
 
-   buffer = NULL;
    fInBuffer = NULL;
    fOutBuffer = NULL;
 
    inputs = 0;
    outputs = 0;
    numInputControls = 0;
+   mLength = 0;
 
    unsigned long p;
 
@@ -108,7 +124,6 @@ LadspaEffect::LadspaEffect(const LADSPA_Descriptor *data)
    }
 
    flags = PLUGIN_EFFECT;
-
    if (inputs == 0)
       flags |= INSERT_EFFECT;
    else if (outputs == 0)
@@ -174,14 +189,17 @@ bool LadspaEffect::Init()
 bool LadspaEffect::PromptUser()
 {
    if (numInputControls > 0) {
-      LadspaEffectDialog dlog(this, mParent, mData, inputControls, mainRate);
+      double length = mT1 > mT0 ? mT1 - mT0 : sDefaultGenerateLen;
+
+      LadspaEffectDialog dlog(this, mParent, mData, inputControls, mainRate, length);
       dlog.CentreOnParent();
       dlog.ShowModal();
       
       if (!dlog.GetReturnCode())
          return false;
-   }
 
+      mLength = dlog.GetLength();
+   }
    return true;
 }
 
@@ -193,7 +211,14 @@ void LadspaEffect::GetSamples(WaveTrack *track,
    double trackEnd = track->GetEndTime();
    double t0 = mT0 < trackStart? trackStart: mT0;
    double t1 = mT1 > trackEnd? trackEnd: mT1;
-   
+
+   if (flags & INSERT_EFFECT) {
+      t1 = t0 + mLength;
+      if (mT0 == mT1) {
+         track->InsertSilence(t0, t1);
+      }
+   }
+
    if (t1 > t0) {
       *start = track->TimeToLongSamples(t0);
       longSampleCount end = track->TimeToLongSamples(t1);
@@ -255,7 +280,6 @@ bool LadspaEffect::ProcessStereo(int count, WaveTrack *left, WaveTrack *right,
    if (mBlockSize == 0) {
       mBlockSize = left->GetMaxBlockSize() * 2;
 
-      buffer = new float[mBlockSize];
       fInBuffer = new float *[inputs];
       unsigned long i;
       for (i = 0; i < inputs; i++)
@@ -299,33 +323,24 @@ bool LadspaEffect::ProcessStereo(int count, WaveTrack *left, WaveTrack *right,
    longSampleCount rs = rstart;
    while (len) {
       int block = mBlockSize;
-      int i;
       if (block > len)
          block = len;
 
       if (left && inputs > 0) {
-         left->Get((samplePtr)buffer, floatSample, ls, block);
-         for (i = 0; i < block; i++)
-            fInBuffer[0][i] = buffer[i];
+         left->Get((samplePtr)fInBuffer[0], floatSample, ls, block);
       }
       if (right && inputs > 1) {
-         right->Get((samplePtr)buffer, floatSample, rs, block);
-         for (i = 0; i < block; i++)
-            fInBuffer[1][i] = buffer[i];
+         right->Get((samplePtr)fInBuffer[1], floatSample, rs, block);
       }
 
       mData->run(handle, block);
 
       if (left && outputs > 0) {
-         for (i = 0; i < block; i++)
-            buffer[i] = fOutBuffer[0][i];
-         left->Set((samplePtr)buffer, floatSample, ls, block);
+         left->Set((samplePtr)fOutBuffer[0], floatSample, ls, block);
       }      
       
       if (right && outputs > 1) {
-         for (i = 0; i < block; i++)
-            buffer[i] = fOutBuffer[1][i];
-         right->Set((samplePtr)buffer, floatSample, rs, block);
+         right->Set((samplePtr)fOutBuffer[1], floatSample, rs, block);
       }      
 
       len -= block;
@@ -353,35 +368,140 @@ bool LadspaEffect::ProcessStereo(int count, WaveTrack *left, WaveTrack *right,
 
 void LadspaEffect::End()
 {
-   if (buffer) {
-      unsigned long i;
+   unsigned long i;
 
-      delete[]buffer;
+   if (fInBuffer) {
       for (i = 0; i < inputs; i++) {
-         delete fInBuffer[i];
+         if (fInBuffer[i]) {
+            delete [] fInBuffer[i];
+         }
       }
+      delete [] fInBuffer;
+      fInBuffer = NULL;
+   }
+
+   if (fOutBuffer) {
       for (i = 0; i < outputs; i++) {
-         delete fOutBuffer[i];
+         if (fOutBuffer[i]) {
+            delete [] fOutBuffer[i];
+         }
       }
-
-      delete[] fInBuffer;
-      delete[] fOutBuffer;
+      delete [] fOutBuffer;
+      fOutBuffer = NULL;
    }
-   buffer = NULL;
-   fInBuffer = NULL;
-   fOutBuffer = NULL;
 }
 
-const int LADSPA_SLIDER_ID = 13100;
-const int LADSPA_TEXTCTRL_ID = 13101;
-const int LADSPA_PREVIEW_ID = 13102;
+class Slider:public wxSlider
+{
+ public:
+   Slider(wxWindow *parent, wxWindowID id,
+          int value, int minValue, int maxValue,
+          const wxPoint& pos = wxDefaultPosition,
+          const wxSize& size = wxDefaultSize,
+          long style = wxSL_HORIZONTAL,
+          const wxValidator& validator = wxDefaultValidator,
+          const wxString& name = wxSliderNameStr)
+   : wxSlider(parent, id, value, minValue, maxValue,
+              pos, size, style, validator, name)
+   {
+   };
+
+   void OnSetFocus(wxFocusEvent &event)
+   {
+      wxScrolledWindow *p = (wxScrolledWindow *) GetParent();
+      wxRect r = GetRect();
+      wxRect rv = p->GetRect();
+      rv.y = 0;
+
+      event.Skip();
+
+      int y;
+      int yppu;
+      p->GetScrollPixelsPerUnit(NULL, &yppu);
+
+      if (r.y >= rv.y && r.GetBottom() <= rv.GetBottom()) {
+         return;
+      }
+
+      if (r.y < rv.y) {
+         p->CalcUnscrolledPosition(0, r.y, NULL, &r.y);
+         y = r.y / yppu;
+      }
+      else {
+         p->CalcUnscrolledPosition(0, r.y, NULL, &r.y);
+         y = (r.GetBottom() - rv.GetBottom() + yppu) / yppu;
+      }
+
+      p->Scroll(-1, y);
+   };
+
+   DECLARE_EVENT_TABLE();
+};
+
+BEGIN_EVENT_TABLE(Slider, wxSlider)
+    EVT_SET_FOCUS(Slider::OnSetFocus)
+END_EVENT_TABLE()
+
+class TextCtrl:public wxTextCtrl
+{
+ public:
+   TextCtrl(wxWindow *parent, wxWindowID id,
+            const wxString& value = wxEmptyString,
+            const wxPoint& pos = wxDefaultPosition,
+            const wxSize& size = wxDefaultSize, long style = 0,
+            const wxValidator& validator = wxDefaultValidator,
+            const wxString& name = wxTextCtrlNameStr)
+   : wxTextCtrl(parent, id, value,
+                pos, size, style, validator, name)
+   {
+   };
+
+   void OnSetFocus(wxFocusEvent &event)
+   {
+      wxScrolledWindow *p = (wxScrolledWindow *) GetParent();
+      wxRect r = GetRect();
+      wxRect rv = p->GetRect();
+      rv.y = 0;
+
+      event.Skip();
+
+      int y;
+      int yppu;
+      p->GetScrollPixelsPerUnit(NULL, &yppu);
+
+      if (r.y >= rv.y && r.GetBottom() <= rv.GetBottom()) {
+         return;
+      }
+
+      if (r.y < rv.y) {
+         p->CalcUnscrolledPosition(0, r.y, NULL, &r.y);
+         y = r.y / yppu;
+      }
+      else {
+         p->CalcUnscrolledPosition(0, r.y, NULL, &r.y);
+         y = (r.GetBottom() - rv.GetBottom() + yppu) / yppu;
+      }
+
+      p->Scroll(-1, y);
+   };
+
+   DECLARE_EVENT_TABLE();
+};
+
+BEGIN_EVENT_TABLE(TextCtrl, wxTextCtrl)
+    EVT_SET_FOCUS(TextCtrl::OnSetFocus)
+END_EVENT_TABLE()
+
+const int LADSPA_PREVIEW_ID = 13100;
+const int LADSPA_SECONDS_ID = 13101;
 
 BEGIN_EVENT_TABLE(LadspaEffectDialog, wxDialog)
     EVT_BUTTON(wxID_OK, LadspaEffectDialog::OnOK)
     EVT_BUTTON(wxID_CANCEL, LadspaEffectDialog::OnCancel)
     EVT_BUTTON(LADSPA_PREVIEW_ID, LadspaEffectDialog::OnPreview)
-    EVT_SLIDER(LADSPA_SLIDER_ID, LadspaEffectDialog::OnSlider)
-    EVT_TEXT(LADSPA_TEXTCTRL_ID, LadspaEffectDialog::OnTextCtrl)
+    EVT_SLIDER(wxID_ANY, LadspaEffectDialog::OnSlider)
+    EVT_TEXT(wxID_ANY, LadspaEffectDialog::OnTextCtrl)
+    EVT_CHECKBOX(wxID_ANY, LadspaEffectDialog::OnCheckBox)
 END_EVENT_TABLE()
 
 IMPLEMENT_CLASS(LadspaEffectDialog, wxDialog)
@@ -390,12 +510,14 @@ LadspaEffectDialog::LadspaEffectDialog(LadspaEffect *eff,
                                        wxWindow * parent,
                                        const LADSPA_Descriptor *data,
                                        float *inputControls,
-                                       int sampleRate)
+                                       int sampleRate,
+                                       double length)
    :wxDialog(parent, -1, LAT1CTOWX(data->Name),
              wxDefaultPosition, wxDefaultSize,
-             wxDEFAULT_DIALOG_STYLE),
+             wxDEFAULT_DIALOG_STYLE | wxRESIZE_BORDER),
     effect(eff)
 {
+   mLength = length;
    numParams = 0;
    this->mData = data;
    this->inputControls = inputControls;
@@ -411,8 +533,8 @@ LadspaEffectDialog::LadspaEffectDialog(LadspaEffect *eff,
 		inSlider = false;
 	#endif
    inText = false;
-   targetSlider = NULL;
 
+   toggles = new wxCheckBox*[mData->PortCount];
    sliders = new wxSlider*[mData->PortCount];
    fields = new wxTextCtrl*[mData->PortCount];
 	labels = new wxStaticText*[mData->PortCount];
@@ -428,15 +550,16 @@ LadspaEffectDialog::LadspaEffectDialog(LadspaEffect *eff,
       }
    }
 
-   wxBoxSizer *mainSizer = new wxBoxSizer(wxVERTICAL);
    wxControl *item;
 
+   wxBoxSizer *vSizer = new wxBoxSizer(wxVERTICAL);
+
    if (mData->Maker &&
        mData->Maker[0] && 
        LAT1CTOWX(mData->Maker) != wxString(_("None"))) {       
       item = new wxStaticText(this, 0,
                               wxString(_("Author: "))+LAT1CTOWX(mData->Maker));
-      mainSizer->Add(item, 0, wxALL, 5);
+      vSizer->Add(item, 0, wxALL, 5);
    }
    
    if (mData->Copyright &&
@@ -445,46 +568,16 @@ LadspaEffectDialog::LadspaEffectDialog(LadspaEffect *eff,
       
       item = new wxStaticText(this, 0,
                               LAT1CTOWX(mData->Copyright));
-      mainSizer->Add(item, 0, wxALL, 5);
+      vSizer->Add(item, 0, wxALL, 5);
    }
 
-   wxSizer *paramSizer =
-      new wxStaticBoxSizer(new wxStaticBox(this, -1,
-                                           _("Effect Settings")),
-                           wxVERTICAL );
-
-   wxFlexGridSizer *gridSizer =
-      new wxFlexGridSizer(3, 0, 0);
-
-   for (p = 0; p < numParams; p++) {
-
-      item = new wxStaticText(this, 0, wxString(mData->PortNames[ports[p]], wxConvISO8859_1));
-      gridSizer->Add(item, 0, wxALIGN_CENTER_VERTICAL | wxALL, 5);
-
-      wxString fieldText;
-      LADSPA_PortRangeHint hint = mData->PortRangeHints[ports[p]];
-      if (LADSPA_IS_HINT_INTEGER(hint.HintDescriptor))
-         fieldText.Printf(wxT("%d"), (int)(inputControls[ports[p]] + 0.5));
-      else
-         fieldText.Printf(wxT("%f"), inputControls[ports[p]]);
-      fields[p] = new wxTextCtrl(this, LADSPA_TEXTCTRL_ID, fieldText);
-      gridSizer->Add(fields[p], 0, wxALL, 5);
-
-      sliders[p] =
-          new wxSlider(this, LADSPA_SLIDER_ID,
-                       0, 0, 1000,
-                       wxDefaultPosition,
-                       wxSize(200, -1));
-      gridSizer->Add(sliders[p], 0, wxALL, 5);
-   }
-
-   // Set all of the sliders based on the value in the
-   // text fields
-	inSlider = false; // Now we're ready for HandleText to actually do something.
-   HandleText();
-   
-   paramSizer->Add(gridSizer, 1, wxALL, 5);
-   mainSizer->Add(paramSizer, 1, wxALL, 5);
+   wxScrolledWindow *w = new wxScrolledWindow(this,
+                                              wxID_ANY,
+                                              wxDefaultPosition,
+                                              wxDefaultSize,
+                                              wxVSCROLL | wxTAB_TRAVERSAL);
+   w->SetScrollRate(0, 20);
+   vSizer->Add(w, 1, wxEXPAND|wxALL, 5);
 
    wxBoxSizer *okSizer = new wxBoxSizer(wxHORIZONTAL);
 
@@ -493,20 +586,130 @@ LadspaEffectDialog::LadspaEffectDialog(LadspaEffect *eff,
    button = new wxButton(this, LADSPA_PREVIEW_ID, effect->GetPreviewName());
    okSizer->Add(button, 0, wxALIGN_CENTRE | wxALL, 5);
 
-   button = new wxButton(this, wxID_CANCEL, _("Cancel"));
+   button = new wxButton(this, wxID_CANCEL, _("&Cancel"));
    okSizer->Add(button, 0, wxALIGN_CENTRE | wxALL, 5);
 
-   button = new wxButton(this, wxID_OK, _("OK"));
+   button = new wxButton(this, wxID_OK, _("&OK"));
    button->SetDefault();
-   button->SetFocus();
    okSizer->Add(button, 0, wxALIGN_CENTRE | wxALL, 5);
 
-   mainSizer->Add(okSizer, 0, wxALIGN_CENTRE | wxALL, 5);
+   vSizer->Add(okSizer, 0, wxALIGN_CENTRE | wxALL, 5);
 
-   SetAutoLayout(TRUE);
-   SetSizer(mainSizer);
-   mainSizer->Fit(this);
-   mainSizer->SetSizeHints(this);
+   SetSizer(vSizer);
+
+   wxSizer *paramSizer =
+      new wxStaticBoxSizer(wxVERTICAL, w, _("Effect Settings"));
+
+   wxFlexGridSizer *gridSizer =
+      new wxFlexGridSizer(5, 0, 0);
+   gridSizer->AddGrowableCol(3);
+
+   for (p = 0; p < numParams; p++) {
+      item = new wxStaticText(w, 0, wxString(mData->PortNames[ports[p]], wxConvISO8859_1));
+      gridSizer->Add(item, 0, wxALIGN_CENTER_VERTICAL | wxALL, 5);
+
+      wxString fieldText;
+      LADSPA_PortRangeHint hint = mData->PortRangeHints[ports[p]];
+
+      if (LADSPA_IS_HINT_TOGGLED(hint.HintDescriptor)) {
+         toggles[p] = new wxCheckBox(w, p, wxT(""));
+         toggles[p]->SetValue(inputControls[ports[p]] > 0);
+         gridSizer->Add(toggles[p], 0, wxALL, 5);
+         ConnectFocus(toggles[p]);
+
+         gridSizer->Add(1, 1, 0);
+         gridSizer->Add(1, 1, 0);
+         gridSizer->Add(1, 1, 0);
+      }
+      else {
+         if (LADSPA_IS_HINT_INTEGER(hint.HintDescriptor))
+            fieldText.Printf(wxT("%d"), (int)(inputControls[ports[p]] + 0.5));
+         else
+            fieldText = Internat::ToDisplayString(inputControls[ports[p]]);
+
+         fields[p] = new wxTextCtrl(w, p, fieldText);
+         gridSizer->Add(fields[p], 0, wxALIGN_CENTER_VERTICAL | wxALL, 5);
+         ConnectFocus(fields[p]);
+
+         wxString bound;
+         double lower = 0.0;
+         double upper = 0.0;
+         bool haslo = false;
+         bool hashi = false;
+         bool forceint = false;
+
+         if (LADSPA_IS_HINT_BOUNDED_BELOW(hint.HintDescriptor)) {
+            lower = hint.LowerBound;
+            haslo = true;
+         }
+         if (LADSPA_IS_HINT_BOUNDED_ABOVE(hint.HintDescriptor)) {
+            upper = hint.UpperBound;
+            hashi = true;
+         }
+         if (LADSPA_IS_HINT_SAMPLE_RATE(hint.HintDescriptor)) {
+            lower *= sampleRate * 1000;
+            upper *= sampleRate;
+            forceint = true;
+         }
+
+         wxString str;
+         if (haslo) {
+            if (LADSPA_IS_HINT_INTEGER(hint.HintDescriptor) || forceint)
+               str.Printf(wxT("%d"), (int)(lower + 0.5));
+            else
+               str = Internat::ToDisplayString(lower);
+            item = new wxStaticText(w, 0, str);
+            gridSizer->Add(item, 0, wxALIGN_CENTER_VERTICAL | wxALIGN_RIGHT | wxALL, 5);
+         }
+         else {
+            gridSizer->Add(1, 1, 0);
+         }
+
+         sliders[p] =
+             new wxSlider(w, p,
+                          0, 0, 1000,
+                          wxDefaultPosition,
+                          wxSize(200, -1));
+         gridSizer->Add(sliders[p], 0, wxALIGN_CENTER_VERTICAL | wxEXPAND | wxALL, 5);
+         ConnectFocus(sliders[p]);
+
+         if (hashi) {
+            if (LADSPA_IS_HINT_INTEGER(hint.HintDescriptor) || forceint)
+               str.Printf(wxT("%d"), (int)(upper + 0.5));
+            else
+               str = Internat::ToDisplayString(upper);
+            item = new wxStaticText(w, 0, str);
+            gridSizer->Add(item, 0, wxALIGN_CENTER_VERTICAL | wxALIGN_LEFT | wxALL, 5);
+         }
+         else {
+            gridSizer->Add(1, 1, 0);
+         }
+      }
+   }
+
+   // Now add the length control
+   if (effect->GetEffectFlags() & INSERT_EFFECT) {
+      item = new wxStaticText(w, 0, _("Length (seconds)"));
+      gridSizer->Add(item, 0, wxALIGN_CENTER_VERTICAL | wxALL, 5);
+      mSeconds = new wxTextCtrl(w, LADSPA_SECONDS_ID, Internat::ToDisplayString(length));
+      gridSizer->Add(mSeconds, 0, wxALIGN_CENTER_VERTICAL | wxALL, 5);
+      gridSizer->Add(1, 1, 0);
+      gridSizer->Add(1, 1, 0);
+      gridSizer->Add(1, 1, 0);
+      ConnectFocus(mSeconds);
+   }
+
+   // Set all of the sliders based on the value in the
+   // text fields
+	inSlider = false; // Now we're ready for HandleText to actually do something.
+   HandleText();
+   
+   paramSizer->Add(gridSizer, 1, wxEXPAND | wxALL, 5);
+   w->SetSizer(paramSizer);
+
+   Layout();
+   Fit();
+   SetSizeHints(GetSize());
 }
 
 LadspaEffectDialog::~LadspaEffectDialog()
@@ -516,20 +719,17 @@ LadspaEffectDialog::~LadspaEffectDialog()
    delete[]labels;
 }
 
+void LadspaEffectDialog::OnCheckBox(wxCommandEvent &event)
+{
+   int p = event.GetId();
+
+   inputControls[ports[p]] = toggles[p]->GetValue();
+}
+
 void LadspaEffectDialog::OnSlider(wxCommandEvent &event)
 {
-   targetSlider = (wxSlider *)event.GetEventObject();
-   HandleSlider();
-   targetSlider = NULL;
-}
+   int p = event.GetId();
 
-void LadspaEffectDialog::OnTextCtrl(wxCommandEvent & WXUNUSED(event))
-{
-	HandleText();
-}
-
-void LadspaEffectDialog::HandleSlider()
-{
    // if we don't add the following three lines, changing
    // the value of the slider will change the text, which
    // will change the slider, and so on.  This gets rid of
@@ -538,43 +738,45 @@ void LadspaEffectDialog::HandleSlider()
       return;
    inSlider = true;
 
-   for (unsigned long p = 0; p < numParams; p++) {
-      if (targetSlider && targetSlider!=sliders[p])
-         continue;
+   float val;
+   float lower = float(0.0);
+   float upper = float(10.0);
+   float range;
+   bool forceint = false;
 
-      float val;
-      float lower = float(0.0);
-      float upper = float(10.0);
-      float range;
-
-      LADSPA_PortRangeHint hint = mData->PortRangeHints[ports[p]];
-      if (LADSPA_IS_HINT_BOUNDED_BELOW(hint.HintDescriptor))
-         lower = hint.LowerBound;
-      if (LADSPA_IS_HINT_BOUNDED_ABOVE(hint.HintDescriptor))
-         upper = hint.UpperBound;
-      if (LADSPA_IS_HINT_SAMPLE_RATE(hint.HintDescriptor)) {
-         lower *= sampleRate;
-         upper *= sampleRate;
-      }
-
-      range = upper - lower;
-
-      val = (sliders[p]->GetValue() / 1000.0) * range + lower;
-
-      wxString str;
-      if (LADSPA_IS_HINT_INTEGER(hint.HintDescriptor))
-         str.Printf(wxT("%d"), (int)(val + 0.5));
-      else
-         str.Printf(wxT("%f"), val);
-
-      fields[p]->SetValue(str);
-
-      inputControls[ports[p]] = val;
+   LADSPA_PortRangeHint hint = mData->PortRangeHints[ports[p]];
+   if (LADSPA_IS_HINT_BOUNDED_BELOW(hint.HintDescriptor))
+      lower = hint.LowerBound;
+   if (LADSPA_IS_HINT_BOUNDED_ABOVE(hint.HintDescriptor))
+      upper = hint.UpperBound;
+   if (LADSPA_IS_HINT_SAMPLE_RATE(hint.HintDescriptor)) {
+      lower *= sampleRate;
+      upper *= sampleRate;
+      forceint = true;
    }
 
+   range = upper - lower;
+
+   val = (sliders[p]->GetValue() / 1000.0) * range + lower;
+
+   wxString str;
+   if (LADSPA_IS_HINT_INTEGER(hint.HintDescriptor) || forceint)
+      str.Printf(wxT("%d"), (int)(val + 0.5));
+   else
+      str = Internat::ToDisplayString(val);
+
+   fields[p]->SetValue(str);
+
+   inputControls[ports[p]] = val;
+
    inSlider = false;
 }
 
+void LadspaEffectDialog::OnTextCtrl(wxCommandEvent & WXUNUSED(event))
+{
+	HandleText();
+}
+
 void LadspaEffectDialog::HandleText()
 {
    // if we don't add the following three lines, changing
@@ -592,10 +794,14 @@ void LadspaEffectDialog::HandleText()
       float upper = float(10.0);
       float range;
 
-      fields[p]->GetValue().ToDouble(&dval);
+      LADSPA_PortRangeHint hint = mData->PortRangeHints[ports[p]];
+      if (LADSPA_IS_HINT_TOGGLED(hint.HintDescriptor)) {
+         continue;
+      }
+
+      dval = Internat::CompatibleToDouble(fields[p]->GetValue());
       val = dval;
 
-      LADSPA_PortRangeHint hint = mData->PortRangeHints[ports[p]];
       if (LADSPA_IS_HINT_BOUNDED_BELOW(hint.HintDescriptor))
          lower = hint.LowerBound;
       if (LADSPA_IS_HINT_BOUNDED_ABOVE(hint.HintDescriptor))
@@ -634,6 +840,57 @@ void LadspaEffectDialog::OnPreview(wxCommandEvent & WXUNUSED(event))
    effect->Preview();
 }
 
+void LadspaEffectDialog::ConnectFocus(wxControl *c)
+{
+   c->GetEventHandler()->Connect(wxEVT_SET_FOCUS,
+                                 wxFocusEventHandler(LadspaEffectDialog::ControlSetFocus));
+}
+
+void LadspaEffectDialog::DisconnectFocus(wxControl *c)
+{
+   c->GetEventHandler()->Disconnect(wxEVT_SET_FOCUS,
+                                 wxFocusEventHandler(LadspaEffectDialog::ControlSetFocus));
+}
+
+void LadspaEffectDialog::ControlSetFocus(wxFocusEvent &event)
+{
+   wxControl *c = (wxControl *) event.GetEventObject();
+   wxScrolledWindow *p = (wxScrolledWindow *) c->GetParent();
+   wxRect r = c->GetRect();
+   wxRect rv = p->GetRect();
+   rv.y = 0;
+
+   event.Skip();
+
+   int y;
+   int yppu;
+   p->GetScrollPixelsPerUnit(NULL, &yppu);
+
+   if (r.y >= rv.y && r.GetBottom() <= rv.GetBottom()) {
+      return;
+   }
+
+   if (r.y < rv.y) {
+      p->CalcUnscrolledPosition(0, r.y, NULL, &r.y);
+      y = r.y / yppu;
+   }
+   else {
+      p->CalcUnscrolledPosition(0, r.y, NULL, &r.y);
+      y = (r.GetBottom() - rv.GetBottom() + yppu) / yppu;
+   }
+
+   p->Scroll(-1, y);
+};
+
+double LadspaEffectDialog::GetLength()
+{
+   if (effect->GetEffectFlags() & INSERT_EFFECT) {
+      mLength = Internat::CompatibleToDouble(mSeconds->GetValue());
+   }
+
+   return mLength;
+}
+
 // Indentation settings for Vim and Emacs and unique identifier for Arch, a
 // version control system. Please do not modify past this point.
 //
