@@ -6,7 +6,37 @@
 
   Dominic Mazzoni
 
-**********************************************************************/
+******************************************************************//**
+
+\class EffectNyquist
+\brief An Effect that calls up a Nyquist (XLISP) plug in, i.e. many possible 
+effects from this one class.
+
+*//****************************************************************//**
+
+\class NyquistInputDialog
+\brief Dialog used with EffectNyquist
+
+*//****************************************************************//**
+
+\class NyquistOutputDialog
+\brief Dialog used with EffectNyquist
+
+*//****************************************************************//**
+
+\class NyquistDialog
+\brief Base class for NyquistInputDialog and NyquistOutputDialog.
+
+*//****************************************************************//**
+
+\class NyqControl
+\brief A control on a NyquistDialog.
+
+*//*******************************************************************/
+
+
+
+#include "../../Audacity.h"
 
 #include <math.h>
 #include <locale.h>
@@ -17,14 +47,18 @@
 #include <wx/msgdlg.h>
 #include <wx/textdlg.h>
 #include <wx/textfile.h>
+#include <wx/choice.h>
 
-#include "../../Audacity.h"
 #include "../../AudacityApp.h"
 #include "../../LabelTrack.h"
 #include "../../Internat.h"
 
 #include "Nyquist.h"
 
+#ifndef nyx_returns_start_and_end_time
+#error You need to update lib-src/libnyquist
+#endif
+
 #include <locale.h>
 
 #include <wx/arrimpl.cpp>
@@ -35,6 +69,7 @@ wxString EffectNyquist::mXlispPath;
 #define NYQ_CTRL_INT 0
 #define NYQ_CTRL_REAL 1
 #define NYQ_CTRL_STRING 2
+#define NYQ_CTRL_CHOICE 3
 
 extern "C" {
    extern void set_xlisp_path(const char *p);
@@ -129,10 +164,11 @@ void EffectNyquist::Parse(wxString line)
       return;
    }
 
-   // We support versions 1 and 2
+   // We support versions 1, 2 and 3
    // (Version 2 added support for string parameters.)
+   // (Version 3 added support for choice parameters.)
    if (len>=2 && tokens[0]==wxT("version")) {
-      if (tokens[1]==wxT("1") || tokens[1]==wxT("2")) {
+      if (tokens[1]==wxT("1") || tokens[1]==wxT("2") || tokens[1]==wxT("3")) {
          // We're okay
       }
       else {
@@ -167,6 +203,8 @@ void EffectNyquist::Parse(wxString line)
  
       if (tokens[3]==wxT("string"))
          ctrl.type = NYQ_CTRL_STRING;
+      else if( tokens[ 3 ] == wxT( "choice" ) )
+         ctrl.type = NYQ_CTRL_CHOICE;
       else {
          if (len < 8)
             return;
@@ -188,7 +226,7 @@ void EffectNyquist::Parse(wxString line)
 
 void EffectNyquist::ParseFile()
 {
-   wxTextFile f(FILENAME(mFileName.GetFullPath()));
+   wxTextFile f(mFileName.GetFullPath());
    if (!f.Open())
       return;
 
@@ -227,7 +265,7 @@ EffectNyquist::EffectNyquist(wxString fName)
    wxLogNull dontLog;
 
    mName = wxFileName(fName).GetName();
-   mFileName = wxFileName(FILENAME(fName));
+   mFileName = wxFileName(fName);
    mFileModified = mFileName.GetModificationTime();
    ParseFile();
 }
@@ -241,7 +279,7 @@ bool EffectNyquist::SetXlispPath()
    wxString fname;
 
    fname = mXlispPath + wxFILE_SEP_PATH + wxT("nyinit.lsp");
-   if (!(::wxFileExists(FILENAME(fname))))
+   if (!(::wxFileExists(fname)))
       mXlispPath = wxT("");
 
    if (mXlispPath == wxT("")) {
@@ -271,7 +309,7 @@ bool EffectNyquist::SetXlispPath()
    #endif // Unicode/ANSI
 
    fname = mXlispPath + wxFILE_SEP_PATH + wxT("nyinit.lsp");
-   return ::wxFileExists(FILENAME(fname));
+   return ::wxFileExists(fname);
 }
 
 bool EffectNyquist::PromptUser()
@@ -319,6 +357,9 @@ bool EffectNyquist::PromptUser()
       if (ctrl->val == UNINITIALIZED_CONTROL)
          ctrl->val = GetCtrlValue(ctrl->valStr);
 
+      if( ctrl->type == NYQ_CTRL_CHOICE )
+         continue;
+      
       ctrl->low = GetCtrlValue(ctrl->lowStr);
       ctrl->high = GetCtrlValue(ctrl->highStr);
 
@@ -362,11 +403,7 @@ bool EffectNyquist::Process()
    mDebugOutput = wxT("");
    while (mCurTrack[0]) {
       mCurNumChannels = 1;
-      double trackStart = mCurTrack[0]->GetStartTime();
-      double trackEnd = mCurTrack[0]->GetEndTime();
-      double t0 = mT0 < trackStart? trackStart: mT0;
-      double t1 = mT1 > trackEnd? trackEnd: mT1;
-      if (t1 >= t0) {
+      if (mT1 >= mT0) {
          if (mCurTrack[0]->GetLinked()) {
             mCurNumChannels = 2;
 
@@ -377,11 +414,11 @@ bool EffectNyquist::Process()
                             wxOK | wxCENTRE, mParent);
                return false;
             }
-            mCurStart[1] = mCurTrack[1]->TimeToLongSamples(t0);
+            mCurStart[1] = mCurTrack[1]->TimeToLongSamples(mT0);
          }
 
-         mCurStart[0] = mCurTrack[0]->TimeToLongSamples(t0);
-         longSampleCount end = mCurTrack[0]->TimeToLongSamples(t1);
+         mCurStart[0] = mCurTrack[0]->TimeToLongSamples(mT0);
+         longSampleCount end = mCurTrack[0]->TimeToLongSamples(mT1);
          mCurLen = (sampleCount)(end - mCurStart[0]);
 
          success = ProcessOne();
@@ -497,9 +534,10 @@ bool EffectNyquist::ProcessOne()
    if (mDebug)
       nyx_capture_output(65536);
 
-   nyx_set_input_audio(StaticGetCallback, (void *)this,
-                       mCurNumChannels,
-                       mCurLen, mCurTrack[0]->GetRate());
+   if( !( mFlags & INSERT_EFFECT ) )
+      nyx_set_input_audio(StaticGetCallback, (void *)this,
+                          mCurNumChannels,
+                          mCurLen, mCurTrack[0]->GetRate());
 
    wxString cmd;
 
@@ -512,7 +550,8 @@ bool EffectNyquist::ProcessOne()
          cmd = cmd+wxString::Format(wxT("(setf %s %f)\n"),
                                     mControls[j].var.c_str(),
                                     mControls[j].val);
-      else if (mControls[j].type==NYQ_CTRL_INT)
+      else if (mControls[j].type==NYQ_CTRL_INT || 
+            mControls[j].type == NYQ_CTRL_CHOICE)
          cmd = cmd+wxString::Format(wxT("(setf %s %d)\n"),
                                     mControls[j].var.c_str(),
                                     (int)(mControls[j].val));
@@ -589,12 +628,12 @@ bool EffectNyquist::ProcessOne()
       }
 
       for(l=0; l<numLabels; l++) {
-         double t;
+         double t0, t1;
          const char *str;
 
-         nyx_get_label(l, &t, &str);
+         nyx_get_label(l, &t0, &t1, &str);
 
-         ltrack->AddLabel(t + mT0, t + mT0, LAT1CTOWX(str));
+         ltrack->AddLabel(t0 + mT0, t1 + mT0, LAT1CTOWX(str));
       }
 
       nyx_cleanup();
@@ -625,12 +664,11 @@ bool EffectNyquist::ProcessOne()
    double rate = mCurTrack[0]->GetRate();
    for(i=0; i<outChannels; i++) {
       sampleFormat format = mCurTrack[i]->GetSampleFormat();
-      mOutputTrack[i] = mFactory->NewWaveTrack(format);
 
       if (outChannels == mCurNumChannels)
          rate = mCurTrack[i]->GetRate();
 
-      mOutputTrack[i]->SetRate( rate );
+      mOutputTrack[i] = mFactory->NewWaveTrack(format, rate);
       mCurBuffer[i] = NULL;
    }
 
@@ -670,6 +708,7 @@ bool EffectNyquist::ProcessOne()
 
 #define ID_NYQ_SLIDER 2000
 #define ID_NYQ_TEXT   3000
+#define ID_NYQ_CHOICE 4000
 
 BEGIN_EVENT_TABLE(NyquistDialog, wxDialog)
    EVT_BUTTON(wxID_OK, NyquistDialog::OnOk)
@@ -679,6 +718,8 @@ BEGIN_EVENT_TABLE(NyquistDialog, wxDialog)
                      wxEVT_COMMAND_SLIDER_UPDATED, NyquistDialog::OnSlider)
    EVT_COMMAND_RANGE(ID_NYQ_TEXT, ID_NYQ_TEXT+99,
                       wxEVT_COMMAND_TEXT_UPDATED, NyquistDialog::OnText)
+   EVT_COMMAND_RANGE( ID_NYQ_CHOICE, ID_NYQ_CHOICE + 99,
+                     wxEVT_COMMAND_CHOICE_SELECTED, NyquistDialog::OnChoice )
 END_EVENT_TABLE()
 
 NyquistDialog::NyquistDialog(wxWindow * parent, wxWindowID id,
@@ -705,8 +746,6 @@ NyquistDialog::NyquistDialog(wxWindow * parent, wxWindowID id,
    for(unsigned int i=0; i<mControls->GetCount(); i++) {
       wxControl  *item;
       NyqControl *ctrl = &((*mControls)[i]);
-      int val = (int)(0.5 + ctrl->ticks * (ctrl->val - ctrl->low)/
-                      (ctrl->high - ctrl->low));
 
       item = new wxStaticText(this, -1, ctrl->name);
       grid->Add(item, 0, wxALIGN_RIGHT | 
@@ -720,8 +759,42 @@ NyquistDialog::NyquistDialog(wxWindow * parent, wxWindowID id,
          grid->Add(item, 0, wxALIGN_CENTRE |
                    wxALIGN_CENTER_VERTICAL | wxALL, 5);
       }
+      else if( ctrl->type == NYQ_CTRL_CHOICE )
+      {
+         //str is coma separated labels for each choice
+         wxString str = ctrl->label;
+         wxArrayString choices;
+         
+         while( 1 )
+         {       
+            int ci = str.Find( ',' ); //coma index
+            
+            if( ci == -1 )
+            {
+               choices.Add( str );
+               break;
+            }
+            else
+               choices.Add( str.Left( ci ) );
+            
+            str = str.Right( str.length() - ci - 1 );
+         }
+
+         wxChoice *choice = new wxChoice( this, ID_NYQ_CHOICE + i, 
+               wxDefaultPosition, wxSize( 150, -1 ), choices );
+         
+         int val = ( int )ctrl->val;
+         if( val >= 0 && val < choice->GetCount() )
+            choice->SetSelection( val );
+        
+         grid->Add( 10, 10 );
+         grid->Add( choice, 0, wxALIGN_CENTRE | 
+               wxALIGN_CENTER_VERTICAL | wxALL, 5 );
+      }
       else {
          // Integer or Real
+         int val = (int)(0.5 + ctrl->ticks * (ctrl->val - ctrl->low)/
+               (ctrl->high - ctrl->low));
 
          item = new wxTextCtrl(this, ID_NYQ_TEXT+i, wxT(""),
                                wxDefaultPosition, wxSize(60, -1));
@@ -734,23 +807,25 @@ NyquistDialog::NyquistDialog(wxWindow * parent, wxWindowID id,
                    wxALIGN_CENTER_VERTICAL | wxALL, 5);
       }
          
-      item = new wxStaticText(this, -1, ctrl->label);
-      grid->Add(item, 0, wxALIGN_LEFT | 
-                wxALIGN_CENTER_VERTICAL | wxALL, 5);
+      if( ctrl->type != NYQ_CTRL_CHOICE )
+      {
+         item = new wxStaticText(this, -1, ctrl->label);
+         grid->Add(item, 0, wxALIGN_LEFT | 
+                   wxALIGN_CENTER_VERTICAL | wxALL, 5);
+      }
    }
    mainSizer->Add(grid, 0, wxALIGN_CENTRE | wxALL, 5);
 
    hSizer = new wxBoxSizer(wxHORIZONTAL);
 
-   button = new wxButton(this, wxID_CANCEL, _("Cancel"));
+   button = new wxButton(this, wxID_CANCEL, _("&Cancel"));
    hSizer->Add(button, 0, wxALIGN_CENTRE | wxALL, 5);
 
-   button = new wxButton(this, wxID_MORE, _("Debug"));
+   button = new wxButton(this, wxID_MORE, _("&Debug"));
    hSizer->Add(button, 0, wxALIGN_CENTRE | wxALL, 5);
 
-   button = new wxButton(this, wxID_OK, _("OK"));
+   button = new wxButton(this, wxID_OK, _("&OK"));
    button->SetDefault();
-   button->SetFocus();
    hSizer->Add(button, 0, wxALIGN_CENTRE | wxALL, 5);
 
    mainSizer->Add(hSizer, 0, wxALIGN_CENTRE | wxALL, 5);
@@ -775,7 +850,7 @@ void NyquistDialog::OnSlider(wxCommandEvent & /* event */)
    for(unsigned int i=0; i<mControls->GetCount(); i++) {
       NyqControl *ctrl = &((*mControls)[i]);
 
-      if (ctrl->type == NYQ_CTRL_STRING)
+      if (ctrl->type == NYQ_CTRL_STRING || ctrl->type == NYQ_CTRL_CHOICE )
          continue;
 
       wxSlider *slider = (wxSlider *)FindWindow(ID_NYQ_SLIDER + i);
@@ -807,23 +882,42 @@ void NyquistDialog::OnSlider(wxCommandEvent & /* event */)
    mInHandler = false;
 }
 
-void NyquistDialog::OnText(wxCommandEvent & /* event */)
+void NyquistDialog::OnChoice( wxCommandEvent &event )
+{
+   if( mInHandler )
+      return; // prevent recursing forever
+   mInHandler = true;
+
+   unsigned int ctrlId = event.GetId() - ID_NYQ_CHOICE;
+   wxASSERT( ctrlId >= 0 && ctrlId < mControls->GetCount() );
+   
+   NyqControl *ctrl = &( mControls->Item( ctrlId ) );
+   wxChoice *choice = ( wxChoice * )FindWindow( ID_NYQ_CHOICE + ctrlId );
+   wxASSERT( choice );
+
+   ctrl->val = choice->GetSelection();
+
+   mInHandler = false;   
+}
+
+void NyquistDialog::OnText(wxCommandEvent &event)
 {
    if (mInHandler)
       return; // prevent recursing forever
    mInHandler = true;
 
-   for(unsigned int i=0; i<mControls->GetCount(); i++) {
-      NyqControl *ctrl = &((*mControls)[i]);
-      wxTextCtrl *text = (wxTextCtrl *)FindWindow(ID_NYQ_TEXT + i);
-      wxASSERT(text);
+   unsigned int ctrlId = event.GetId() - ID_NYQ_TEXT;
+   wxASSERT( ctrlId >= 0 && ctrlId < mControls->GetCount() );
 
-      ctrl->valStr = text->GetValue();
+   NyqControl *ctrl = &((*mControls)[ ctrlId ]);
+   wxTextCtrl *text = (wxTextCtrl *)FindWindow(ID_NYQ_TEXT + ctrlId);
+   wxASSERT(text);
 
-      if (ctrl->type == NYQ_CTRL_STRING)
-         continue;
+   ctrl->valStr = text->GetValue();
 
-      wxSlider *slider = (wxSlider *)FindWindow(ID_NYQ_SLIDER + i);
+   if( ctrl->type != NYQ_CTRL_STRING )
+   {
+      wxSlider *slider = (wxSlider *)FindWindow(ID_NYQ_SLIDER + ctrlId);
       wxASSERT(slider);
 
       ctrl->valStr.ToDouble(&ctrl->val);
@@ -962,7 +1056,6 @@ NyquistOutputDialog::NyquistOutputDialog(wxWindow * parent, wxWindowID id,
 
    button = new wxButton(this, wxID_OK, _("OK"));
    button->SetDefault();
-   button->SetFocus();
    hSizer->Add(button, 0, wxALIGN_CENTRE | wxALL, 5);
 
    mainSizer->Add(hSizer, 0, wxALIGN_CENTRE |
