@@ -6,14 +6,17 @@
 
   Dominic Mazzoni
 
-  Presents a dialog box allowing the user to export multiple files
-  either by exporting each track as a separate file, or by
-  exporting each label as a separate file.
-
   2004.06.23 added mp3 export using tompg.exe for multiple export, by ChackoN
   tested under w2k only
 
-**********************************************************************/
+*******************************************************************//**
+
+\class ExportMultipleDialog
+\brief Presents a dialog box allowing the user to export multiple files
+  either by exporting each track as a separate file, or by
+  exporting each label as a separate file.
+
+*//********************************************************************/
 
 #include "../Audacity.h"
 
@@ -38,6 +41,7 @@
 #include "ExportMP3.h"
 #include "ExportOGG.h"
 #include "ExportCL.h"
+#include "ExportFLAC.h"
 
 #include "../Internat.h"
 #include "../FileFormats.h"
@@ -76,7 +80,7 @@ class ExportMultipleDialog : public wxDialog
    wxString prefix;
 
    // output
-   int format; // PCM, MP3, Ogg
+   int format; // PCM, MP3, Ogg, FLAC
    wxString dir;
    bool forceMono;
 
@@ -261,7 +265,7 @@ wxString getMpgExePath()
 {
    wxString mMpgExePath = gPrefs->Read(wxT("/MP3/MpgExePath"), wxT(""));
 
-   if (mMpgExePath==wxT("") || !::wxFileExists(FILENAME(mMpgExePath))) {
+   if (mMpgExePath==wxT("") || !::wxFileExists(mMpgExePath)) {
          
       int action = wxMessageBox(
          _("Audacity needs to know the location of the TOMPG.EXE program.\nPlease download and save the freely available file in any convenient\nlocation such as the Plug-Ins subdir of Audacity and let Audacity\nknow about this path.  Would you like to help Audacity locate the file now?"),
@@ -314,7 +318,7 @@ static bool DoExport(AudacityProject *project,
          sf_header_extension(format & SF_FORMAT_TYPEMASK);
       wxString fullPath = MakeFullPath(overwrite,
                                        dir, name, extension);
-      return ExportPCM(project, stereo, fullPath,
+      return ExportPCM(project, stereo ? 2 : 1, fullPath,
                        selectionOnly, t0, t1);
    } break;
    case 1: {
@@ -326,10 +330,14 @@ static bool DoExport(AudacityProject *project,
       return ExportMP3(project, stereo, fullPath,
                        selectionOnly, t0, t1); }
    case 2: {
+#ifdef USE_LIBVORBIS
       wxString fullPath = MakeFullPath(overwrite,
                                        dir, name, wxT(".ogg"));
-      return ExportOGG(project, stereo, fullPath,
+      return ExportOGG(project, stereo ? 2 : 1, fullPath,
                        selectionOnly, t0, t1);
+#else
+      return false;
+#endif
    }
    case 3: {//tompg.  First make wav files, then call tompg.exe to convert
       wxString mp3OutputFile = MakeFullPath(overwrite,dir, name, wxT(".mp3"));
@@ -346,7 +354,7 @@ static bool DoExport(AudacityProject *project,
             sf_header_extension(format & SF_FORMAT_TYPEMASK) + wxT("_tmp");
          wxString tmpWavOuput = MakeFullPath(overwrite,
                                        dir, name, extension);
-         if (ExportPCM(project, stereo, tmpWavOuput,
+         if (ExportPCM(project, stereo ? 2 : 1, tmpWavOuput,
                        selectionOnly, t0, t1)){
             //2. convert the wav to mp3 using tompg.exe
             wxString tompgCmd = wxT("\"") + mpgPath + wxT("\" ") + tmpWavOuput + wxT(" ") + mp3OutputFile + mono + wxT(" -B") + mpgBitrateString;
@@ -357,13 +365,7 @@ static bool DoExport(AudacityProject *project,
             wxLogStatus(_T("command '%s' terminated with exit code %d."),
                tompgCmd.c_str(), code);
 
-            #ifdef _UNICODE
-               /* remove doesn't handle fn_Str() in Unicode build.  
-                  May or may not actually work. */
-               remove(tmpWavOuput.mb_str()); //delete the tmp wav file
-            #else // ANSI
-               remove(tmpWavOuput.fn_str()); //delete the tmp wav file
-            #endif // Unicode/ANSI
+            remove(OSFILENAME(tmpWavOuput)); //delete the tmp wav file
 
             wxEndBusyCursor();
 
@@ -375,6 +377,20 @@ static bool DoExport(AudacityProject *project,
       else
          return false;
    }
+   case 4: {
+#ifdef USE_LIBFLAC
+      Tags *tags = project->GetTags();
+      tags->SetTitle(name);            
+      tags->SetTrackNumber(trackNumber);
+      wxString fullPath = MakeFullPath(overwrite,
+                                       dir, name, wxT(".flac"));
+      return ExportFLAC(project, (stereo ? 2: 1), fullPath,
+                       selectionOnly, t0, t1);
+      
+#else
+      return false;
+#endif
+   }
 
    }
 }
@@ -453,9 +469,9 @@ bool ExportMultipleByTrack(AudacityProject *project,
          
          i++;
          
-         if (tr)
-            tr = iter.Next();
       }
+      if (tr)
+         tr = iter.Next();
    }   
    
    RestoreSelected(tracks, saveSelected);
@@ -568,10 +584,21 @@ bool ExportMultipleByLabel(AudacityProject *project,
       l++;
    }
    
-   ::wxMessageBox(wxString::Format(_("Successfully exported %d files."),
-                                   count),
-                  _("Export Multiple"),
-                  wxOK | wxCENTRE, project);
+
+   if (ok)
+   {      
+      ::wxMessageBox(wxString::Format(_("Successfully exported %d file(s)."),
+                                      count),
+                     _("Export Multiple"),
+                     wxOK | wxCENTRE, project);
+   }
+   else
+   {
+      ::wxMessageBox(wxString::Format(_("Something went wrong after exporting %d file(s)."),
+                                      count),
+                     _("Export Multiple"),
+                     wxOK | wxCENTRE, project);
+   }
    
    return ok;
 }
@@ -602,7 +629,7 @@ bool ExportMultiple(AudacityProject *project)
    if (!dlog.GetReturnCode())
       return false;
 
-   if (dlog.format == 1) { // MP3
+   if (dlog.format == 1 || dlog.format == 4) { // MP3 or FLAC
       Tags *tags = project->GetTags();
       if (tags->IsEmpty()) {
          wxString saveTitle = tags->GetTitle();
@@ -612,7 +639,8 @@ bool ExportMultiple(AudacityProject *project)
          tags->AllowEditTitle(false);
          tags->AllowEditTrackNumber(false);
          bool rval = tags->ShowEditDialog(project,
-                                          _("Edit the ID3 tags for all MP3 files"));
+                                          _("Edit the ID3 tags for all MP3 files"),
+                                          true);
          tags->AllowEditTitle(true);
          tags->AllowEditTrackNumber(true);
          if (!rval) {
@@ -670,13 +698,14 @@ ExportMultipleDialog::ExportMultipleDialog(wxWindow *parent, wxWindowID id):
    hSizer->Add(new wxStaticText(this, -1, _("Export format:")),
                0, wxALL | wxALIGN_CENTER_VERTICAL, 5);
 
-   int numFormats = 4;
-   wxString formatList[4];
+   int numFormats = 5;
+   wxString formatList[5];
    int format = ReadExportFormatPref();
    formatList[0] = sf_header_shortname(format & SF_FORMAT_TYPEMASK);
    formatList[1] = wxT("MP3");
    formatList[2] = wxT("Ogg Vorbis");
    formatList[3] = wxT("MP3_mpg");
+   formatList[4] = wxT("FLAC");
 
    mFormat = new wxChoice(this, FormatID,
                           wxDefaultPosition, wxDefaultSize,
@@ -812,10 +841,10 @@ ExportMultipleDialog::ExportMultipleDialog(wxWindow *parent, wxWindowID id):
 
    hSizer = new wxBoxSizer(wxHORIZONTAL);
 
-   mCancel = new wxButton(this, wxID_CANCEL, _("Cancel"));
+   mCancel = new wxButton(this, wxID_CANCEL, _("&Cancel"));
    hSizer->Add(mCancel, 0, wxALL, 5);
 
-   mExport = new wxButton(this, wxID_OK, _("Export"));
+   mExport = new wxButton(this, wxID_OK, _("E&xport"));
    mExport->SetDefault();
    hSizer->Add(mExport, 0, wxALL, 5);
 
@@ -857,7 +886,7 @@ void ExportMultipleDialog::EnableControls()
 void ExportMultipleDialog::CopyDataToControls()
 {
    wxString defaultPath = gPrefs->Read(wxT("/DefaultExportPath"),
-                                       FROMFILENAME(::wxGetCwd()));
+                                       ::wxGetCwd());
    wxString path = gPrefs->Read(wxT("/DefaultMultiplExportPath"),
                                 defaultPath);
    mDir->SetValue(path);
