@@ -6,14 +6,32 @@
 
   Dominic Mazzoni
 
-  VU Meter, for displaying recording/playback level
+  2004.06.25 refresh rate limited to 30mS, by ChackoN
+
+*******************************************************************//**
+
+\class Meter
+\brief VU Meter, for displaying recording/playback level
 
   This is a bunch of common code that can display many different
   forms of VU meters and other displays.
 
-  2004.06.25 refresh rate limited to 30mS, by ChackoN
+*//****************************************************************//**
 
-**********************************************************************/
+\class MeterBar
+\brief A struct used by Meter to hold the position of one bar.
+
+*//****************************************************************//**
+
+\class MeterUpdateMsg
+\brief Message used to update the Meter
+
+*//****************************************************************//**
+
+\class MeterUpdateQueue
+\brief Queue of MeterUpdateMsg used to feed the Meter.
+
+*//******************************************************************/
 
 #include "../Audacity.h"
 
@@ -35,11 +53,14 @@
 #include "../AudioIO.h"
 #include "../AColor.h"
 #include "../ImageManipulation.h"
-#include "../../images/MixerImages.h"
+//#include "../../images/MixerImages.h"
 #include "../Project.h"
-#include "../MeterToolBar.h"
+#include "../toolbars/MeterToolBar.h"
 #include "../Prefs.h"
 
+#include "../Theme.h"
+#include "../AllThemeResources.h"
+
 //
 // The Meter passes itself messages via this queue so that it can
 // communicate between the audio thread and the GUI thread.
@@ -121,6 +142,7 @@ enum {
 BEGIN_EVENT_TABLE(Meter, wxPanel)
    EVT_TIMER(OnMeterUpdateID, Meter::OnMeterUpdate)
    EVT_MOUSE_EVENTS(Meter::OnMouse)
+   EVT_ERASE_BACKGROUND(Meter::OnErase)
    EVT_PAINT(Meter::OnPaint)
    EVT_SIZE(Meter::OnSize)
    EVT_MENU(OnDisableMeterID, Meter::OnDisableMeter)
@@ -160,7 +182,6 @@ Meter::Meter(wxWindow* parent, wxWindowID id,
    mNumBars(0),
    mLayoutValid(false),
    mBitmap(NULL),
-   mBackgroundBitmap(NULL),
    mIcon(NULL)
 {
    int i;
@@ -177,11 +198,8 @@ Meter::Meter(wxWindow* parent, wxWindowID id,
       mMeterDisabled = gPrefs->Read(wxT("/Meter/MeterOutputDisabled"), (long)0);
    }
 
-//   CreateIcon(2);
-
-   wxColour origColour(204, 204, 204);
-   mPeakPeakPen = wxPen(wxColour(102, 102, 255), 1, wxSOLID);
-   mDisabledPen = wxPen(wxColour(192, 192, 192), 1, wxSOLID);
+   mPeakPeakPen = wxPen(theTheme.Colour( clrMeterPeak),        1, wxSOLID);
+   mDisabledPen = wxPen(theTheme.Colour( clrMeterDisabledPen), 1, wxSOLID);
 
    /* i18n-hint: One-letter abbreviation for Left, in VU Meter */
    mLeftText = _("L");
@@ -192,23 +210,23 @@ Meter::Meter(wxWindow* parent, wxWindowID id,
    mRightSize = wxSize(0, 0);
 
    if (mIsInput) {
-      mPen = wxPen(wxColour(204, 70, 70), 1, wxSOLID);
-      mBrush = wxBrush(wxColour(204, 70, 70), wxSOLID);
-      mRMSBrush = wxBrush(wxColour(255, 102, 102), wxSOLID);
-      mClipBrush = wxBrush(wxColour(255, 53, 53), wxSOLID);
-      mLightPen = wxPen(wxColour(255, 153, 153), 1, wxSOLID);
-      mDarkPen = wxPen(wxColour(153, 61, 61), 1, wxSOLID);
-   }
-   else {
-      mPen = wxPen(wxColour(70, 204, 70), 1, wxSOLID);
-      mBrush = wxBrush(wxColour(70, 204, 70), wxSOLID);
-      mRMSBrush = wxBrush(wxColour(102, 255, 102), wxSOLID);
-      mClipBrush = wxBrush(wxColour(255, 53, 53), wxSOLID);
-      mLightPen = wxPen(wxColour(153, 255, 153), 1, wxSOLID);
-      mDarkPen = wxPen(wxColour(61, 164, 61), 1, wxSOLID);
+      mPen       = wxPen(   theTheme.Colour( clrMeterInputPen         ), 1, wxSOLID);
+      mBrush     = wxBrush( theTheme.Colour( clrMeterInputBrush       ), wxSOLID);
+      mRMSBrush  = wxBrush( theTheme.Colour( clrMeterInputRMSBrush    ), wxSOLID);
+      mClipBrush = wxBrush( theTheme.Colour( clrMeterInputClipBrush   ), wxSOLID);
+      mLightPen  = wxPen(   theTheme.Colour( clrMeterInputLightPen    ), 1, wxSOLID);
+      mDarkPen   = wxPen(   theTheme.Colour( clrMeterInputDarkPen     ), 1, wxSOLID);
+   }   
+   else {                                                         
+      mPen       = wxPen(   theTheme.Colour( clrMeterOutputPen        ), 1, wxSOLID);
+      mBrush     = wxBrush( theTheme.Colour( clrMeterOutputBrush      ), wxSOLID);
+      mRMSBrush  = wxBrush( theTheme.Colour( clrMeterOutputRMSBrush   ), wxSOLID);
+      mClipBrush = wxBrush( theTheme.Colour( clrMeterOutputClipBrush  ), wxSOLID);
+      mLightPen  = wxPen(   theTheme.Colour( clrMeterOutputLightPen   ), 1, wxSOLID);
+      mDarkPen   = wxPen(   theTheme.Colour( clrMeterOutputDarkPen    ), 1, wxSOLID);
    }
 
-      mDisabledBkgndBrush = wxBrush(wxColour(160, 160, 160), wxSOLID);
+   mDisabledBkgndBrush = wxBrush(theTheme.Colour( clrMeterDisabledBrush), wxSOLID);
 //   mDisabledBkgndBrush = wxBrush(
 //            wxSystemSettings::GetSystemColour(wxSYS_COLOUR_3DSHADOW), wxSOLID);
 //            wxSystemSettings::GetSystemColour(wxSYS_COLOUR_3DLIGHT), wxSOLID);
@@ -233,43 +251,38 @@ Meter::Meter(wxWindow* parent, wxWindowID id,
 
 void Meter::CreateIcon(int aquaOffset)
 {
-   wxColour backgroundColour =  mBkgndBrush.GetColour();
-//      wxSystemSettings::GetSystemColour(wxSYS_COLOUR_3DFACE);
-//   mBkgndBrush = wxBrush(backgroundColour, wxSOLID);
-   wxImage *image, *alpha;
-
+   /// \todo Remove wasteful delete/new pair.  It is done in every call to layout.
    if (mIcon) {
+//      wxASSERT( false );
       delete mIcon;
       mIcon = NULL;
    }
-
-   if (mIsInput) {
-      image = new wxImage(wxBitmap(Mic).ConvertToImage());
-      alpha = new wxImage(wxBitmap(MicAlpha).ConvertToImage());
+   if(mIsInput)
+   {
+      /// JKC: !!!! If you pass theTheme.Bitmap(bmpMic) you get a white rather than a black mic.
+      /// Weird behaviour in wxWidgets, I guess.
+      mIcon = new wxBitmap(theTheme.Image( bmpMic ));
    }
-   else {
-      image = new wxImage(wxBitmap(Speaker).ConvertToImage());
-      alpha = new wxImage(wxBitmap(SpeakerAlpha).ConvertToImage());
+   else
+   {
+      mIcon = new wxBitmap(theTheme.Image( bmpSpeaker ));
    }
-
-   wxImage *bkgnd = CreateSysBackground(25, 25, aquaOffset,
-					backgroundColour);
-   wxImage *final = OverlayImage(bkgnd, image, alpha, 0, 0);
-   mIcon = new wxBitmap(final);
-
-   delete image;
-   delete alpha;
-   delete bkgnd;
-   delete final;
 }
 
 Meter::~Meter()
 {
+   // LLL:  This prevents a crash during termination if monitoring
+   //       is active.
+   if (gAudioIO->IsMonitoring())
+      gAudioIO->StopStream();
    delete mIcon;
    if (mBitmap)
       delete mBitmap;
-   if (mBackgroundBitmap)
-     delete mBackgroundBitmap;
+}
+
+void Meter::OnErase(wxEraseEvent &evt)
+{
+   // Ignore it to prevent flashing
 }
 
 void Meter::OnPaint(wxPaintEvent &evt)
@@ -579,26 +592,12 @@ void Meter::HandleLayout()
    int barw, barh;
    int i;
 
-#ifdef USE_AQUA_THEME
-   if (!mBackgroundBitmap ||
-       mBackgroundBitmap->GetWidth() != mWidth ||
-       mBackgroundBitmap->GetWidth() != mHeight) {
-     if (mBackgroundBitmap)
-       delete mBackgroundBitmap;
-
-     wxImage *image = CreateAquaBackground(mWidth, mHeight, 0);
-     mBackgroundBitmap = new wxBitmap(image);
-     delete image;
-   }
-#endif
-
    mRuler.SetFlip(true);
    mRuler.SetLabelEdges(true);
 
    switch(mStyle) {
    default:
       wxPrintf(wxT("Style not handled yet!\n"));
-   case Disable:
       break;
    case VerticalStereo:
       mMenuRect = wxRect(mWidth - menuWidth - 5, mHeight - menuHeight - 2,
@@ -639,7 +638,7 @@ void Meter::HandleLayout()
       mRuler.SetBounds(mBar[1].r.x + mBar[1].r.width + 1,
                        mBar[1].r.y,
                        mBar[1].r.x + width,
-                       mBar[1].r.height);
+                       mBar[1].r.y + mBar[1].r.height);
       if (mDB) {
          mRuler.SetRange(0, -mDBRange);
          mRuler.SetFormat(Ruler::LinearDBFormat);
@@ -746,15 +745,13 @@ void Meter::HandlePaint(wxDC &dc)
    if (!mLayoutValid)
       HandleLayout();
 
-#ifdef USE_AQUA_THEME
-   dc.DrawBitmap(*mBackgroundBitmap, 0, 0);
-#else
+#ifndef USE_AQUA_THEME
    dc.SetPen(*wxTRANSPARENT_PEN);
    dc.SetBrush(mBkgndBrush);
    dc.DrawRectangle(0, 0, mWidth, mHeight);
 #endif
 
-   dc.DrawBitmap(*mIcon, mIconPos.x, mIconPos.y);
+   dc.DrawBitmap(*mIcon, mIconPos.x, mIconPos.y, true);
 
    // Draws a beveled button and a down pointing triangle.
    // The style and sizing matches the ones in the title 
@@ -812,7 +809,7 @@ void Meter::RepaintBarsNow()
 void Meter::DrawMeterBar(wxDC &dc, MeterBar *meterBar)
 {
    wxRect r = meterBar->r;
-   wxRect rRMS;
+   wxRect rRMS = meterBar->r;
    dc.SetPen(*wxTRANSPARENT_PEN);
    dc.SetBrush(mBkgndBrush);
    dc.DrawRectangle(r);
@@ -849,9 +846,9 @@ void Meter::DrawMeterBar(wxDC &dc, MeterBar *meterBar)
       ht = (int)(meterBar->peak * r.height + 0.5);
       r = wxRect(r.x, r.y + r.height - ht,
                  r.width, ht);
-      ht = (int)(meterBar->rms * r.height + 0.5);
-      rRMS = wxRect(r.x, r.y + r.height - ht,
-                    r.width, ht);
+      ht = (int)(meterBar->rms * rRMS.height + 0.5);
+      rRMS = wxRect(rRMS.x, rRMS.y + rRMS.height - ht,
+                    rRMS.width, ht);
 
    }
    else {
@@ -870,9 +867,9 @@ void Meter::DrawMeterBar(wxDC &dc, MeterBar *meterBar)
       wd = (int)(meterBar->peak * r.width + 0.5);
       r = wxRect(r.x, r.y,
                  wd, r.height);
-      wd = (int)(meterBar->rms * r.width + 0.5);
-      rRMS = wxRect(r.x, r.y,
-                    wd, r.height);
+      wd = (int)(meterBar->rms * rRMS.width + 0.5);
+      rRMS = wxRect(rRMS.x, rRMS.y,
+                    wd, rRMS.height);
    }
    dc.SetPen(*wxTRANSPARENT_PEN);
    dc.SetBrush(mBrush);
@@ -913,11 +910,11 @@ void Meter::StartMonitoring()
          OnDisableMeter(dummy);
       }
       
-      gAudioIO->StartMonitoring(44100.0);
       AudacityProject *p = GetActiveProject();
       if (p) {
-         MeterToolBar *bar;
-         bar = p->GetMeterToolBar();
+         gAudioIO->StartMonitoring(p->GetRate());
+
+         MeterToolBar *bar = p->GetMeterToolBar();
          if (bar) {
             Meter *play, *record;
             bar->GetMeters(&play, &record);
@@ -944,8 +941,8 @@ void Meter::OnDisableMeter(wxCommandEvent &evt)
 
       mBkgndBrush = mSavedBkgndBrush;
       mLightPen = mSavedLightPen;
+      Refresh(false);
 
-      SetStyle(mSavedStyle);
       mMeterDisabled = false;
       }
    else
@@ -968,9 +965,6 @@ void Meter::OnDisableMeter(wxCommandEvent &evt)
       mLayoutValid = false;
       Refresh(false);
 
-
-      mSavedStyle = mStyle;
-      SetStyle(Disable);
       mMeterDisabled = true;
       }
    if (mIsInput)
