@@ -6,18 +6,76 @@
 
   Dominic Mazzoni
 
-  This file contains the implementations of Ruler and RulerPanel.
-  See Ruler.h for information about what these classes are for.
+*******************************************************************//**
 
-**********************************************************************/
+\class Ruler
+\brief Used to display a Ruler.
+
+  This is a generic class which can be used to display just about
+  any kind of ruler.  
+
+  At a minimum, the user must specify the dimensions of the
+  ruler, its orientation (horizontal or vertical), and the
+  values displayed at the two ends of the ruler (min and max).
+  By default, this class will display tick marks at reasonable
+  round numbers and fractions, for example, 100, 50, 10, 5, 1,
+  0.5, 0.1, etc.
+
+  The class is designed to display a small handful of
+  labeled Major ticks, and a few Minor ticks between each of
+  these.  Minor ticks are labeled if there is enough space.
+  Labels will never run into each other.
+
+  In addition to Real numbers, the Ruler currently supports
+  two other formats for its display:
+
+  Integer - never shows tick marks for fractions of an integer
+  
+  Time - Assumes values represent seconds, and labels the tick
+         marks in "HH:MM:SS" format, e.g. 4000 seconds becomes
+         "1:06:40", for example.  Will display fractions of
+         a second, and tick marks are all reasonable round
+         numbers for time (i.e. 15 seconds, 30 seconds, etc.)
+*//***************************************************************//**
+
+\class RulerPanel
+\brief RulerPanel class allows you to work with a Ruler like
+  any other wxWindow.
+
+*//***************************************************************//**
+
+\class Ruler::Label
+\brief An array of these created by the Ruler is used to determine 
+what and where text annotations to the numbers on the Ruler get drawn.
+
+\todo Check whether Ruler is costing too much time in malloc/free of
+array of Ruler::Label.
+
+*//******************************************************************/
+
+#include "../Audacity.h"
 
 #include <math.h>
 
 #include <wx/dcscreen.h>
+#include <wx/dcmemory.h>
+#include <wx/dcbuffer.h>
 
 #include "../Internat.h"
+#include "../Project.h"
 #include "Ruler.h"
+#include "../toolbars/ControlToolBar.h"
+#include "../Theme.h"
+#include "../AllThemeResources.h"
 
+#define max(a,b)  ( (a<b)?b:a )
+
+#define SELECT_TOLERANCE_PIXEL 10
+
+#define PLAY_REGION_TRIANGLE_SIZE 6
+#define PLAY_REGION_RECT_WIDTH 1
+#define PLAY_REGION_RECT_HEIGHT 3
+#define PLAY_REGION_GLOBAL_OFFSET_Y 7
 
 //
 // Ruler
@@ -41,6 +99,11 @@ Ruler::Ruler()
    mRight = -1;
    mBottom = -1;
 
+   // Note: the font size is now adjusted automatically whenever
+   // Invalidate is called on a horizontal Ruler, unless the user
+   // calls SetFonts manually.  So the defaults here are not used
+   // often.
+
    int fontSize = 10;
 #ifdef __WXMSW__
    fontSize = 8;
@@ -48,6 +111,7 @@ Ruler::Ruler()
 
    mMinorFont = new wxFont(fontSize, wxSWISS, wxNORMAL, wxNORMAL);
    mMajorFont = new wxFont(fontSize, wxSWISS, wxNORMAL, wxBOLD);
+   mUserFonts = false;
 
    #ifdef __WXMAC__
    mMinorFont->SetNoAntiAliasing(true);
@@ -180,6 +244,9 @@ void Ruler::SetFonts(const wxFont &minorFont, const wxFont &majorFont)
    mMinorFont->SetNoAntiAliasing(true);
    mMajorFont->SetNoAntiAliasing(true);
    #endif
+   
+   // Won't override these fonts
+   mUserFonts = true;
 
    Invalidate();
 }
@@ -286,9 +353,26 @@ void Ruler::FindLinearTickSizes(double UPP)
          mMajor = 6.0;
          return;
       }
-      mMinor = 3.0;
-      mMajor = 12.0;
-      return;
+      if (units < 3.0) {
+         mMinor = 3.0;
+         mMajor = 12.0;
+      }
+      d = 10.0;
+      for(;;) {
+         if (units < d) {
+            mMinor = d;
+            mMajor = d*5.0;
+            return;
+         }
+         d *= 5.0;
+         if (units < d) {
+            mMinor = d;
+            mMajor = d*2.0;
+            return;
+         }
+         d *= 2.0;
+      }
+      break;
 
    case IntFormat:
       d = 1.0;
@@ -386,6 +470,7 @@ void Ruler::FindLinearTickSizes(double UPP)
 
    case RealFormat:
       d = 0.000001;
+      // mDigits is number of digits after the decimal point.
       mDigits = 6;
       for(;;) {
          if (units < d) {
@@ -401,7 +486,12 @@ void Ruler::FindLinearTickSizes(double UPP)
          }
          d *= 2.0;
          mDigits--;
+         // More than 10 digit numbers?  Something is badly wrong.
+         // Probably units is coming in with too high a value.
+         wxASSERT( mDigits >= -10 );
       }
+      mMinor = d;
+      mMajor = d * 2.0;
       break;
 
    }
@@ -518,9 +608,16 @@ wxString Ruler::LabelString(double d, bool major)
 void Ruler::Tick(int pos, double d, bool major)
 {
    wxString l;
-   wxCoord strW, strH;
+   wxCoord strW, strH, strD, strL;
    int strPos, strLen, strLeft, strTop;
 
+   // FIXME: We don't draw a tick if of end of our label arrays
+   // But we shouldn't have an array of labels.
+   if( mNumMinor >= mLength )
+      return;
+   if( mNumMajor >= mLength )
+      return;
+
    Label *label;
    if (major)
       label = &mMajorLabels[mNumMajor++];
@@ -533,9 +630,8 @@ void Ruler::Tick(int pos, double d, bool major)
    label->text = wxT("");
 
    mDC->SetFont(major? *mMajorFont: *mMinorFont);
-
    l = LabelString(d, major);
-   mDC->GetTextExtent(l, &strW, &strH);
+   mDC->GetTextExtent(l, &strW, &strH, &strD, &strL);
 
    if (mOrientation == wxHORIZONTAL) {
       strLen = strW;
@@ -545,10 +641,16 @@ void Ruler::Tick(int pos, double d, bool major)
       if (strPos + strW >= mLength)
          strPos = mLength - strW;
       strLeft = mLeft + strPos;
-      if (mFlip)
+      if (mFlip) {
          strTop = mTop + 4;
-      else
+         mMaxHeight = max(mMaxHeight, strH + 4);
+      }
+      else {
          strTop = mBottom - strH - 6;
+         // Still leaves room for an umlaut or whatever above normal character
+         strTop = mTop- mLead;
+         mMaxHeight = max(mMaxHeight, strH + 6);
+      }
    }
    else {
       strLen = strH;
@@ -558,12 +660,23 @@ void Ruler::Tick(int pos, double d, bool major)
       if (strPos + strH >= mLength)
          strPos = mLength - strH;
       strTop = mTop + strPos;
-      if (mFlip)
+      if (mFlip) {
          strLeft = mLeft + 5;
-      else
+         mMaxWidth = max(mMaxWidth, strW + 5);
+      }
+      else {
          strLeft = mRight - strW - 6;
+         mMaxWidth = max(mMaxWidth, strW + 6);
+      }
    }
 
+
+   // FIXME: we shouldn't even get here if strPos < 0.
+   // Ruler code currently does  not handle very small or
+   // negative sized windows (i.e. don't draw) properly.
+   if( strPos < 0 )
+      return;
+
    // See if any of the pixels we need to draw this
    // label is already covered
 
@@ -580,7 +693,6 @@ void Ruler::Tick(int pos, double d, bool major)
 
    // And mark these pixels, plus some surrounding
    // ones (the spacing between labels), as covered
-
    int leftMargin = mSpacing;
    if (strPos < leftMargin)
       leftMargin = strPos;
@@ -605,11 +717,67 @@ void Ruler::Update( Envelope *speedEnv, long minSpeed, long maxSpeed )
 {
    // This gets called when something has been changed
    // (i.e. we've been invalidated).  Recompute all
-   // tick positions.
+   // tick positions and font size.
 
    int i;
    int j;
 
+   if (!mUserFonts) {
+     int fontSize = 4;
+     wxCoord strW, strH, strD, strL;
+     wxString exampleText = wxT("0.9");   //ignored for height calcs on all platforms
+     int desiredPixelHeight;
+
+     if (mOrientation == wxHORIZONTAL)
+       desiredPixelHeight = mBottom-mTop-5; // height less ticks and 1px gap
+     else
+       desiredPixelHeight = (mRight-mLeft)/2;
+
+     if (desiredPixelHeight < 8)
+       desiredPixelHeight = 8;
+     if (desiredPixelHeight > 12)
+       desiredPixelHeight = 12;
+
+      // Keep making the font bigger until it's too big, then subtract one.
+      mDC->SetFont(wxFont(fontSize, wxSWISS, wxNORMAL, wxBOLD));
+      mDC->GetTextExtent(exampleText, &strW, &strH, &strD, &strL);
+      while( (strH-strD-strL) <= desiredPixelHeight && fontSize < 40) {
+         fontSize++;
+         mDC->SetFont(wxFont(fontSize, wxSWISS, wxNORMAL, wxBOLD));
+         mDC->GetTextExtent(exampleText, &strW, &strH, &strD, & strL);
+      }
+      fontSize--;
+      mDC->SetFont(wxFont(fontSize, wxSWISS, wxNORMAL, wxNORMAL));
+      mDC->GetTextExtent(exampleText, &strW, &strH, &strD, &strL);
+      mLead = strL;
+
+     if (mMinorFont)
+        delete mMinorFont;
+     mMinorFont = new wxFont(fontSize, wxSWISS, wxNORMAL, wxNORMAL);
+
+     if (mMajorFont)
+        delete mMajorFont;
+     mMajorFont = new wxFont(fontSize, wxSWISS, wxNORMAL, wxBOLD);
+   }
+
+   // If ruler is being resized, we could end up with it being too small.
+   // Values of mLength of zero or below cause bad array allocations and 
+   // division by zero.  So...
+   // IF too small THEN bail out and don't draw.
+   if( mLength <= 0 )
+      return;
+
+   if (mOrientation == wxHORIZONTAL) {
+      mMaxWidth = mLength;
+      mMaxHeight = 0;
+   }
+   else {
+      mMaxWidth = 0;
+      mMaxHeight = mLength;
+   }
+
+   // FIXME: Surely we do not need to allocate storage for the labels?
+   // We can just recompute them as we need them?
    mNumMajor = 0;
    mMajorLabels = new Label[mLength+1];
    mNumMinor = 0;
@@ -737,7 +905,7 @@ void Ruler::Update( Envelope *speedEnv, long minSpeed, long maxSpeed )
       for(i=loDecade; i<hiDecade; i++) {
          for(j=2; j<=9; j++) {
             val = decade * j;
-            if(val > mMin && val < mMax) {
+            if(val >= mMin && val < mMax) {
                pos = (int)(((log10(val) - loLog)*scale)+0.5);
                Tick(pos, val, false);
             }
@@ -758,6 +926,9 @@ void Ruler::Draw(wxDC& dc, Envelope *speedEnv, long minSpeed, long maxSpeed)
 {
    mDC = &dc;
 
+   if( mLength <=0 )
+      return;
+
    if (!mValid)
       Update( speedEnv, minSpeed, maxSpeed );
 
@@ -836,7 +1007,23 @@ void Ruler::Draw(wxDC& dc, Envelope *speedEnv, long minSpeed, long maxSpeed)
    }
 }
 
+void Ruler::GetMaxSize(wxCoord *width, wxCoord *height)
+{
 
+   if (!mValid) {
+      wxMemoryDC tmpDC;
+      wxBitmap tmpBM(1, 1);
+      tmpDC.SelectObject(tmpBM);
+      mDC = &tmpDC;
+      Update( NULL, 0, 0 );
+   }
+
+   if (width)
+      *width = mMaxWidth;
+
+   if (height)
+      *height = mMaxHeight;
+}
 
 
 //
@@ -844,8 +1031,9 @@ void Ruler::Draw(wxDC& dc, Envelope *speedEnv, long minSpeed, long maxSpeed)
 //
 
 BEGIN_EVENT_TABLE(RulerPanel, wxPanel)
-    EVT_PAINT(RulerPanel::OnPaint)
-    EVT_SIZE(RulerPanel::OnSize)
+   EVT_ERASE_BACKGROUND(RulerPanel::OnErase)
+   EVT_PAINT(RulerPanel::OnPaint)
+   EVT_SIZE(RulerPanel::OnSize)
 END_EVENT_TABLE()
 
 IMPLEMENT_CLASS(RulerPanel, wxPanel)
@@ -858,17 +1046,27 @@ RulerPanel::RulerPanel(wxWindow* parent, wxWindowID id,
    int width, height;
    GetClientSize(&width, &height);
 
-   ruler.SetBounds(0, 0, width, height);
+   ruler.SetBounds(0, 0, width-1, height-1);
 }
 
 RulerPanel::~RulerPanel()
 {
 }
 
+void RulerPanel::OnErase(wxEraseEvent &evt)
+{
+   // Ignore it to prevent flashing
+}
+
 void RulerPanel::OnPaint(wxPaintEvent &evt)
 {
    wxPaintDC dc(this);
 
+#if defined(__WXGTK__)
+   dc.SetBackground(wxBrush(wxSystemSettings::GetColour(wxSYS_COLOUR_3DFACE)));
+#endif
+
+   dc.Clear();
    ruler.Draw(dc);
 }
 
@@ -890,155 +1088,488 @@ void RulerPanel::OnSize(wxSizeEvent &evt)
   out of the widgets subdirectory into its own source file.
 
 **********************************************************************/
-//#include <math.h>
-//#include <wx/dcscreen.h>
-//#include "Ruler.h"
 
 #include "../ViewInfo.h"
 #include "../AColor.h"
 
-#if 0
-AdornedRulerPanel::AdornedRulerPanel(wxWindow* parent, wxWindowID id,
-              const wxPoint& pos,
-              const wxSize& size ) :
-   RulerPanel( parent, id, pos, size )
+BEGIN_EVENT_TABLE(AdornedRulerPanel, wxPanel)
+   EVT_ERASE_BACKGROUND(AdornedRulerPanel::OnErase)
+   EVT_PAINT(AdornedRulerPanel::OnPaint)
+   EVT_SIZE(AdornedRulerPanel::OnSize)
+   EVT_MOUSE_EVENTS(AdornedRulerPanel::OnMouseEvents)
+END_EVENT_TABLE()
+
+AdornedRulerPanel::AdornedRulerPanel(wxWindow* parent,
+                                     wxWindowID id,
+                                     const wxPoint& pos,
+                                     const wxSize& size,
+                                     ViewInfo *viewinfo):
+   wxPanel( parent, id, pos, size )
 {
-   ruler.SetLabelEdges(false);
-   ruler.SetFormat(Ruler::TimeFormat);
+   SetLabel( _("Vertical Ruler") );
+   SetName( _("Vertical Ruler") );
 
+   mLeftOffset = 0;
+   mCurPos = -1;
+   mIndPos = -1;
+   mIndType = -1;
+   mPlayRegionStart = -1;
+   mPlayRegionEnd = -1;
+   mMouseEventState = mesNone;
+
+   mBuffer = new wxBitmap( 1, 1 );
+   mViewInfo = viewinfo;
+
+   mOuter = GetClientRect();
+
+   mInner = mOuter;
+   mInner.x += 1;          // +1 for left bevel
+   mInner.y += 1;          // +1 for top bevel
+   mInner.width -= 2;      // -2 for left and right bevels
+   mInner.height -= 3;     // -3 for top and bottom bevels and bottom line
+
+   ruler.SetBounds( mInner.GetLeft(),
+                    mInner.GetTop(),
+                    mInner.GetRight(),
+                    mInner.GetBottom() );
+   ruler.SetLabelEdges( false );
+   ruler.SetFormat( Ruler::TimeFormat );
 }
-#endif
-
-AdornedRulerPanel::AdornedRulerPanel()
-{
-   ruler.SetLabelEdges(false);
-   ruler.SetFormat(Ruler::TimeFormat);
-}
-
 
 AdornedRulerPanel::~AdornedRulerPanel()
 {
+   delete mBuffer;
 }
 
-void AdornedRulerPanel::SetSize( const wxRect & r )
+void AdornedRulerPanel::OnErase(wxEraseEvent &evt)
 {
-   mRect = r;
+   // Ignore it to prevent flashing
 }
 
-void AdornedRulerPanel::GetSize( int * width, int * height )
+void AdornedRulerPanel::OnPaint(wxPaintEvent &evt)
 {
-   *width = mRect.width;
-   *height= mRect.height;
+#if defined(__WXMAC__)
+   wxPaintDC dc( this );
+#else
+   wxBufferedPaintDC dc( this );
+#endif
+
+   DoDraw( &dc );
 }
 
-
-void AdornedRulerPanel::DrawAdornedRuler(
-   wxDC * dc, ViewInfo * pViewInfo, bool text, bool indicator, bool bRecording)
+void AdornedRulerPanel::OnSize(wxSizeEvent &evt)
 {
-   wxRect r;
+   mOuter = GetClientRect();
 
-   mViewInfo = pViewInfo;
+   mInner = mOuter;
+   mInner.x += 1;          // +1 for left bevel
+   mInner.y += 1;          // +1 for top bevel
+   mInner.width -= 2;      // -2 for left and right bevels
+   mInner.height -= 3;     // -3 for top and bottom bevels and bottom line
 
-   GetSize(&r.width, &r.height);
-   r.x = 0;
-   r.y = 0;
+   ruler.SetBounds( mInner.GetLeft(),
+                    mInner.GetTop(),
+                    mInner.GetRight(),
+                    mInner.GetBottom() );
 
-   DrawBorder(dc, r);
+   if( mBuffer )
+   {
+      delete mBuffer;
+   }
 
-   if (pViewInfo->sel0 < pViewInfo->sel1)
-      DrawSelection(dc, r);
-
-   if( indicator )
-      DrawIndicator(dc, bRecording);
-
-   DrawMarks(dc, r, text);
+   mBuffer = new wxBitmap( mOuter.GetWidth(), mOuter.GetHeight() );
 
+   Refresh( false );
 }
 
-void AdornedRulerPanel::DrawBorder(wxDC * dc, wxRect & r)
+double AdornedRulerPanel::Pos2Time(int p)
+{
+   return (p-mLeftOffset) / mViewInfo->zoom + mViewInfo->h;
+}
+
+int AdornedRulerPanel::Time2Pos(double t)
+{
+   return mLeftOffset + (int)((t-mViewInfo->h) * mViewInfo->zoom + 0.5);
+}
+
+bool AdornedRulerPanel::IsWithinMarker(int mousePosX, double markerTime)
+{
+   if (markerTime < 0)
+      return false;
+      
+   int pixelPos = Time2Pos(markerTime);
+   int boundLeft = pixelPos - SELECT_TOLERANCE_PIXEL;
+   int boundRight = pixelPos + SELECT_TOLERANCE_PIXEL;
+   
+   return mousePosX >= boundLeft && mousePosX < boundRight;
+}
+
+void AdornedRulerPanel::OnMouseEvents(wxMouseEvent &evt)
+{
+   bool isWithinStart = IsWithinMarker(evt.GetX(), mPlayRegionStart);
+   bool isWithinEnd = IsWithinMarker(evt.GetX(), mPlayRegionEnd);
+   
+   if (isWithinStart || isWithinEnd)
+      SetCursor(wxCursor(wxCURSOR_SIZEWE));
+   else
+      SetCursor(wxCursor(wxCURSOR_HAND));
+      
+   double mouseTime = Pos2Time(evt.GetX());
+   
+   if (evt.LeftDown())
+   {
+      mButtonDownMousePos = evt.GetX();
+
+      if (isWithinStart && isWithinEnd)
+      {
+         // Both could be selected, check which marker is nearer
+         if (fabs(mouseTime-mPlayRegionStart) < fabs(mouseTime-mPlayRegionEnd))
+            mMouseEventState = mesDraggingPlayRegionStart;
+         else
+            mMouseEventState = mesDraggingPlayRegionEnd;
+      } else if (isWithinStart)
+         mMouseEventState = mesDraggingPlayRegionStart;
+      else if (isWithinEnd)
+         mMouseEventState = mesDraggingPlayRegionEnd;
+      else {
+         // First, we enter "click" mode to avoid selecting a small range
+         // accidentially
+         mMouseEventState = mesSelectingPlayRegionClick;
+         mPlayRegionStart = mouseTime;
+         mPlayRegionEnd = mouseTime;
+         Refresh();
+      }
+         
+      CaptureMouse();
+   }
+   
+   switch (mMouseEventState)
+   {
+   case mesNone:
+      // do nothing
+      break;
+   case mesDraggingPlayRegionStart:
+      mPlayRegionStart = mouseTime;
+      Refresh();
+      break;
+   case mesDraggingPlayRegionEnd:
+      mPlayRegionEnd = mouseTime;
+      Refresh();
+      break;
+   case mesSelectingPlayRegionClick:
+      if (abs(evt.GetX() - mButtonDownMousePos) > SELECT_TOLERANCE_PIXEL)
+      {
+         // User moved mouse at least SELECT_TOLERANCE_PIXEL, so change
+         // from "click" mode to "range" mode to allow selecting a range
+         mMouseEventState = mesSelectingPlayRegionRange;
+         mPlayRegionEnd = mouseTime;
+         Refresh();
+      }
+      break;
+   case mesSelectingPlayRegionRange:
+      mPlayRegionEnd = mouseTime;
+      Refresh();
+      break;
+   }
+   
+   if (evt.LeftUp())
+   {
+      if (HasCapture())
+         ReleaseMouse();
+
+      if (mPlayRegionStart < 0 && mPlayRegionEnd >= 0)
+      {
+         // This makes no sense, remove play region
+         mPlayRegionStart = -1;
+         mPlayRegionEnd = -1;
+      }
+      
+      if (mPlayRegionEnd < mPlayRegionStart)
+      {
+         // Swap values to make sure mPlayRegionStart > mPlayRegionEnd
+         double tmp = mPlayRegionStart;
+         mPlayRegionStart = mPlayRegionEnd;
+         mPlayRegionEnd = tmp;
+      }
+      
+      bool startPlaying = mPlayRegionStart >= 0 && 
+         (mMouseEventState == mesSelectingPlayRegionClick ||
+          mMouseEventState == mesSelectingPlayRegionRange);
+          
+      mMouseEventState = mesNone;
+      
+      if (startPlaying)
+      {
+         ControlToolBar* ctb = mProject->GetControlToolBar();
+         ctb->StopPlaying();
+         ctb->PlayDefault();
+      }
+   }
+}
+
+void AdornedRulerPanel::DoDraw(wxDC *dc)
+{
+   DoDrawBorder( dc );
+
+   if( mViewInfo->sel0 < mViewInfo->sel1 )
+   {
+      DoDrawSelection( dc );
+   }
+
+   if( mIndType >= 0 )
+   {
+      DoDrawIndicator( dc );
+   }
+
+   DoDrawMarks( dc, true );
+
+   if( mViewInfo->sel0 == mViewInfo->sel1 )
+   {
+      DoDrawCursor( dc );
+   }
+   
+   DoDrawPlayRegion(dc);
+}
+
+void AdornedRulerPanel::DoDrawPlayRegion(wxDC *dc)
+{
+   double start, end;
+   GetPlayRegion(&start, &end);
+   
+   if (start >= 0)
+   {
+      int x1 = Time2Pos(start) + 1;
+      int x2 = Time2Pos(end);
+      int y = mInner.height/2;
+
+      bool isLocked = mProject->IsPlayRegionLocked();
+      AColor::PlayRegionColor(dc, isLocked);
+   
+      wxPoint tri[3];
+      wxRect r;
+
+      tri[0].x = x1;
+      tri[0].y = y + PLAY_REGION_GLOBAL_OFFSET_Y;
+      tri[1].x = x1 + PLAY_REGION_TRIANGLE_SIZE;
+      tri[1].y = y - PLAY_REGION_TRIANGLE_SIZE + PLAY_REGION_GLOBAL_OFFSET_Y;
+      tri[2].x = x1 + PLAY_REGION_TRIANGLE_SIZE;
+      tri[2].y = y + PLAY_REGION_TRIANGLE_SIZE + PLAY_REGION_GLOBAL_OFFSET_Y;
+      dc->DrawPolygon(3, tri);
+
+      r.x = x1;
+      r.y = y - PLAY_REGION_TRIANGLE_SIZE + PLAY_REGION_GLOBAL_OFFSET_Y;
+      r.width = PLAY_REGION_RECT_WIDTH;
+      r.height = PLAY_REGION_TRIANGLE_SIZE*2 + 1;
+      dc->DrawRectangle(r);
+   
+      if (end != start)
+      {
+         tri[0].x = x2;
+         tri[0].y = y + PLAY_REGION_GLOBAL_OFFSET_Y;
+         tri[1].x = x2 - PLAY_REGION_TRIANGLE_SIZE;
+         tri[1].y = y - PLAY_REGION_TRIANGLE_SIZE + PLAY_REGION_GLOBAL_OFFSET_Y;
+         tri[2].x = x2 - PLAY_REGION_TRIANGLE_SIZE;
+         tri[2].y = y + PLAY_REGION_TRIANGLE_SIZE + PLAY_REGION_GLOBAL_OFFSET_Y;
+         dc->DrawPolygon(3, tri);
+
+         r.x = x2 - PLAY_REGION_RECT_WIDTH + 1;
+         r.y = y - PLAY_REGION_TRIANGLE_SIZE + PLAY_REGION_GLOBAL_OFFSET_Y;
+         r.width = PLAY_REGION_RECT_WIDTH;
+         r.height = PLAY_REGION_TRIANGLE_SIZE*2 + 1;
+         dc->DrawRectangle(r);
+   
+         r.x = x1 + PLAY_REGION_TRIANGLE_SIZE;
+         r.y = y - PLAY_REGION_RECT_HEIGHT/2 + PLAY_REGION_GLOBAL_OFFSET_Y;
+         r.width = x2-x1 - PLAY_REGION_TRIANGLE_SIZE*2;
+         r.height = PLAY_REGION_RECT_HEIGHT;
+         dc->DrawRectangle(r);
+      }
+   }
+}
+
+void AdornedRulerPanel::DoDrawBorder(wxDC * dc)
 {
    // Draw AdornedRulerPanel border
-   AColor::Medium(dc, false);
-   dc->DrawRectangle(r);
+   AColor::Medium( dc, false );
+   dc->DrawRectangle( mInner );
 
-   r.width--;
-   r.height--;
-   AColor::Bevel(*dc, true, r);
+   wxRect r = mOuter;
+   r.width -= 1;                 // -1 for bevel
+   r.height -= 2;                // -2 for bevel and for bottom line
+   AColor::Bevel( *dc, true, r );
 
-   dc->SetPen(*wxBLACK_PEN);
-   dc->DrawLine(r.x, r.y + r.height + 1, r.x + r.width + 1,
-                r.y + r.height + 1);
+   dc->SetPen( *wxBLACK_PEN );
+   dc->DrawLine( mOuter.x,
+                 mOuter.y + mOuter.height - 1,
+                 mOuter.x + mOuter.width,
+                 mOuter.y + mOuter.height - 1 );
 }
 
-void AdornedRulerPanel::DrawSelection(wxDC * dc, const wxRect r)
+void AdornedRulerPanel::DoDrawMarks(wxDC * dc, bool /*text */ )
+{
+   double min = mViewInfo->h - mLeftOffset / mViewInfo->zoom;
+   double max = min + mInner.width / mViewInfo->zoom;
+
+   ruler.SetRange( min, max );
+   ruler.Draw( *dc );
+}
+
+void AdornedRulerPanel::DrawSelection()
+{
+#if defined(__WXMAC__)
+   wxClientDC dc( this );
+#else
+   wxClientDC cdc( this );
+   wxBufferedDC dc( &cdc, *mBuffer );
+#endif
+
+   DoDraw( &dc );
+}
+
+void AdornedRulerPanel::DoDrawSelection(wxDC * dc)
 {
    // Draw selection
-   double sel0 = mViewInfo->sel0 - mViewInfo->h +
-       GetLeftOffset() / mViewInfo->zoom;
-   double sel1 = mViewInfo->sel1 - mViewInfo->h +
-       GetLeftOffset() / mViewInfo->zoom;
+   double zoom = mViewInfo->zoom;
+   double sel0 = mViewInfo->sel0 - mViewInfo->h + mLeftOffset / zoom;
+   double sel1 = mViewInfo->sel1 - mViewInfo->h + mLeftOffset / zoom;
 
-   if (sel0 < 0.0)
+   if( sel0 < 0.0 )
       sel0 = 0.0;
-   if (sel1 > (r.width / mViewInfo->zoom))
-      sel1 = r.width / mViewInfo->zoom;
 
-   int p0 = int (sel0 * mViewInfo->zoom + 0.5);
-   int p1 = int (sel1 * mViewInfo->zoom + 0.5);
+   if( sel1 > ( mInner.width / zoom ) )
+      sel1 = mInner.width / zoom;
 
-   wxBrush selectedBrush;
-   selectedBrush.SetColour(148, 148, 170);
-   wxPen selectedPen;
-   selectedPen.SetColour(148, 148, 170);
-   dc->SetBrush(selectedBrush);
-   dc->SetPen(selectedPen);
+   int p0 = int ( sel0 * zoom + 1.5 );
+   int p1 = int ( sel1 * zoom + 2.5 );
 
-   wxRect sr;
-   sr.x = p0;
-   sr.y = 1;
-   sr.width = p1 - p0 - 1;
-   sr.height = GetRulerHeight() - 3;
-   dc->DrawRectangle(sr);
+   dc->SetBrush( wxBrush( theTheme.Colour( clrRulerBackground )) );
+   dc->SetPen(   wxPen(   theTheme.Colour( clrRulerBackground )) );
+
+   wxRect r;
+   r.x = p0;
+   r.y = 1;
+   r.width = p1 - p0 - 1;
+   r.height = mInner.height;
+   dc->DrawRectangle( r );
 }
 
-void AdornedRulerPanel::DrawMarks(wxDC * dc, const wxRect r, bool /*text */ )
+void AdornedRulerPanel::DrawCursor(double pos)
 {
-   ruler.SetBounds(r.x, r.y, r.x + r.width - 1, r.y + r.height - 1);
-   double min = mViewInfo->h - GetLeftOffset() / mViewInfo->zoom;
-   double max = min + r.width / mViewInfo->zoom;
-   ruler.SetRange(min, max);
+   mCurPos = pos;
 
-   ruler.Draw(*dc);
+#if defined(__WXMAC__)
+   wxClientDC dc( this );
+#else
+   wxClientDC cdc( this );
+   wxBufferedDC dc( &cdc, *mBuffer );
+#endif
+
+   DoDraw( &dc );
+}
+
+void AdornedRulerPanel::DoDrawCursor(wxDC *dc)
+{
+   int x = mLeftOffset + int ( ( mCurPos - mViewInfo->h ) * mViewInfo->zoom );
+
+   // Draw cursor in ruler
+   dc->DrawLine( x, 1, x, mInner.height );
 }
 
 //
 //This draws the little triangular indicator on the 
 //AdornedRulerPanel.
 //
-void AdornedRulerPanel::DrawIndicator(wxDC * dc, bool bRecording)
+void AdornedRulerPanel::ClearIndicator()
 {
-   // Draw indicator
-   double ind = indicatorPos; 
+   mIndType = -1;
 
-   if (ind >= mViewInfo->h && ind <= (mViewInfo->h + mViewInfo->screen)) {
-      int indp =
-          GetLeftOffset() + int ((ind - mViewInfo->h) * mViewInfo->zoom);
+#if defined(__WXMAC__)
+   wxClientDC dc( this );
+#else
+   wxClientDC cdc( this );
+   wxBufferedDC dc( &cdc, *mBuffer );
+#endif
 
-      AColor::IndicatorColor(dc, bRecording );
-//      dc->SetPen(*wxTRANSPARENT_PEN);
-//      dc->SetBrush(*wxBLACK_BRUSH);
+   DoDraw( &dc );
+}
 
-      int indsize = 6;
+void AdornedRulerPanel::DrawIndicator( double pos, bool rec )
+{
+   mIndPos = pos;
+   if( mIndPos < 0 )
+   {
+      ClearIndicator();
+      return;
+   }
 
-      wxPoint tri[3];
-      tri[0].x = indp;
-      tri[0].y = (indsize * 3)/2 + 1;
-      tri[1].x = indp - indsize;
-      tri[1].y = 1;
-      tri[2].x = indp + indsize;
-      tri[2].y = 1;
+   mIndType = ( rec ? 1 : 0 );
 
-      dc->DrawPolygon(3, tri);
+#if defined(__WXMAC__)
+   wxClientDC dc( this );
+#else
+   wxClientDC cdc( this );
+   wxBufferedDC dc( &cdc, *mBuffer );
+#endif
+
+   DoDraw( &dc );
+}
+
+void AdornedRulerPanel::DoDrawIndicator(wxDC * dc)
+{
+   if( mIndType < 0 )
+   {
+      return;
+   }
+
+   int indsize = 6;
+   int x = mLeftOffset + int ( ( mIndPos - mViewInfo->h ) * mViewInfo->zoom );
+
+   wxPoint tri[ 3 ];
+   tri[ 0 ].x = x - indsize;
+   tri[ 0 ].y = 1;
+   tri[ 1 ].x = x + indsize;
+   tri[ 1 ].y = 1;
+   tri[ 2 ].x = x;
+   tri[ 2 ].y = ( indsize * 3 ) / 2 + 1;
+
+   AColor::IndicatorColor( dc, ( mIndType ? true : false ) );
+   dc->DrawPolygon( 3, tri );
+}
+
+void AdornedRulerPanel::SetPlayRegion(double playRegionStart,
+                                      double playRegionEnd)
+{
+   // This is called by AudacityProject to make the play region follow
+   // the current selection. But while the user is selecting a play region
+   // with the mouse directly in the ruler, changes from outside are blocked.
+   if (mMouseEventState != mesNone)
+      return;
+      
+   mPlayRegionStart = playRegionStart;
+   mPlayRegionEnd = playRegionEnd;
+   Refresh();
+}
+
+void AdornedRulerPanel::ClearPlayRegion()
+{
+   mPlayRegionStart = -1;
+   mPlayRegionEnd = -1;
+   Refresh();
+}
+
+void AdornedRulerPanel::GetPlayRegion(double* playRegionStart,
+                                      double* playRegionEnd)
+{
+   if (mPlayRegionStart >= 0 && mPlayRegionEnd >= 0 &&
+       mPlayRegionEnd < mPlayRegionStart)
+   {
+      // swap values to make sure end > start
+      *playRegionStart = mPlayRegionEnd;
+      *playRegionEnd = mPlayRegionStart;
+   } else
+   {
+      *playRegionStart = mPlayRegionStart;
+      *playRegionEnd = mPlayRegionEnd;
    }
 }
 
